@@ -17,7 +17,6 @@
 #endif
 
 #include "swoole.h"
-#include "coroutine.h"
 #include "error.h"
 
 #if __linux__
@@ -28,6 +27,8 @@
 #include <valgrind/valgrind.h>
 #endif
 
+typedef void (*coroutine_func_t)(void*);
+
 namespace swoole
 {
 //namespace start
