@@ -22,17 +22,21 @@
 #include <string>
 #include <unordered_map>
 
-#define DEFAULT_MAX_CORO_NUM 3000
-#define DEFAULT_STACK_SIZE   8192
-#define MAX_CORO_NUM_LIMIT   0x80000
+#define SW_DEFAULT_MAX_CORO_NUM              3000
+#define SW_DEFAULT_STACK_SIZE                8192
+#define SW_DEFAULT_SOCKET_CONNECT_TIMEOUT    1
+#define SW_DEFAULT_SOCKET_TIMEOUT            -1
 
-#define CORO_END 0
-#define CORO_LIMIT -1
-#define CORO_INVALID -2
+#define CORO_END         0
+#define CORO_LIMIT      -1
+#define CORO_INVALID    -2
 
 typedef enum
 {
-    SW_CORO_INIT = 0, SW_CORO_WAITING, SW_CORO_RUNNING, SW_CORO_END,
+    SW_CORO_INIT = 0,
+    SW_CORO_WAITING,
+    SW_CORO_RUNNING,
+    SW_CORO_END,
 } sw_coro_state;
 
 typedef void (*coro_php_create_t)();
@@ -83,12 +87,40 @@ public:
         return task;
     }
 
-    static long create(coroutine_func_t fn, void* args);
+    static long create(coroutine_func_t fn, void* args = nullptr);
     static int sleep(double sec);
     static swString* read_file(const char *file, int lock);
     static ssize_t write_file(const char *file, char *buf, size_t length, int lock, int flags);
 };
+
+struct CoroutineG
+{
+    int stack_size;
+    int call_stack_size;
+    long last_cid;
+    Coroutine* call_stack[SW_MAX_CORO_NESTING_LEVEL];
+    coro_php_yield_t onYield; /* before php yield coro */
+    coro_php_resume_t onResume; /* before php resume coro */
+    coro_php_close_t onClose; /* before php close coro */
+    std::unordered_map<long, Coroutine*> coroutines;
+
+    CoroutineG()
+    {
+        stack_size = SW_DEFAULT_C_STACK_SIZE;
+        call_stack_size = 0;
+        last_cid = 1;
+        onYield = nullptr;
+        onResume = nullptr;
+        onClose = nullptr;
+    }
+
+    inline size_t count()
+    {
+        return coroutines.size();
+    }
+};
 }
+
 /* co task */
 void* coroutine_get_current_task();
 void* coroutine_get_task_by_cid(long cid);
@@ -103,6 +135,7 @@ void coroutine_set_onYield(coro_php_yield_t func);
 void coroutine_set_onResume(coro_php_resume_t func);
 void coroutine_set_onResumeBack(coro_php_resume_t func);
 void coroutine_set_onClose(coro_php_close_t func);
+void coroutine_print_list();
 
 inline static long coroutine_get_cid(swoole::Coroutine* co)
 {
@@ -112,5 +145,4 @@ inline static long coroutine_get_cid(swoole::Coroutine* co)
 void internal_coro_yield(void *arg);
 void internal_coro_resume(void *arg);
 
-std::unordered_map<long, swoole::Coroutine*>* coroutine_get_map();
-
+extern swoole::CoroutineG swCoroG;
