@@ -27,23 +27,23 @@ extern "C"
 
 enum swHttpVersion
 {
-    HTTP_VERSION_10 = 1,
-    HTTP_VERSION_11,
+    SW_HTTP_VERSION_10 = 1,
+    SW_HTTP_VERSION_11,
 };
 
 enum swHttpMethod
 {
-    HTTP_DELETE = 1, HTTP_GET, HTTP_HEAD, HTTP_POST, HTTP_PUT, HTTP_PATCH,
+    SW_HTTP_DELETE = 1, SW_HTTP_GET, SW_HTTP_HEAD, SW_HTTP_POST, SW_HTTP_PUT, SW_HTTP_PATCH,
     /* pathological */
-    HTTP_CONNECT, HTTP_OPTIONS, HTTP_TRACE,
+    SW_HTTP_CONNECT, SW_HTTP_OPTIONS, SW_HTTP_TRACE,
     /* webdav */
-    HTTP_COPY, HTTP_LOCK, HTTP_MKCOL, HTTP_MOVE, HTTP_PROPFIND, HTTP_PROPPATCH, HTTP_UNLOCK,
+    SW_HTTP_COPY, SW_HTTP_LOCK, SW_HTTP_MKCOL, SW_HTTP_MOVE, SW_HTTP_PROPFIND, SW_HTTP_PROPPATCH, SW_HTTP_UNLOCK,
     /* subversion */
-    HTTP_REPORT, HTTP_MKACTIVITY, HTTP_CHECKOUT, HTTP_MERGE,
+    SW_HTTP_REPORT, SW_HTTP_MKACTIVITY, SW_HTTP_CHECKOUT, SW_HTTP_MERGE,
     /* upnp */
-    HTTP_MSEARCH, HTTP_NOTIFY, HTTP_SUBSCRIBE, HTTP_UNSUBSCRIBE,
+    SW_HTTP_MSEARCH, SW_HTTP_NOTIFY, SW_HTTP_SUBSCRIBE, SW_HTTP_UNSUBSCRIBE,
     /* Http2 */
-    HTTP_PRI,
+    SW_HTTP_PRI,
 };
 
 enum swHttpStatusCode
