@@ -81,13 +81,18 @@ int clock_gettime(clock_id_t which_clock, struct timespec *t);
 #endif
 #endif
 
-#ifndef HAVE_DAEMON
-int daemon(int nochdir, int noclose);
+#if __APPLE__
+#define  daemon(nochdir, noclose)    swoole_daemon(nochdir, noclose)
 #endif
 
 /*----------------------------------------------------------------------------*/
 
-#define SWOOLE_VERSION "4.2.9"
+#define SWOOLE_MAJOR_VERSION      4
+#define SWOOLE_MINOR_VERSION      2
+#define SWOOLE_RELEASE_VERSION    10
+#define SWOOLE_EXTRA_VERSION      ""
+#define SWOOLE_VERSION            "4.2.10"
+#define SWOOLE_VERSION_ID         40210
 #define SWOOLE_BUG_REPORT \
     "A bug occurred in Swoole-v" SWOOLE_VERSION ", please report it.\n"\
     "The Swoole developers probably don't know about it,\n"\
@@ -187,10 +192,21 @@ typedef unsigned long ulong_t;
 #define MIN(A, B)              ((A) < (B) ? (A) : (B))
 #endif
 
+#ifdef SW_DEBUG
+#define SW_ASSERT(e)           assert(e)
+#else
+#define SW_ASSERT(e)
+#endif
 #define SW_STRS(s)             s, sizeof(s)
 #define SW_STRL(s)             s, sizeof(s)-1
 #define SW_START_SLEEP         usleep(100000)  //sleep 1s,wait fork and pthread_create
 
+#ifdef SW_USE_EMALLOC
+#define sw_malloc              emalloc
+#define sw_free                efree
+#define sw_calloc              ecalloc
+#define sw_realloc             erealloc
+#else
 #ifdef SW_USE_JEMALLOC
 #include <jemalloc/jemalloc.h>
 #define sw_malloc              je_malloc
@@ -203,6 +219,10 @@ typedef unsigned long ulong_t;
 #define sw_calloc              calloc
 #define sw_realloc             realloc
 #endif
+#endif
+
+#define SW_MEM_ALIGNED_SIZE(size)               SW_MM_ALIGNED_SIZE_EX(size, 8)
+#define SW_MEM_ALIGNED_SIZE_EX(size, alignment) (((size) + ((alignment) - 1LL)) & ~((alignment) - 1LL))
 
 static sw_inline char* swoole_strdup(const char *s)
 {
@@ -295,6 +315,7 @@ enum swGlobal_hook_type
 {
     SW_GLOBAL_HOOK_BEFORE_SERVER_START,
     SW_GLOBAL_HOOK_BEFORE_CLIENT_START,
+    SW_GLOBAL_HOOK_BEFORE_WORKER_START,
     SW_GLOBAL_HOOK_ON_CORO_START,
     SW_GLOBAL_HOOK_ON_CORO_STOP,
 };
@@ -360,10 +381,10 @@ enum swWorker_status
     SwooleGS->lock_2.unlock(&SwooleGS->lock_2);}
 
 #define swError(str,...)       SwooleGS->lock_2.lock(&SwooleGS->lock_2);\
-snprintf(sw_error, SW_ERROR_MSG_SIZE, str, ##__VA_ARGS__);\
-swLog_put(SW_LOG_ERROR, sw_error);\
-SwooleGS->lock_2.unlock(&SwooleGS->lock_2);\
-exit(1)
+    snprintf(sw_error, SW_ERROR_MSG_SIZE, str, ##__VA_ARGS__);\
+    swLog_put(SW_LOG_ERROR, sw_error);\
+    SwooleGS->lock_2.unlock(&SwooleGS->lock_2);\
+    exit(1)
 
 #define swSysError(str,...) SwooleGS->lock_2.lock(&SwooleGS->lock_2);\
     snprintf(sw_error,SW_ERROR_MSG_SIZE,"%s(:%d): " str " Error: %s[%d].",__func__,__LINE__,##__VA_ARGS__,strerror(errno),errno);\
@@ -378,10 +399,7 @@ exit(1)
     swLog_put(level, sw_error);\
     SwooleGS->lock_2.unlock(&SwooleGS->lock_2);}}while(0)
 
-#ifdef SW_DEBUG_REMOTE_OPEN
-#define swDebug(str,...) int __debug_log_n = snprintf(sw_error, SW_ERROR_MSG_SIZE, str, ##__VA_ARGS__);\
-write(SwooleG.debug_fd, sw_error, __debug_log_n);
-#elif defined(SW_DEBUG)
+#ifdef SW_DEBUG
 #define swDebug(str,...) if (SW_LOG_DEBUG >= SwooleG.log_level){\
     SwooleGS->lock_2.lock(&SwooleGS->lock_2);\
     snprintf(sw_error, SW_ERROR_MSG_SIZE, "%s(:%d): " str, __func__, __LINE__, ##__VA_ARGS__);\
@@ -414,6 +432,8 @@ enum swTraceType
     SW_TRACE_SSL              = 1u << 19,
     SW_TRACE_NORMAL           = 1u << 20,
     SW_TRACE_CHANNEL          = 1u << 21,
+    SW_TRACE_TIMER            = 1u << 22,
+    SW_TRACE_SOCKET           = 1u << 23,
 };
 
 #ifdef SW_LOG_TRACE_OPEN
@@ -678,7 +698,7 @@ typedef ssize_t (*swProtocol_length_function)(struct _swProtocol *, swConnection
 uint32_t swoole_utf8_decode(u_char **p, size_t n);
 size_t swoole_utf8_length(u_char *p, size_t n);
 void swoole_random_string(char *buf, size_t size);
-char* swoole_get_mime_type(char *file);
+const char* swoole_get_mime_type(const char *file);
 
 static sw_inline char *swoole_strlchr(char *p, char *last, char c)
 {
@@ -698,6 +718,22 @@ static sw_inline size_t swoole_size_align(size_t size, int pagesize)
     return size + (pagesize - (size % pagesize));
 }
 
+#define SW_STRINGL(s)      s->str, s->length
+#define SW_STRINGS(s)      s->str, s->size
+#define SW_STRINGCVL(s)    s->str + s->offset, s->length - s->offset
+
+swString *swString_new(size_t size);
+swString *swString_dup(const char *src_str, int length);
+swString *swString_dup2(swString *src);
+
+void swString_print(swString *str);
+int swString_append(swString *str, swString *append_str);
+int swString_append_ptr(swString *str, const char *append_str, size_t length);
+int swString_write(swString *str, off_t offset, swString *write_str);
+int swString_write_ptr(swString *str, off_t offset, char *write_str, size_t length);
+int swString_extend(swString *str, size_t new_size);
+char* swString_alloc(swString *str, size_t __size);
+
 static sw_inline void swString_clear(swString *str)
 {
     str->length = 0;
@@ -710,31 +746,6 @@ static sw_inline void swString_free(swString *str)
     sw_free(str);
 }
 
-static sw_inline size_t swString_length(swString *str)
-{
-    return str->length;
-}
-
-static sw_inline size_t swString_size(swString *str)
-{
-    return str->size;
-}
-
-swString *swString_new(size_t size);
-swString *swString_dup(const char *src_str, int length);
-swString *swString_dup2(swString *src);
-
-void swString_print(swString *str);
-void swString_free(swString *str);
-int swString_append(swString *str, swString *append_str);
-int swString_append_ptr(swString *str, const char *append_str, int length);
-int swString_write(swString *str, off_t offset, swString *write_str);
-int swString_write_ptr(swString *str, off_t offset, char *write_str, int length);
-int swString_extend(swString *str, size_t new_size);
-char* swString_alloc(swString *str, size_t __size);
-
-#define SWSTRING_CURRENT_VL(buffer) buffer->str + buffer->offset, buffer->length - buffer->offset
-
 static sw_inline int swString_extend_align(swString *str, size_t _new_size)
 {
     size_t align_size = str->size * 2;
@@ -745,8 +756,6 @@ static sw_inline int swString_extend_align(swString *str, size_t _new_size)
     return swString_extend(str, align_size);
 }
 
-#define swString_length(s) (s->length)
-#define swString_ptr(s) (s->str)
 //------------------------------Base--------------------------------
 typedef struct _swDataHead
 {
@@ -808,15 +817,6 @@ typedef struct
     char filename[0];
 } swSendFile_request;
 
-//------------------TimeWheel--------------------
-typedef struct
-{
-    uint16_t current;
-    uint16_t size;
-    swHashMap **wheel;
-
-} swTimeWheel;
-
 typedef void * (*swThreadStartFunc)(void *);
 typedef int (*swHandle)(swEventData *buf);
 typedef void (*swSignalHandler)(int);
@@ -879,6 +879,7 @@ typedef struct _swMsgQueue
 
 int swMsgQueue_create(swMsgQueue *q, int blocking, key_t msg_key, int perms);
 void swMsgQueue_set_blocking(swMsgQueue *q, uint8_t blocking);
+int swMsgQueue_set_capacity(swMsgQueue *q, int queue_bytes);
 int swMsgQueue_push(swMsgQueue *q, swQueue_data *in, int data_length);
 int swMsgQueue_pop(swMsgQueue *q, swQueue_data *out, int buffer_length);
 int swMsgQueue_stat(swMsgQueue *q, int *queue_num, int *queue_bytes);
@@ -1303,7 +1304,6 @@ int swoole_tmpfile(char *filename);
 swString* swoole_file_get_contents(char *filename);
 int swoole_file_put_contents(char *filename, char *content, size_t length);
 long swoole_file_size(char *filename);
-void swoole_open_remote_debug(void);
 char *swoole_dec2hex(int value, int base);
 int swoole_version_compare(const char *version1, const char *version2);
 #ifdef HAVE_EXECINFO
@@ -1342,7 +1342,8 @@ double swoole_microtime(void);
 void swoole_rtrim(char *str, int len);
 void swoole_redirect_stdout(int new_fd);
 #ifndef _WIN32
-int swoole_shell_exec(char *command, pid_t *pid, uint8_t get_error_stream);
+int swoole_shell_exec(const char *command, pid_t *pid, uint8_t get_error_stream);
+int swoole_daemon(int nochdir, int noclose);
 #endif
 SW_API int swoole_add_function(const char *name, void* func);
 SW_API void* swoole_get_function(char *name, uint32_t length);
@@ -1443,6 +1444,7 @@ void swSignal_callback(int signo);
 swSignalHandler swSignal_get_handler(int signo);
 void swSignal_clear(void);
 void swSignal_none(void);
+char* swSignal_str(int sig);
 
 #ifdef HAVE_SIGNALFD
 void swSignalfd_init();
@@ -1511,9 +1513,9 @@ struct _swReactor
      */
     swArray *socket_array;
 
-    swReactor_handle handle[SW_MAX_FDTYPE];        //默认事件
-    swReactor_handle write_handle[SW_MAX_FDTYPE];  //扩展事件1(一般为写事件)
-    swReactor_handle error_handle[SW_MAX_FDTYPE];  //扩展事件2(一般为错误事件,如socket关闭)
+    swReactor_handle handle[SW_MAX_FDTYPE];        // default event
+    swReactor_handle write_handle[SW_MAX_FDTYPE];  // ext event 1 (maybe writable event)
+    swReactor_handle error_handle[SW_MAX_FDTYPE];  // ext event 2 (error event, maybe socket closed)
 
     int (*add)(swReactor *, int fd, int fdtype);
     int (*set)(swReactor *, int fd, int fdtype);
@@ -1755,6 +1757,13 @@ static sw_inline int swReactor_events(int fdtype)
 
 int swReactor_create(swReactor *reactor, int max_event);
 int swReactor_setHandle(swReactor *, int, swReactor_handle);
+
+static inline void swReactor_before_wait(swReactor *reactor)
+{
+    reactor->running = 1;
+    reactor->start = 1;
+}
+
 int swReactor_empty(swReactor *reactor);
 
 void swReactor_defer_task_create(swReactor *reactor);
@@ -2017,7 +2026,7 @@ struct _swTimer_node
     void *data;
     swTimerCallback callback;
     int64_t exec_msec;
-    uint32_t interval;
+    int64_t interval;
     uint64_t round;
     long id;
     int type;                 //0 normal node 1 node for client_coro
@@ -2039,7 +2048,7 @@ struct _swTimer
     uint8_t initialized;
     swHeap *heap;
     swHashMap *map;
-    int num;
+    uint32_t num;
     int lasttime;
     uint64_t round;
     long _next_id;
@@ -2051,7 +2060,7 @@ struct _swTimer
     int (*set)(swTimer *timer, long exec_msec);
 };
 
-swTimer_node* swTimer_add(swTimer *timer, int _msec, int interval, void *data, swTimerCallback callback);
+swTimer_node* swTimer_add(swTimer *timer, long _msec, int interval, void *data, swTimerCallback callback);
 int swTimer_del(swTimer *timer, swTimer_node *node);
 void swTimer_free(swTimer *timer);
 int swTimer_select(swTimer *timer);
@@ -2065,14 +2074,6 @@ static sw_inline swTimer_node* swTimer_get(swTimer *timer, long id)
 int swSystemTimer_init(int msec);
 void swSystemTimer_signal_handler(int sig);
 int swSystemTimer_event_handler(swReactor *reactor, swEvent *event);
-
-swTimeWheel* swTimeWheel_new(uint16_t size);
-void swTimeWheel_free(swTimeWheel *tw);
-void swTimeWheel_forward(swTimeWheel *tw, swReactor *reactor);
-void swTimeWheel_add(swTimeWheel *tw, swConnection *conn);
-void swTimeWheel_update(swTimeWheel *tw, swConnection *conn);
-void swTimeWheel_remove(swTimeWheel *tw, swConnection *conn);
-#define swTimeWheel_new_index(tw)   (tw->current == 0 ? tw->size - 1 : tw->current - 1)
 //--------------------------------------------------------------
 //Share Memory
 typedef struct
@@ -2159,7 +2160,6 @@ typedef struct
     int signal_fd;
     int log_fd;
     int null_fd;
-    int debug_fd;
 
     /**
      * worker(worker and task_worker) process chroot / user / group
@@ -2168,9 +2168,9 @@ typedef struct
     char *user;
     char *group;
 
-    uint8_t log_level;
+    uint32_t log_level;
     char *log_file;
-    int trace_flags;
+    uint32_t trace_flags;
 
     uint16_t cpu_num;
 
@@ -2187,10 +2187,10 @@ typedef struct
     uint32_t socket_buffer_size;
 
     swServer *serv;
-    swFactory *factory;
 
     swMemoryPool *memory_pool;
     swReactor *main_reactor;
+    swReactor *origin_main_reactor;
 
     char *task_tmpdir;
     uint16_t task_tmpdir_len;
