@@ -42,7 +42,7 @@ struct aio_task
 
 int swoole_coroutine_socket(int domain, int type, int protocol)
 {
-    if (SwooleG.main_reactor == nullptr || coroutine_get_current_cid() == -1)
+    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
     {
         return socket(domain, type, protocol);
     }
@@ -58,7 +58,7 @@ int swoole_coroutine_socket(int domain, int type, int protocol)
 
 ssize_t swoole_coroutine_send(int sockfd, const void *buf, size_t len, int flags)
 {
-    if (SwooleG.main_reactor == nullptr || coroutine_get_current_cid() == -1)
+    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
     {
         _no_coro: return ::send(sockfd, buf, len, flags);
     }
@@ -71,19 +71,17 @@ ssize_t swoole_coroutine_send(int sockfd, const void *buf, size_t len, int flags
     ssize_t retval = socket->send(buf, len);
     if (retval < 0)
     {
-        errno = socket->errCode;
         return -1;
     }
     else
     {
-        errno = 0;
         return retval;
     }
 }
 
 ssize_t swoole_coroutine_sendmsg(int sockfd, const struct msghdr *msg, int flags)
 {
-    if (SwooleG.main_reactor == nullptr || coroutine_get_current_cid() == -1)
+    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
     {
         _no_coro: return ::sendmsg(sockfd, msg, flags);
     }
@@ -96,19 +94,17 @@ ssize_t swoole_coroutine_sendmsg(int sockfd, const struct msghdr *msg, int flags
     ssize_t retval = socket->sendmsg(msg, flags);
     if (retval < 0)
     {
-        errno = socket->errCode;
         return -1;
     }
     else
     {
-        errno = 0;
         return retval;
     }
 }
 
 ssize_t swoole_coroutine_recvmsg(int sockfd, struct msghdr *msg, int flags)
 {
-    if (SwooleG.main_reactor == nullptr || coroutine_get_current_cid() == -1)
+    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
     {
         _no_coro: return ::recvmsg(sockfd, msg, flags);
     }
@@ -121,19 +117,17 @@ ssize_t swoole_coroutine_recvmsg(int sockfd, struct msghdr *msg, int flags)
     ssize_t retval = socket->recvmsg(msg, flags);
     if (retval < 0)
     {
-        errno = socket->errCode;
         return -1;
     }
     else
     {
-        errno = 0;
         return retval;
     }
 }
 
 ssize_t swoole_coroutine_recv(int sockfd, void *buf, size_t len, int flags)
 {
-    if (SwooleG.main_reactor == nullptr || coroutine_get_current_cid() == -1)
+    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
     {
         _no_coro: return ::recv(sockfd, buf, len, flags);
     }
@@ -143,22 +137,28 @@ ssize_t swoole_coroutine_recv(int sockfd, void *buf, size_t len, int flags)
         goto _no_coro;
     }
     Socket *socket = (Socket *) conn->object;
-    ssize_t retval = socket->recv(buf, len);
+    ssize_t retval;
+    if (flags & MSG_PEEK)
+    {
+        retval = socket->peek(buf, len);
+    }
+    else
+    {
+        retval = socket->recv(buf, len);
+    }
     if (retval < 0)
     {
-        errno = socket->errCode;
         return -1;
     }
     else
     {
-        errno = 0;
         return retval;
     }
 }
 
 int swoole_coroutine_close(int fd)
 {
-    if (SwooleG.main_reactor == nullptr || coroutine_get_current_cid() == -1)
+    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
     {
         _no_coro: return close(fd);
     }
@@ -176,7 +176,7 @@ int swoole_coroutine_close(int fd)
 
 int swoole_coroutine_connect(int sockfd, const struct sockaddr *addr, socklen_t addrlen)
 {
-    if (SwooleG.main_reactor == nullptr || coroutine_get_current_cid() == -1)
+    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
     {
         _no_coro: return connect(sockfd, addr, addrlen);
     }
@@ -188,19 +188,17 @@ int swoole_coroutine_connect(int sockfd, const struct sockaddr *addr, socklen_t
     Socket *socket = (Socket *) conn->object;
     if (socket->connect(addr, addrlen) == false)
     {
-        errno = socket->errCode;
         return -1;
     }
     else
     {
-        errno = 0;
         return 0;
     }
 }
 
 int swoole_coroutine_poll(struct pollfd *fds, nfds_t nfds, int timeout)
 {
-    if (SwooleG.main_reactor == nullptr || coroutine_get_current_cid() == -1 || nfds != 1)
+    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current() || nfds != 1))
     {
         _poll: return poll(fds, nfds, timeout);
     }
@@ -321,7 +319,7 @@ static void aio_onWriteFileCompleted(swAio_event *event)
 
 int swoole_coroutine_open(const char *pathname, int flags, mode_t mode)
 {
-    if (SwooleG.main_reactor == nullptr || coroutine_get_current_cid() == -1)
+    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
     {
         return open(pathname, flags, mode);
     }
@@ -347,7 +345,7 @@ int swoole_coroutine_open(const char *pathname, int flags, mode_t mode)
 
 ssize_t swoole_coroutine_read(int fd, void *buf, size_t count)
 {
-    if (SwooleG.main_reactor == nullptr || coroutine_get_current_cid() == -1)
+    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
     {
         return read(fd, buf, count);
     }
@@ -380,7 +378,7 @@ ssize_t swoole_coroutine_read(int fd, void *buf, size_t count)
 
 ssize_t swoole_coroutine_write(int fd, const void *buf, size_t count)
 {
-    if (SwooleG.main_reactor == nullptr || coroutine_get_current_cid() == -1)
+    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
     {
         return write(fd, buf, count);
     }
@@ -413,7 +411,7 @@ ssize_t swoole_coroutine_write(int fd, const void *buf, size_t count)
 
 off_t swoole_coroutine_lseek(int fd, off_t offset, int whence)
 {
-    if (SwooleG.main_reactor == nullptr || coroutine_get_current_cid() == -1)
+    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
     {
         return lseek(fd, offset, whence);
     }
@@ -439,7 +437,7 @@ off_t swoole_coroutine_lseek(int fd, off_t offset, int whence)
 
 int swoole_coroutine_fstat(int fd, struct stat *statbuf)
 {
-    if (SwooleG.main_reactor == nullptr || coroutine_get_current_cid() == -1)
+    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
     {
         return fstat(fd, statbuf);
     }
@@ -464,7 +462,7 @@ int swoole_coroutine_fstat(int fd, struct stat *statbuf)
 
 int swoole_coroutine_unlink(const char *pathname)
 {
-    if (SwooleG.main_reactor == nullptr || coroutine_get_current_cid() == -1)
+    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
     {
         return unlink(pathname);
     }
@@ -488,7 +486,7 @@ int swoole_coroutine_unlink(const char *pathname)
 
 int swoole_coroutine_statvfs(const char *path, struct statvfs *buf)
 {
-    if (SwooleG.main_reactor == nullptr || coroutine_get_current_cid() == -1)
+    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
     {
         return statvfs(path, buf);
     }
@@ -513,7 +511,7 @@ int swoole_coroutine_statvfs(const char *path, struct statvfs *buf)
 
 int swoole_coroutine_mkdir(const char *pathname, mode_t mode)
 {
-    if (SwooleG.main_reactor == nullptr || coroutine_get_current_cid() == -1)
+    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
     {
         return mkdir(pathname, mode);
     }
@@ -538,7 +536,7 @@ int swoole_coroutine_mkdir(const char *pathname, mode_t mode)
 
 int swoole_coroutine_rmdir(const char *pathname)
 {
-    if (SwooleG.main_reactor == nullptr || coroutine_get_current_cid() == -1)
+    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
     {
         return rmdir(pathname);
     }
@@ -562,7 +560,7 @@ int swoole_coroutine_rmdir(const char *pathname)
 
 int swoole_coroutine_rename(const char *oldpath, const char *newpath)
 {
-    if (SwooleG.main_reactor == nullptr || coroutine_get_current_cid() == -1)
+    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
     {
         return rename(oldpath, newpath);
     }
@@ -587,7 +585,7 @@ int swoole_coroutine_rename(const char *oldpath, const char *newpath)
 
 int swoole_coroutine_access(const char *pathname, int mode)
 {
-    if (SwooleG.main_reactor == nullptr || coroutine_get_current_cid() == -1)
+    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
     {
         return access(pathname, mode);
     }
@@ -612,7 +610,7 @@ int swoole_coroutine_access(const char *pathname, int mode)
 
 int swoole_coroutine_flock(int fd, int operation)
 {
-    if (SwooleG.main_reactor == nullptr || coroutine_get_current_cid() == -1)
+    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
     {
         return flock(fd, operation);
     }
@@ -738,7 +736,7 @@ static void handler_readdir(swAio_event *event)
 
 DIR *swoole_coroutine_opendir(const char *name)
 {
-    if (SwooleG.main_reactor == nullptr || coroutine_get_current_cid() == -1)
+    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
     {
         return opendir(name);
     }
@@ -762,7 +760,7 @@ DIR *swoole_coroutine_opendir(const char *name)
 
 struct dirent *swoole_coroutine_readdir(DIR *dirp)
 {
-    if (SwooleG.main_reactor == nullptr || coroutine_get_current_cid() == -1)
+    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
     {
         return readdir(dirp);
     }
