@@ -69,21 +69,19 @@ int swManager_start(swFactory *factory)
         {
             return SW_ERR;
         }
-
-        swProcessPool *pool = &serv->gs->task_workers;
-        swTaskWorker_init(pool);
+        swTaskWorker_init(serv);
 
         swWorker *worker;
         for (i = 0; i < serv->task_worker_num; i++)
         {
-            worker = &pool->workers[i];
-            if (swWorker_create(worker) < 0)
+            worker = &serv->gs->task_workers.workers[i];
+            if (swServer_worker_create(serv, worker) < 0)
             {
                 return SW_ERR;
             }
             if (serv->task_ipc_mode == SW_TASK_IPC_UNIXSOCK)
             {
-                swServer_store_pipe_fd(SwooleG.serv, worker->pipe_object);
+                swServer_store_pipe_fd(serv, worker->pipe_object);
             }
         }
     }
@@ -102,7 +100,7 @@ int swManager_start(swFactory *factory)
         LL_FOREACH(serv->user_worker_list, user_worker)
         {
             memcpy(&serv->user_workers[i], user_worker->worker, sizeof(swWorker));
-            if (swWorker_create(&serv->user_workers[i]) < 0)
+            if (swServer_worker_create(serv, &serv->user_workers[i]) < 0)
             {
                 return SW_ERR;
             }
@@ -140,7 +138,6 @@ int swManager_start(swFactory *factory)
          */
         for (i = 0; i < serv->worker_num; i++)
         {
-            //close(worker_pipes[i].pipes[0]);
             pid = swManager_spawn_worker(factory, i);
             if (pid < 0)
             {
@@ -311,7 +308,7 @@ static int swManager_loop(swFactory *factory)
             //reload task & event workers
             else if (ManagerProcess.reload_all_worker == 1)
             {
-                swNotice("Server is reloading now.");
+                swNotice("Server is reloading all workers now.");
                 if (reload_init == 0)
                 {
                     reload_init = 1;
@@ -352,7 +349,7 @@ static int swManager_loop(swFactory *factory)
                     swWarn("cannot reload task workers, task workers is not started.");
                     continue;
                 }
-                swNotice("Server is reloading now.");
+                swNotice("Server is reloading task workers now.");
                 if (reload_init == 0)
                 {
                     memcpy(reload_workers, serv->gs->task_workers.workers, sizeof(swWorker) * serv->task_worker_num);
@@ -561,7 +558,7 @@ static void swManager_signal_handle(int sig)
 
 int swManager_wait_other_worker(swProcessPool *pool, pid_t pid, int status)
 {
-    swServer *serv = SwooleG.serv;
+    swServer *serv = (swServer *) pool->ptr;
     swWorker *exit_worker;
 
     if (serv->gs->task_workers.map)
