@@ -18,6 +18,7 @@
 
 static int swReactorProcess_loop(swProcessPool *pool, swWorker *worker);
 static int swReactorProcess_onPipeRead(swReactor *reactor, swEvent *event);
+static int swReactorProcess_onClose(swReactor *reactor, swEvent *event);
 static int swReactorProcess_send2client(swFactory *, swSendData *);
 static int swReactorProcess_send2worker(int, void *, int);
 static void swReactorProcess_onTimeout(swTimer *timer, swTimer_node *tnode);
@@ -98,31 +99,37 @@ int swReactorProcess_start(swServer *serv)
         return SW_ERR;
     }
 
+    /**
+     * store to swProcessPool object
+     */
     serv->gs->event_workers.ptr = serv;
+    serv->gs->event_workers.worker_num = serv->worker_num;
+    serv->gs->event_workers.use_msgqueue = 0;
     serv->gs->event_workers.main_loop = swReactorProcess_loop;
-    serv->gs->event_workers.type = SW_PROCESS_WORKER;
+    serv->gs->event_workers.onWorkerNotFound = swManager_wait_other_worker;
+
+    int i;
+    for (i = 0; i < serv->worker_num; i++)
+    {
+        serv->gs->event_workers.workers[i].pool = &serv->gs->event_workers;
+        serv->gs->event_workers.workers[i].id = i;
+        serv->gs->event_workers.workers[i].type = SW_PROCESS_WORKER;
+    }
 
     //no worker
     if (serv->worker_num == 1 && serv->task_worker_num == 0 && serv->max_request == 0 && serv->user_worker_list == NULL)
     {
-        swWorker single_worker;
-        bzero(&single_worker, sizeof(single_worker));
-        return swReactorProcess_loop(&serv->gs->event_workers, &single_worker);
+        return swReactorProcess_loop(&serv->gs->event_workers, &serv->gs->event_workers.workers[0]);
     }
 
-    swWorker *worker;
-    int i;
     for (i = 0; i < serv->worker_num; i++)
     {
-        worker = &serv->gs->event_workers.workers[i];
-        if (swWorker_create(worker) < 0)
+        if (swServer_worker_create(serv, &serv->gs->event_workers.workers[i]) < 0)
         {
             return SW_ERR;
         }
     }
 
-    serv->gs->event_workers.onWorkerNotFound = swManager_wait_other_worker;
-
     //task workers
     if (serv->task_worker_num > 0)
     {
@@ -130,7 +137,7 @@ int swReactorProcess_start(swServer *serv)
         {
             return SW_ERR;
         }
-        swTaskWorker_init(&serv->gs->task_workers);
+        swTaskWorker_init(serv);
         if (swProcessPool_start(&serv->gs->task_workers) < 0)
         {
             return SW_ERR;
@@ -232,7 +239,6 @@ static int swReactorProcess_onPipeRead(swReactor *reactor, swEvent *event)
 static int swReactorProcess_loop(swProcessPool *pool, swWorker *worker)
 {
     swServer *serv = pool->ptr;
-    swReactor *reactor = &(serv->reactor_threads[0].reactor);
 
     SwooleG.process_type = SW_PROCESS_WORKER;
     SwooleG.pid = getpid();
@@ -273,9 +279,18 @@ static int swReactorProcess_loop(swProcessPool *pool, swWorker *worker)
     }
 
     //create reactor
-    if (swReactor_create(reactor, SW_REACTOR_MAXEVENTS) < 0)
+    swReactor *reactor;
+    if (!SwooleG.main_reactor)
     {
-        return SW_ERR;
+        reactor = &(serv->reactor_threads[0].reactor);
+        if (swReactor_create(reactor, SW_REACTOR_MAXEVENTS) < 0)
+        {
+            return SW_ERR;
+        }
+    }
+    else
+    {
+        reactor = SwooleG.main_reactor;
     }
 
     swListenPort *ls;
@@ -301,14 +316,6 @@ static int swReactorProcess_loop(swProcessPool *pool, swWorker *worker)
     reactor->id = worker->id;
     reactor->ptr = serv;
 
-#ifdef SW_USE_RINGBUFFER
-    serv->reactor_threads[0].buffer_input = swMalloc_new();
-    if (serv->reactor_threads[0].buffer_input == NULL)
-    {
-        return SW_ERR;
-    }
-#endif
-
 #ifdef HAVE_SIGNALFD
     if (SwooleG.use_signalfd)
     {
@@ -366,47 +373,36 @@ static int swReactorProcess_loop(swProcessPool *pool, swWorker *worker)
     //set protocol function point
     swReactorThread_set_protocol(serv, reactor);
 
+    swTimer_node *heartbeat_timer = NULL, *update_timer;
+
     /**
      * 1 second timer, update serv->gs->now
      */
-    if (swTimer_add(&SwooleG.timer, 1000, 1, serv, swServer_master_onTimer) == NULL)
+    if ((update_timer = swTimer_add(&SwooleG.timer, 1000, 1, serv, swServer_master_onTimer)) == NULL)
     {
         return SW_ERR;
     }
 
-    /**
-     * call internal serv hooks
-     */
-    if (SwooleG.serv->hooks[SW_SERVER_HOOK_WORKER_START])
-    {
-        void *hook_args[2];
-        hook_args[0] = serv;
-        hook_args[1] = (void *)(uintptr_t)SwooleWG.id;
-        swServer_call_hook(serv, SW_SERVER_HOOK_WORKER_START, hook_args);
-    }
-
-    if (serv->onWorkerStart)
-    {
-        serv->onWorkerStart(serv, worker->id);
-    }
+    swServer_worker_start(serv, worker);
 
     /**
      * for heartbeat check
      */
     if (serv->heartbeat_check_interval > 0)
     {
-        if (swTimer_add(&SwooleG.timer, serv->heartbeat_check_interval * 1000, 1, reactor, swReactorProcess_onTimeout) == NULL)
+        heartbeat_timer = swTimer_add(&SwooleG.timer, serv->heartbeat_check_interval * 1000, 1, reactor, swReactorProcess_onTimeout);
+        if (heartbeat_timer == NULL)
         {
             return SW_ERR;
         }
     }
 
-    reactor->wait(reactor, NULL);
+    int retval = reactor->wait(reactor, NULL);
 
     /**
      * call internal serv hooks
      */
-    if (SwooleG.serv->hooks[SW_SERVER_HOOK_WORKER_CLOSE])
+    if (serv->hooks[SW_SERVER_HOOK_WORKER_CLOSE])
     {
         void *hook_args[2];
         hook_args[0] = serv;
@@ -414,19 +410,29 @@ static int swReactorProcess_loop(swProcessPool *pool, swWorker *worker)
         swServer_call_hook(serv, SW_SERVER_HOOK_WORKER_CLOSE, hook_args);
     }
 
+    if (heartbeat_timer)
+    {
+        swTimer_del(&SwooleG.timer, heartbeat_timer);
+    }
+
+    if (update_timer)
+    {
+        swTimer_del(&SwooleG.timer, update_timer);
+    }
+
     if (serv->onWorkerStop)
     {
         serv->onWorkerStop(serv, worker->id);
     }
 
-    return SW_OK;
+    return retval;
 }
 
-int swReactorProcess_onClose(swReactor *reactor, swEvent *event)
+static int swReactorProcess_onClose(swReactor *reactor, swEvent *event)
 {
     int fd = event->fd;
     swServer *serv = reactor->ptr;
-    swConnection *conn = swServer_connection_get(SwooleG.serv, fd);
+    swConnection *conn = swServer_connection_get(serv, fd);
     if (conn == NULL || conn->active == 0)
     {
         return SW_ERR;
@@ -463,7 +469,7 @@ static int swReactorProcess_send2worker(int pipe_fd, void *data, int length)
 
 static int swReactorProcess_send2client(swFactory *factory, swSendData *_send)
 {
-    swServer *serv = SwooleG.serv;
+    swServer *serv = (swServer *) factory->ptr;
     int session_id = _send->info.fd;
     if (_send->length == 0)
     {
