@@ -273,6 +273,7 @@ ZEND_END_ARG_INFO()
 #ifdef SW_COROUTINE
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_async_dns_lookup_coro, 0, 0, 1)
     ZEND_ARG_INFO(0, domain_name)
+    ZEND_ARG_INFO(0, timeout)
 ZEND_END_ARG_INFO()
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_coroutine_create, 0, 0, 1)
@@ -500,23 +501,29 @@ static const zend_function_entry swoole_async_methods[] =
 php_vmstat_t php_vmstat;
 #endif
 
-zend_class_entry swoole_server_ce;
-zend_class_entry *swoole_server_class_entry_ptr;
+static zend_class_entry swoole_server_ce;
+zend_class_entry *swoole_server_ce_ptr;
+zend_object_handlers swoole_server_handlers;
 
-zend_class_entry swoole_connection_iterator_ce;
-zend_class_entry *swoole_connection_iterator_class_entry_ptr;
+static zend_class_entry swoole_connection_iterator_ce;
+zend_class_entry *swoole_connection_iterator_ce_ptr;
+static zend_object_handlers swoole_connection_iterator_handlers;
 
 static zend_class_entry swoole_timer_ce;
-static zend_class_entry *swoole_timer_class_entry_ptr;
+static zend_class_entry *swoole_timer_ce_ptr;
+static zend_object_handlers swoole_timer_handlers;
 
 static zend_class_entry swoole_event_ce;
-static zend_class_entry *swoole_event_class_entry_ptr;
+static zend_class_entry *swoole_event_ce_ptr;
+static zend_object_handlers swoole_event_handlers;
 
 static zend_class_entry swoole_async_ce;
-static zend_class_entry *swoole_async_class_entry_ptr;
+static zend_class_entry *swoole_async_ce_ptr;
+static zend_object_handlers swoole_async_handlers;
 
-zend_class_entry swoole_exception_ce;
-zend_class_entry *swoole_exception_class_entry_ptr;
+static zend_class_entry swoole_exception_ce;
+zend_class_entry *swoole_exception_ce_ptr;
+zend_object_handlers swoole_exception_handlers;
 
 zend_module_entry swoole_module_entry =
 {
@@ -558,10 +565,6 @@ STD_PHP_INI_ENTRY("swoole.aio_thread_num", "2", PHP_INI_ALL, OnUpdateLong, aio_t
  * display error
  */
 STD_PHP_INI_ENTRY("swoole.display_errors", "On", PHP_INI_ALL, OnUpdateBool, display_errors, zend_swoole_globals, swoole_globals)
-/**
- * namespace class style
- */
-STD_PHP_INI_ENTRY("swoole.use_namespace", "On", PHP_INI_SYSTEM, OnUpdateBool, use_namespace, zend_swoole_globals, swoole_globals)
 /**
  * use an short class name
  */
@@ -582,12 +585,21 @@ static void php_swoole_init_globals(zend_swoole_globals *swoole_globals)
     swoole_globals->aio_thread_num = SW_AIO_THREAD_NUM_DEFAULT;
     swoole_globals->socket_buffer_size = SW_SOCKET_BUFFER_SIZE;
     swoole_globals->display_errors = 1;
-    swoole_globals->use_namespace = 1;
     swoole_globals->use_shortname = 1;
     swoole_globals->fast_serialize = 0;
     swoole_globals->rshutdown_functions = NULL;
 }
 
+void php_swoole_class_unset_property_deny(zval *zobject, zval *zmember, void **cache_slot)
+{
+    if (EXPECTED(zend_hash_find(&(Z_OBJCE_P(zobject)->properties_info), Z_STR_P(zmember))))
+    {
+        zend_throw_error(NULL, "Property %s of class %s cannot be unset", Z_STRVAL_P(zmember), ZSTR_VAL(Z_OBJCE_P(zobject)->name));
+        return;
+    }
+    std_object_handlers.unset_property(zobject, zmember, cache_slot);
+}
+
 ssize_t php_swoole_length_func(swProtocol *protocol, swConnection *conn, char *data, uint32_t length)
 {
     SwooleG.lock.lock(&SwooleG.lock);
@@ -608,13 +620,13 @@ ssize_t php_swoole_length_func(swProtocol *protocol, swConnection *conn, char *d
         swoole_php_fatal_error(E_WARNING, "length function handler error.");
         goto error;
     }
-    if (EG(exception))
+    if (UNEXPECTED(EG(exception)))
     {
         zend_exception_error(EG(exception), E_ERROR);
         goto error;
     }
     zval_ptr_dtor(zdata);
-    if (retval != NULL)
+    if (retval)
     {
         convert_to_long(retval);
         int length = Z_LVAL_P(retval);
@@ -659,7 +671,7 @@ int php_swoole_dispatch_func(swServer *serv, swConnection *conn, swEventData *da
         swoole_php_fatal_error(E_WARNING, "dispatch function handler error.");
         goto error;
     }
-    if (EG(exception))
+    if (UNEXPECTED(EG(exception)))
     {
         zend_exception_error(EG(exception), E_ERROR);
         goto error;
@@ -667,7 +679,7 @@ int php_swoole_dispatch_func(swServer *serv, swConnection *conn, swEventData *da
     zval_ptr_dtor(zfd);
     zval_ptr_dtor(ztype);
     zval_ptr_dtor(zdata);
-    if (retval != NULL)
+    if (retval)
     {
         convert_to_long(retval);
         int worker_id = (int) Z_LVAL_P(retval);
@@ -704,9 +716,8 @@ static sw_inline uint32_t swoole_get_new_size(uint32_t old_size, int handle)
     return new_size;
 }
 
-void swoole_set_object(zval *zobject, void *ptr)
+void swoole_set_object_by_handle(uint32_t handle, void *ptr)
 {
-    int handle = Z_OBJ_HANDLE_P(zobject);
     assert(handle < SWOOLE_OBJECT_MAX);
 
     if (unlikely(handle >= swoole_objects.size))
@@ -717,7 +728,7 @@ void swoole_set_object(zval *zobject, void *ptr)
         void *old_ptr = swoole_objects.array;
         void *new_ptr = NULL;
 
-        new_ptr = realloc(old_ptr, sizeof(void*) * new_size);
+        new_ptr = sw_realloc(old_ptr, sizeof(void*) * new_size);
         if (!new_ptr)
         {
             swoole_php_fatal_error(E_ERROR, "malloc(%d) failed.", (int )(new_size * sizeof(void *)));
@@ -736,9 +747,8 @@ void swoole_set_object(zval *zobject, void *ptr)
     swoole_objects.array[handle] = ptr;
 }
 
-void swoole_set_property(zval *zobject, int property_id, void *ptr)
+void swoole_set_property_by_handle(uint32_t handle, int property_id, void *ptr)
 {
-    int handle = Z_OBJ_HANDLE_P(zobject);
     assert(handle < SWOOLE_OBJECT_MAX);
 
     if (unlikely(handle >= swoole_objects.property_size[property_id]))
@@ -752,13 +762,13 @@ void swoole_set_property(zval *zobject, int property_id, void *ptr)
         if (old_size == 0)
         {
             new_size = 65536;
-            new_ptr = calloc(new_size, sizeof(void *));
+            new_ptr = sw_calloc(new_size, sizeof(void *));
         }
         else
         {
             new_size = swoole_get_new_size(old_size, handle);
             old_ptr = swoole_objects.property[property_id];
-            new_ptr = realloc(old_ptr, new_size * sizeof(void *));
+            new_ptr = sw_realloc(old_ptr, new_size * sizeof(void *));
         }
         if (new_ptr == NULL)
         {
@@ -801,6 +811,37 @@ int swoole_register_rshutdown_function(swCallback func, int push_back)
     }
 }
 
+void php_swoole_register_shutdown_function(char *function)
+{
+    php_shutdown_function_entry shutdown_function_entry;
+    shutdown_function_entry.arg_count = 1;
+    shutdown_function_entry.arguments = (zval *) safe_emalloc(sizeof(zval), 1, 0);
+    ZVAL_STRING(&shutdown_function_entry.arguments[0], function);
+    register_user_shutdown_function(function, ZSTR_LEN(Z_STR(shutdown_function_entry.arguments[0])), &shutdown_function_entry);
+}
+
+static void php_swoole_old_shutdown_function_move(zval *zv)
+{
+    php_shutdown_function_entry *old_shutdown_function_entry = Z_PTR_P(zv);
+    zend_hash_next_index_insert_mem(BG(user_shutdown_function_names), old_shutdown_function_entry, sizeof(php_shutdown_function_entry));
+    efree(old_shutdown_function_entry);
+}
+
+void php_swoole_register_shutdown_function_prepend(char *function)
+{
+    HashTable *old_user_shutdown_function_names = BG(user_shutdown_function_names);
+    if (!old_user_shutdown_function_names)
+    {
+        php_swoole_register_shutdown_function(function);
+        return;
+    }
+    BG(user_shutdown_function_names) = NULL;
+    php_swoole_register_shutdown_function(function);
+    old_user_shutdown_function_names->pDestructor = php_swoole_old_shutdown_function_move;
+    zend_hash_destroy(old_user_shutdown_function_names);
+    FREE_HASHTABLE(old_user_shutdown_function_names);
+}
+
 void swoole_call_rshutdown_function(void *arg)
 {
     if (SWOOLE_G(rshutdown_functions))
@@ -818,12 +859,7 @@ void swoole_call_rshutdown_function(void *arg)
     }
 }
 
-#ifdef ZTS
-__thread swoole_object_array swoole_objects;
-void ***sw_thread_ctx;
-#else
 swoole_object_array swoole_objects;
-#endif
 
 /* {{{ PHP_MINIT_FUNCTION
  */
@@ -1007,6 +1043,9 @@ PHP_MINIT_FUNCTION(swoole)
     SWOOLE_DEFINE(TRACE_AIO);
     SWOOLE_DEFINE(TRACE_SSL);
     SWOOLE_DEFINE(TRACE_NORMAL);
+    SWOOLE_DEFINE(TRACE_CHANNEL);
+    SWOOLE_DEFINE(TRACE_TIMER);
+    SWOOLE_DEFINE(TRACE_SOCKET);
     REGISTER_LONG_CONSTANT("SWOOLE_TRACE_ALL", 0xffffffff, CONST_CS | CONST_PERSISTENT);
 
     /**
@@ -1023,76 +1062,73 @@ PHP_MINIT_FUNCTION(swoole)
     SWOOLE_DEFINE(IPC_UNIXSOCK);
     SWOOLE_DEFINE(IPC_SOCKET);
 
-    SWOOLE_INIT_CLASS_ENTRY(swoole_server_ce, "swoole_server", "Swoole\\Server", swoole_server_methods);
-    swoole_server_class_entry_ptr = zend_register_internal_class(&swoole_server_ce);
-    swoole_server_class_entry_ptr->serialize = zend_class_serialize_deny;
-    swoole_server_class_entry_ptr->unserialize = zend_class_unserialize_deny;
-    SWOOLE_CLASS_ALIAS(swoole_server, "Swoole\\Server");
+    SWOOLE_INIT_CLASS_ENTRY(swoole_server, "Swoole\\Server", "swoole_server", NULL, swoole_server_methods);
+    SWOOLE_SET_CLASS_SERIALIZABLE(swoole_server, zend_class_serialize_deny, zend_class_unserialize_deny);
+    SWOOLE_SET_CLASS_CLONEABLE(swoole_server, zend_class_clone_deny);
+    SWOOLE_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_server, zend_class_unset_property_deny);
 
     if (!SWOOLE_G(use_shortname))
     {
         zend_hash_str_del(CG(function_table), ZEND_STRL("go"));
         zend_hash_str_del(CG(function_table), ZEND_STRL("defer"));
     }
-    else
-    {
-        sw_zend_register_class_alias("Co\\Server", swoole_server_class_entry_ptr);
-    }
 
-    zend_declare_property_null(swoole_server_class_entry_ptr, ZEND_STRL("onConnect"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_class_entry_ptr, ZEND_STRL("onReceive"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_class_entry_ptr, ZEND_STRL("onClose"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_class_entry_ptr, ZEND_STRL("onPacket"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_class_entry_ptr, ZEND_STRL("onBufferFull"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_class_entry_ptr, ZEND_STRL("onBufferEmpty"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_class_entry_ptr, ZEND_STRL("onStart"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_class_entry_ptr, ZEND_STRL("onShutdown"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_class_entry_ptr, ZEND_STRL("onWorkerStart"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_class_entry_ptr, ZEND_STRL("onWorkerStop"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_class_entry_ptr, ZEND_STRL("onWorkerExit"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_class_entry_ptr, ZEND_STRL("onWorkerError"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_class_entry_ptr, ZEND_STRL("onTask"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_class_entry_ptr, ZEND_STRL("onFinish"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_class_entry_ptr, ZEND_STRL("onManagerStart"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_class_entry_ptr, ZEND_STRL("onManagerStop"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_class_entry_ptr, ZEND_STRL("onPipeMessage"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_server_ce_ptr, ZEND_STRL("onConnect"), ZEND_ACC_PRIVATE);
+    zend_declare_property_null(swoole_server_ce_ptr, ZEND_STRL("onReceive"), ZEND_ACC_PRIVATE);
+    zend_declare_property_null(swoole_server_ce_ptr, ZEND_STRL("onClose"), ZEND_ACC_PRIVATE);
+    zend_declare_property_null(swoole_server_ce_ptr, ZEND_STRL("onPacket"), ZEND_ACC_PRIVATE);
+    zend_declare_property_null(swoole_server_ce_ptr, ZEND_STRL("onBufferFull"), ZEND_ACC_PRIVATE);
+    zend_declare_property_null(swoole_server_ce_ptr, ZEND_STRL("onBufferEmpty"), ZEND_ACC_PRIVATE);
+    zend_declare_property_null(swoole_server_ce_ptr, ZEND_STRL("onStart"), ZEND_ACC_PRIVATE);
+    zend_declare_property_null(swoole_server_ce_ptr, ZEND_STRL("onShutdown"), ZEND_ACC_PRIVATE);
+    zend_declare_property_null(swoole_server_ce_ptr, ZEND_STRL("onWorkerStart"), ZEND_ACC_PRIVATE);
+    zend_declare_property_null(swoole_server_ce_ptr, ZEND_STRL("onWorkerStop"), ZEND_ACC_PRIVATE);
+    zend_declare_property_null(swoole_server_ce_ptr, ZEND_STRL("onWorkerExit"), ZEND_ACC_PRIVATE);
+    zend_declare_property_null(swoole_server_ce_ptr, ZEND_STRL("onWorkerError"), ZEND_ACC_PRIVATE);
+    zend_declare_property_null(swoole_server_ce_ptr, ZEND_STRL("onTask"), ZEND_ACC_PRIVATE);
+    zend_declare_property_null(swoole_server_ce_ptr, ZEND_STRL("onFinish"), ZEND_ACC_PRIVATE);
+    zend_declare_property_null(swoole_server_ce_ptr, ZEND_STRL("onManagerStart"), ZEND_ACC_PRIVATE);
+    zend_declare_property_null(swoole_server_ce_ptr, ZEND_STRL("onManagerStop"), ZEND_ACC_PRIVATE);
+    zend_declare_property_null(swoole_server_ce_ptr, ZEND_STRL("onPipeMessage"), ZEND_ACC_PRIVATE);
 
-    zend_declare_property_null(swoole_server_class_entry_ptr, ZEND_STRL("setting"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_class_entry_ptr, ZEND_STRL("connections"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_class_entry_ptr, ZEND_STRL("host"), ZEND_ACC_PUBLIC);
-    zend_declare_property_long(swoole_server_class_entry_ptr, ZEND_STRL("port"), 0, ZEND_ACC_PUBLIC);
-    zend_declare_property_long(swoole_server_class_entry_ptr, ZEND_STRL("type"), 0, ZEND_ACC_PUBLIC);
-    zend_declare_property_long(swoole_server_class_entry_ptr, ZEND_STRL("mode"), 0, ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_server_class_entry_ptr, ZEND_STRL("ports"), ZEND_ACC_PUBLIC);
-    zend_declare_property_long(swoole_server_class_entry_ptr, ZEND_STRL("master_pid"), 0, ZEND_ACC_PUBLIC);
-    zend_declare_property_long(swoole_server_class_entry_ptr, ZEND_STRL("manager_pid"), 0, ZEND_ACC_PUBLIC);
-    zend_declare_property_long(swoole_server_class_entry_ptr, ZEND_STRL("worker_id"), -1, ZEND_ACC_PUBLIC);
-    zend_declare_property_bool(swoole_server_class_entry_ptr, ZEND_STRL("taskworker"), 0, ZEND_ACC_PUBLIC);
-    zend_declare_property_long(swoole_server_class_entry_ptr, ZEND_STRL("worker_pid"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_server_ce_ptr, ZEND_STRL("setting"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_server_ce_ptr, ZEND_STRL("connections"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_server_ce_ptr, ZEND_STRL("host"), ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_server_ce_ptr, ZEND_STRL("port"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_server_ce_ptr, ZEND_STRL("type"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_server_ce_ptr, ZEND_STRL("mode"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_server_ce_ptr, ZEND_STRL("ports"), ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_server_ce_ptr, ZEND_STRL("master_pid"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_server_ce_ptr, ZEND_STRL("manager_pid"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_server_ce_ptr, ZEND_STRL("worker_id"), -1, ZEND_ACC_PUBLIC);
+    zend_declare_property_bool(swoole_server_ce_ptr, ZEND_STRL("taskworker"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_server_ce_ptr, ZEND_STRL("worker_pid"), 0, ZEND_ACC_PUBLIC);
 
-    SWOOLE_INIT_CLASS_ENTRY(swoole_timer_ce, "swoole_timer", "Swoole\\Timer", swoole_timer_methods);
-    swoole_timer_class_entry_ptr = zend_register_internal_class(&swoole_timer_ce);
-    SWOOLE_CLASS_ALIAS(swoole_timer, "Swoole\\Timer");
+    SWOOLE_INIT_CLASS_ENTRY(swoole_timer, "Swoole\\Timer", "swoole_timer", NULL, swoole_timer_methods);
+    SWOOLE_SET_CLASS_SERIALIZABLE(swoole_timer, zend_class_serialize_deny, zend_class_unserialize_deny);
+    SWOOLE_SET_CLASS_CLONEABLE(swoole_timer, zend_class_clone_deny);
+    SWOOLE_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_timer, zend_class_unset_property_deny);
 
-    SWOOLE_INIT_CLASS_ENTRY(swoole_event_ce, "swoole_event", "Swoole\\Event", swoole_event_methods);
-    swoole_event_class_entry_ptr = zend_register_internal_class(&swoole_event_ce);
-    SWOOLE_CLASS_ALIAS(swoole_event, "Swoole\\Event");
+    SWOOLE_INIT_CLASS_ENTRY(swoole_event, "Swoole\\Event", "swoole_event", NULL, swoole_event_methods);
+    SWOOLE_SET_CLASS_SERIALIZABLE(swoole_event, zend_class_serialize_deny, zend_class_unserialize_deny);
+    SWOOLE_SET_CLASS_CLONEABLE(swoole_event, zend_class_clone_deny);
+    SWOOLE_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_event, zend_class_unset_property_deny);
 
-    SWOOLE_INIT_CLASS_ENTRY(swoole_async_ce, "swoole_async", "Swoole\\Async", swoole_async_methods);
-    swoole_async_class_entry_ptr = zend_register_internal_class(&swoole_async_ce);
-    SWOOLE_CLASS_ALIAS(swoole_async, "Swoole\\Async");
+    SWOOLE_INIT_CLASS_ENTRY(swoole_async, "Swoole\\Async", "swoole_async", NULL, swoole_async_methods);
+    SWOOLE_SET_CLASS_SERIALIZABLE(swoole_async, zend_class_serialize_deny, zend_class_unserialize_deny);
+    SWOOLE_SET_CLASS_CLONEABLE(swoole_async, zend_class_clone_deny);
+    SWOOLE_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_async, zend_class_unset_property_deny);
 
-    SWOOLE_INIT_CLASS_ENTRY(swoole_connection_iterator_ce, "swoole_connection_iterator", "Swoole\\Connection\\Iterator",  swoole_connection_iterator_methods);
-    swoole_connection_iterator_class_entry_ptr = zend_register_internal_class(&swoole_connection_iterator_ce);
-    SWOOLE_CLASS_ALIAS(swoole_connection_iterator, "Swoole\\Connection\\Iterator");
-    zend_class_implements(swoole_connection_iterator_class_entry_ptr, 2, zend_ce_iterator, zend_ce_arrayaccess);
+    SWOOLE_INIT_CLASS_ENTRY(swoole_connection_iterator, "Swoole\\Connection\\Iterator", "swoole_connection_iterator", NULL,  swoole_connection_iterator_methods);
+    SWOOLE_SET_CLASS_SERIALIZABLE(swoole_connection_iterator, zend_class_serialize_deny, zend_class_unserialize_deny);
+    SWOOLE_SET_CLASS_CLONEABLE(swoole_connection_iterator, zend_class_clone_deny);
+    SWOOLE_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_connection_iterator, zend_class_unset_property_deny);
+    zend_class_implements(swoole_connection_iterator_ce_ptr, 2, zend_ce_iterator, zend_ce_arrayaccess);
 #ifdef SW_HAVE_COUNTABLE
-    zend_class_implements(swoole_connection_iterator_class_entry_ptr, 1, zend_ce_countable);
+    zend_class_implements(swoole_connection_iterator_ce_ptr, 1, zend_ce_countable);
 #endif
 
-    SWOOLE_INIT_CLASS_ENTRY(swoole_exception_ce, "swoole_exception", "Swoole\\Exception", NULL);
-    swoole_exception_class_entry_ptr = sw_zend_register_internal_class_ex(&swoole_exception_ce, zend_exception_get_default(), NULL);
-    SWOOLE_CLASS_ALIAS(swoole_exception, "Swoole\\Exception");
+    SWOOLE_INIT_EXCEPTION_CLASS_ENTRY(swoole_exception, "Swoole\\Exception", "swoole_exception", NULL, NULL);
 
     //swoole init
     swoole_init();
@@ -1160,7 +1196,7 @@ PHP_MINIT_FUNCTION(swoole)
     }
 
     swoole_objects.size = 65536;
-    swoole_objects.array = calloc(swoole_objects.size, sizeof(void*));
+    swoole_objects.array = sw_calloc(swoole_objects.size, sizeof(void*));
 
     return SUCCESS;
 }
@@ -1195,12 +1231,13 @@ PHP_MINFO_FUNCTION(swoole)
 #ifdef SW_LOG_TRACE_OPEN
     php_info_print_table_row(2, "trace_log", "enabled");
 #endif
-#if USE_BOOST_CONTEXT
+#ifdef SW_NO_USE_ASM_CONTEXT
+#ifdef HAVE_BOOST_CONTEXT
     php_info_print_table_row(2, "boost.context", "enabled");
-#endif
-#if USE_UCONTEXT
+#else
     php_info_print_table_row(2, "ucontext", "enabled");
 #endif
+#endif
 #ifdef HAVE_EPOLL
     php_info_print_table_row(2, "epoll", "enabled");
 #endif
@@ -1242,9 +1279,6 @@ PHP_MINFO_FUNCTION(swoole)
     php_info_print_table_row(2, "http2", "enabled");
 #endif
 #endif
-#ifdef SW_USE_RINGBUFFER
-    php_info_print_table_row(2, "ringbuffer", "enabled");
-#endif
 #ifdef HAVE_PCRE
     php_info_print_table_row(2, "pcre", "enabled");
 #endif
@@ -1288,20 +1322,8 @@ PHP_MINFO_FUNCTION(swoole)
 PHP_RINIT_FUNCTION(swoole)
 {
     SWOOLE_G(req_status) = PHP_SWOOLE_RINIT_BEGIN;
-    php_swoole_at_shutdown("swoole_call_user_shutdown_begin");
-    //running
+    php_swoole_register_shutdown_function("swoole_call_user_shutdown_begin");
     SwooleG.running = 1;
-
-#ifdef ZTS
-    if (sw_thread_ctx == NULL)
-    {
-        TSRMLS_SET_CTX(sw_thread_ctx);
-    }
-#endif
-
-#ifdef SW_DEBUG_REMOTE_OPEN
-    swoole_open_remote_debug();
-#endif
     SWOOLE_G(req_status) = PHP_SWOOLE_RINIT_END;
     return SUCCESS;
 }
@@ -1311,7 +1333,7 @@ PHP_RSHUTDOWN_FUNCTION(swoole)
     SWOOLE_G(req_status) = PHP_SWOOLE_RSHUTDOWN_BEGIN;
     swoole_call_rshutdown_function(NULL);
     //clear pipe buffer
-    if (swIsWorker())
+    if (SwooleG.serv && swIsWorker())
     {
         swWorker_clean();
     }
@@ -1479,7 +1501,7 @@ PHP_FUNCTION(swoole_set_process_name)
         size = SwooleG.pagesize;
     }
 
-    zval *retval;
+    zval *retval = NULL;
     zval args[1];
     args[0] = *name;
 
@@ -1491,7 +1513,7 @@ PHP_FUNCTION(swoole_set_process_name)
     {
         return;
     }
-    if (EG(exception))
+    if (UNEXPECTED(EG(exception)))
     {
         zend_exception_error(EG(exception), E_ERROR);
     }
