@@ -55,8 +55,8 @@ typedef struct
 
 typedef struct
 {
-    swString *zaddress;
-    int64_t  update_time;
+    char address[16];
+    time_t update_time;
 } dns_cache;
 
 typedef struct
@@ -144,7 +144,7 @@ static void php_swoole_dns_callback(char *domain, swDNSResolver_result *result,
         swoole_php_fatal_error(E_WARNING, "swoole_asyns_dns_lookup handler error.");
         return;
     }
-    if (EG(exception))
+    if (UNEXPECTED(EG(exception)))
     {
         zend_exception_error(EG(exception), E_ERROR);
     }
@@ -191,15 +191,16 @@ static void coro_onDNSCompleted(char *domain, swDNSResolver_result *result, void
     if (cache_iterator == request_cache_map.end())
     {
         cache = (dns_cache *) emalloc(sizeof(dns_cache));
+        bzero(cache, sizeof(dns_cache));
         request_cache_map[key] = cache;
-        cache->zaddress = swString_new(20);
     }
     else
     {
         cache = cache_iterator->second;
     }
 
-    swString_write_ptr(cache->zaddress, 0, Z_STRVAL_P(zaddress), Z_STRLEN_P(zaddress));
+    memcpy(cache->address, Z_STRVAL_P(zaddress), Z_STRLEN_P(zaddress));
+    cache->address[Z_STRLEN_P(zaddress)] = '\0';
 
     cache->update_time = swTimer_get_absolute_msec() + (int64_t) (SwooleG.dns_cache_refresh_time * 1000);
 
@@ -221,7 +222,7 @@ static void coro_onDNSCompleted(char *domain, swDNSResolver_result *result, void
         goto free_zdata;
     }
 
-    if (retval != NULL)
+    if (retval)
     {
         zval_ptr_dtor(retval);
     }
@@ -243,7 +244,7 @@ static void dns_timeout_coro(swTimer *timer, swTimer_node *tnode)
     dns_cache *cache = request_cache_map[std::string(Z_STRVAL_P(req->domain), Z_STRLEN_P(req->domain))];
     if (cache != NULL && cache->update_time > swTimer_get_absolute_msec())
     {
-        ZVAL_STRINGL(zaddress, (*cache->zaddress).str, (*cache->zaddress).length);
+        ZVAL_STRING(zaddress, cache->address);
     }
     else
     {
@@ -256,7 +257,7 @@ static void dns_timeout_coro(swTimer *timer, swTimer_node *tnode)
         goto free_zdata;
     }
 
-    if (retval != NULL)
+    if (retval)
     {
         zval_ptr_dtor(retval);
     }
@@ -302,7 +303,7 @@ static void aio_onDNSCompleted(swAio_event *event)
         swoole_php_fatal_error(E_WARNING, "swoole_async: onAsyncComplete handler error");
         return;
     }
-    if (EG(exception))
+    if (UNEXPECTED(EG(exception)))
     {
         zend_exception_error(EG(exception), E_ERROR);
     }
@@ -391,7 +392,7 @@ static void aio_onFileCompleted(swAio_event *event)
             swoole_php_fatal_error(E_WARNING, "swoole_async: onAsyncComplete handler error");
             return;
         }
-        if (EG(exception))
+        if (UNEXPECTED(EG(exception)))
         {
             zend_exception_error(EG(exception), E_ERROR);
         }
@@ -406,7 +407,7 @@ static void aio_onFileCompleted(swAio_event *event)
     }
     else if(file_req->type == SW_AIO_WRITE)
     {
-        if (retval != NULL && !ZVAL_IS_NULL(retval) && !Z_BVAL_P(retval))
+        if (retval && !ZVAL_IS_NULL(retval) && !Z_BVAL_P(retval))
         {
             open_files.erase(std::string(Z_STRVAL_P(file_req->filename), Z_STRLEN_P(file_req->filename)));
             goto close_file;
@@ -418,7 +419,7 @@ static void aio_onFileCompleted(swAio_event *event)
     }
     else
     {
-        if ((retval != NULL && !ZVAL_IS_NULL(retval) && !Z_BVAL_P(retval)) || isEOF)
+        if ((retval && !ZVAL_IS_NULL(retval) && !Z_BVAL_P(retval)) || isEOF)
         {
             goto close_file;
         }
@@ -901,8 +902,10 @@ PHP_FUNCTION(swoole_async_set)
     }
     if (php_swoole_array_get_value(vht, "log_level", v))
     {
+        zend_long level;
         convert_to_long(v);
-        SwooleG.log_level = Z_LVAL_P(v);
+        level = Z_LVAL_P(v);
+        SwooleG.log_level = (uint32_t) (level < 0 ? UINT32_MAX : level);
     }
     if (php_swoole_array_get_value(vht, "display_errors", v))
     {
@@ -1092,11 +1095,11 @@ static int process_stream_onRead(swReactor *reactor, swEvent *event)
     }
     sw_zval_free(zcallback);
 
-    if (EG(exception))
+    if (UNEXPECTED(EG(exception)))
     {
         zend_exception_error(EG(exception), E_ERROR);
     }
-    if (retval != NULL)
+    if (retval)
     {
         zval_ptr_dtor(retval);
     }
@@ -1166,7 +1169,7 @@ PHP_METHOD(swoole_async, exec)
 PHP_FUNCTION(swoole_async_dns_lookup_coro)
 {
     zval *domain;
-    double timeout = SW_CLIENT_DEFAULT_TIMEOUT;
+    double timeout = COROG.socket_connect_timeout;
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "z|d", &domain, &timeout) == FAILURE)
     {
         RETURN_FALSE;
@@ -1193,7 +1196,7 @@ PHP_FUNCTION(swoole_async_dns_lookup_coro)
         cache = request_cache_map[key];
         if (cache->update_time > swTimer_get_absolute_msec())
         {
-            RETURN_STRINGL((*cache->zaddress).str, (*cache->zaddress).length);
+            RETURN_STRING(cache->address);
         }
     }
 
