@@ -110,10 +110,12 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_atomic_waitup, 0, 0, 0)
 ZEND_END_ARG_INFO()
 
 static zend_class_entry swoole_atomic_ce;
-zend_class_entry *swoole_atomic_class_entry_ptr;
+zend_class_entry *swoole_atomic_ce_ptr;
+static zend_object_handlers swoole_atomic_handlers;
 
 static zend_class_entry swoole_atomic_long_ce;
-zend_class_entry *swoole_atomic_long_class_entry_ptr;
+zend_class_entry *swoole_atomic_long_ce_ptr;
+static zend_object_handlers swoole_atomic_long_handlers;
 
 static const zend_function_entry swoole_atomic_methods[] =
 {
@@ -141,17 +143,15 @@ static const zend_function_entry swoole_atomic_long_methods[] =
 
 void swoole_atomic_init(int module_number)
 {
-    SWOOLE_INIT_CLASS_ENTRY(swoole_atomic_ce, "swoole_atomic", "Swoole\\Atomic", swoole_atomic_methods);
-    swoole_atomic_class_entry_ptr = zend_register_internal_class(&swoole_atomic_ce);
-    swoole_atomic_class_entry_ptr->serialize = zend_class_serialize_deny;
-    swoole_atomic_class_entry_ptr->unserialize = zend_class_unserialize_deny;
-    SWOOLE_CLASS_ALIAS(swoole_atomic, "Swoole\\Atomic");
+    SWOOLE_INIT_CLASS_ENTRY(swoole_atomic, "Swoole\\Atomic", "swoole_atomic", NULL, swoole_atomic_methods);
+    SWOOLE_SET_CLASS_SERIALIZABLE(swoole_atomic, zend_class_serialize_deny, zend_class_unserialize_deny);
+    SWOOLE_SET_CLASS_CLONEABLE(swoole_atomic, zend_class_clone_deny);
+    SWOOLE_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_atomic, zend_class_unset_property_deny);
 
-    SWOOLE_INIT_CLASS_ENTRY(swoole_atomic_long_ce, "swoole_atomic_long", "Swoole\\Atomic\\Long", swoole_atomic_long_methods);
-    swoole_atomic_long_class_entry_ptr = zend_register_internal_class(&swoole_atomic_long_ce);
-    swoole_atomic_long_class_entry_ptr->serialize = zend_class_serialize_deny;
-    swoole_atomic_long_class_entry_ptr->unserialize = zend_class_unserialize_deny;
-    SWOOLE_CLASS_ALIAS(swoole_atomic_long, "Swoole\\Atomic\\Long");
+    SWOOLE_INIT_CLASS_ENTRY(swoole_atomic_long, "Swoole\\Atomic\\Long", "swoole_atomic_long", NULL, swoole_atomic_long_methods);
+    SWOOLE_SET_CLASS_SERIALIZABLE(swoole_atomic_long, zend_class_serialize_deny, zend_class_unserialize_deny);
+    SWOOLE_SET_CLASS_CLONEABLE(swoole_atomic_long, zend_class_clone_deny);
+    SWOOLE_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_atomic_long, zend_class_unset_property_deny);
 }
 
 PHP_METHOD(swoole_atomic, __construct)
@@ -166,7 +166,7 @@ PHP_METHOD(swoole_atomic, __construct)
     sw_atomic_t *atomic = SwooleG.memory_pool->alloc(SwooleG.memory_pool, sizeof(sw_atomic_t));
     if (atomic == NULL)
     {
-        zend_throw_exception(swoole_exception_class_entry_ptr, "global memory allocation failure.", SW_ERROR_MALLOC_FAIL);
+        zend_throw_exception(swoole_exception_ce_ptr, "global memory allocation failure.", SW_ERROR_MALLOC_FAIL);
         RETURN_FALSE;
     }
     *atomic = (sw_atomic_t) value;
@@ -246,9 +246,10 @@ PHP_METHOD(swoole_atomic, wait)
     SW_CHECK_RETURN(swoole_futex_wait(atomic, timeout));
 #else
     timeout = timeout <= 0 ? SW_MAX_INT : timeout;
+    sw_atomic_long_t i = sw_atomic_add_fetch(atomic, 1);
     while (timeout > 0)
     {
-        if (sw_atomic_cmp_set(atomic, 1, 0))
+        if (*atomic < i)
         {
             RETURN_TRUE;
         }
@@ -274,7 +275,7 @@ PHP_METHOD(swoole_atomic, wakeup)
 #ifdef HAVE_FUTEX
     SW_CHECK_RETURN(swoole_futex_wakeup(atomic, (int ) n));
 #else
-    *atomic = 1;
+    sw_atomic_fetch_sub(atomic, n);
 #endif
 }
 
@@ -290,7 +291,7 @@ PHP_METHOD(swoole_atomic_long, __construct)
     sw_atomic_long_t *atomic = SwooleG.memory_pool->alloc(SwooleG.memory_pool, sizeof(sw_atomic_long_t));
     if (atomic == NULL)
     {
-        zend_throw_exception(swoole_exception_class_entry_ptr, "global memory allocation failure.", SW_ERROR_MALLOC_FAIL);
+        zend_throw_exception(swoole_exception_ce_ptr, "global memory allocation failure.", SW_ERROR_MALLOC_FAIL);
         RETURN_FALSE;
     }
     *atomic = (sw_atomic_long_t) value;
