@@ -134,31 +134,27 @@ static const zend_function_entry swoole_client_coro_methods[] =
     PHP_FE_END
 };
 
-zend_class_entry swoole_client_coro_ce;
-zend_class_entry *swoole_client_coro_class_entry_ptr;
+static zend_class_entry swoole_client_coro_ce;
+zend_class_entry *swoole_client_coro_ce_ptr;
+static zend_object_handlers swoole_client_coro_handlers;
 
 void swoole_client_coro_init(int module_number)
 {
-    INIT_CLASS_ENTRY(swoole_client_coro_ce, "Swoole\\Coroutine\\Client", swoole_client_coro_methods);
-    swoole_client_coro_class_entry_ptr = zend_register_internal_class(&swoole_client_coro_ce);
-    swoole_client_coro_class_entry_ptr->serialize = zend_class_serialize_deny;
-    swoole_client_coro_class_entry_ptr->unserialize = zend_class_unserialize_deny;
+    SWOOLE_INIT_CLASS_ENTRY(swoole_client_coro, "Swoole\\Coroutine\\Client", NULL, "Co\\Client", swoole_client_coro_methods);
+    SWOOLE_SET_CLASS_SERIALIZABLE(swoole_client_coro, zend_class_serialize_deny, zend_class_unserialize_deny);
+    SWOOLE_SET_CLASS_CLONEABLE(swoole_client_coro, zend_class_clone_deny);
+    SWOOLE_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_client_coro, zend_class_unset_property_deny);
 
-    if (SWOOLE_G(use_shortname))
-    {
-        sw_zend_register_class_alias("Co\\Client", swoole_client_coro_class_entry_ptr);
-    }
+    zend_declare_property_long(swoole_client_coro_ce_ptr, ZEND_STRL("errCode"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_client_coro_ce_ptr, ZEND_STRL("sock"), -1, ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_client_coro_ce_ptr, ZEND_STRL("type"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_client_coro_ce_ptr, ZEND_STRL("setting"), ZEND_ACC_PUBLIC);
+    zend_declare_property_bool(swoole_client_coro_ce_ptr, ZEND_STRL("connected"), 0, ZEND_ACC_PUBLIC);
 
-    zend_declare_property_long(swoole_client_coro_class_entry_ptr, ZEND_STRL("errCode"), 0, ZEND_ACC_PUBLIC);
-    zend_declare_property_long(swoole_client_coro_class_entry_ptr, ZEND_STRL("sock"), 0, ZEND_ACC_PUBLIC);
-    zend_declare_property_long(swoole_client_coro_class_entry_ptr, ZEND_STRL("type"), 0, ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_client_coro_class_entry_ptr, ZEND_STRL("setting"), ZEND_ACC_PUBLIC);
-    zend_declare_property_bool(swoole_client_coro_class_entry_ptr, ZEND_STRL("connected"), 0, ZEND_ACC_PUBLIC);
-
-    zend_declare_class_constant_long(swoole_client_coro_class_entry_ptr, ZEND_STRL("MSG_OOB"), MSG_OOB);
-    zend_declare_class_constant_long(swoole_client_coro_class_entry_ptr, ZEND_STRL("MSG_PEEK"), MSG_PEEK);
-    zend_declare_class_constant_long(swoole_client_coro_class_entry_ptr, ZEND_STRL("MSG_DONTWAIT"), MSG_DONTWAIT);
-    zend_declare_class_constant_long(swoole_client_coro_class_entry_ptr, ZEND_STRL("MSG_WAITALL"), MSG_WAITALL);
+    zend_declare_class_constant_long(swoole_client_coro_ce_ptr, ZEND_STRL("MSG_OOB"), MSG_OOB);
+    zend_declare_class_constant_long(swoole_client_coro_ce_ptr, ZEND_STRL("MSG_PEEK"), MSG_PEEK);
+    zend_declare_class_constant_long(swoole_client_coro_ce_ptr, ZEND_STRL("MSG_DONTWAIT"), MSG_DONTWAIT);
+    zend_declare_class_constant_long(swoole_client_coro_ce_ptr, ZEND_STRL("MSG_WAITALL"), MSG_WAITALL);
 }
 
 static sw_inline Socket* client_get_ptr(zval *zobject)
@@ -171,7 +167,7 @@ static sw_inline Socket* client_get_ptr(zval *zobject)
     else
     {
         SwooleG.error = SW_ERROR_CLIENT_NO_CONNECTION;
-        zend_update_property_long(swoole_client_coro_class_entry_ptr, zobject, ZEND_STRL("errCode"), SwooleG.error);
+        zend_update_property_long(swoole_client_coro_ce_ptr, zobject, ZEND_STRL("errCode"), SwooleG.error);
         swoole_php_error(E_WARNING, "client is not connected to server.");
         return NULL;
     }
@@ -214,12 +210,14 @@ static Socket* client_coro_new(zval *zobject, int port)
     }
 #endif
 
+    swoole_set_object(zobject, cli);
+
     return cli;
 }
 
 bool php_swoole_client_coro_socket_free(Socket *cli)
 {
-    bool ret = cli->close();
+    bool ret;
     //TODO: move to Socket method, we should not manage it externally
     //socks5 proxy config
     if (cli->socks5_proxy)
@@ -261,7 +259,9 @@ bool php_swoole_client_coro_socket_free(Socket *cli)
     {
         zval *zcallback = (zval *) cli->protocol.private_data;
         sw_zval_free(zcallback);
+        cli->protocol.private_data = nullptr;
     }
+    ret = cli->close();
     delete cli;
 
     return ret;
@@ -359,7 +359,6 @@ void php_swoole_client_coro_check_setting(Socket *cli, zval *zset)
             if (!sw_zend_is_callable(v, 0, &func_name))
             {
                 swoole_php_fatal_error(E_ERROR, "function '%s' is not callable", func_name);
-                efree(func_name);
                 return;
             }
             efree(func_name);
@@ -437,19 +436,13 @@ void php_swoole_client_coro_check_setting(Socket *cli, zval *zset)
     if (php_swoole_array_get_value(vht, "open_tcp_nodelay", v))
     {
         convert_to_boolean(v);
-        if (Z_BVAL_P(v))
-        {
-            goto _open_tcp_nodelay;
-        }
+        cli->set_tcp_nodelay(Z_BVAL_P(v));
     }
     else
     {
-        _open_tcp_nodelay: value = 1;
-        if (setsockopt(cli->socket->fd, IPPROTO_TCP, TCP_NODELAY, &value, sizeof(value)) < 0)
-        {
-            swSysError("setsockopt(%d, TCP_NODELAY) failed.", cli->socket->fd);
-        }
+        cli->set_tcp_nodelay(1);
     }
+
     /**
      * socks5 proxy
      */
@@ -640,7 +633,7 @@ static PHP_METHOD(swoole_client_coro, __construct)
 
     php_swoole_check_reactor();
 
-    zend_update_property_long(swoole_client_coro_class_entry_ptr, getThis(), ZEND_STRL("type"), type);
+    zend_update_property_long(swoole_client_coro_ce_ptr, getThis(), ZEND_STRL("type"), type);
     //init
     swoole_set_object(getThis(), NULL);
     swoole_set_property(getThis(), client_property_callback, NULL);
@@ -660,7 +653,7 @@ static PHP_METHOD(swoole_client_coro, __destruct)
     {
         zval *zobject = getThis();
         zval *retval = NULL;
-        sw_zend_call_method_with_0_params(&zobject, swoole_client_coro_class_entry_ptr, NULL, "close", &retval);
+        sw_zend_call_method_with_0_params(&zobject, swoole_client_coro_ce_ptr, NULL, "close", &retval);
         if (retval)
         {
             zval_ptr_dtor(retval);
@@ -680,7 +673,7 @@ static PHP_METHOD(swoole_client_coro, set)
         RETURN_FALSE;
     }
 
-    zval *zsetting = sw_zend_read_property_array(swoole_client_coro_class_entry_ptr, getThis(), ZEND_STRL("setting"), 1);
+    zval *zsetting = sw_zend_read_property_array(swoole_client_coro_ce_ptr, getThis(), ZEND_STRL("setting"), 1);
     php_array_merge(Z_ARRVAL_P(zsetting), Z_ARRVAL_P(zset));
 
     RETURN_TRUE;
@@ -691,7 +684,7 @@ static PHP_METHOD(swoole_client_coro, connect)
     zend_long port = 0, sock_flag = 0;
     char *host = NULL;
     size_t host_len;
-    double timeout = SW_CLIENT_DEFAULT_TIMEOUT;
+    double timeout = COROG.socket_connect_timeout;
 
     ZEND_PARSE_PARAMETERS_START(1, 4)
         Z_PARAM_STRING(host, host_len)
@@ -701,9 +694,9 @@ static PHP_METHOD(swoole_client_coro, connect)
         Z_PARAM_LONG(sock_flag)
     ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
-    if (host_len <= 0)
+    if (host_len == 0)
     {
-        swoole_php_fatal_error(E_WARNING, "The host is empty.");
+        swoole_php_fatal_error(E_ERROR, "The host is empty.");
         RETURN_FALSE;
     }
 
@@ -719,23 +712,22 @@ static PHP_METHOD(swoole_client_coro, connect)
     {
         RETURN_FALSE;
     }
-    cli->_timeout = timeout;
-    swoole_set_object(getThis(), cli);
 
-    zval *zset = sw_zend_read_property(swoole_client_coro_class_entry_ptr, getThis(), ZEND_STRL("setting"), 1);
+    zval *zset = sw_zend_read_property(swoole_client_coro_ce_ptr, getThis(), ZEND_STRL("setting"), 1);
     if (zset && ZVAL_IS_ARRAY(zset))
     {
         php_swoole_client_coro_check_setting(cli, zset);
     }
 
-    sw_coro_check_bind("client", cli->has_bound(swoole::SOCKET_LOCK_RW));
+    sw_coro_check_bind("client", cli->has_bound());
+    cli->set_timeout(timeout, true);
     if (!cli->connect(host, port, sock_flag))
     {
-        zend_update_property_long(swoole_client_coro_class_entry_ptr, getThis(), ZEND_STRL("errCode"), cli->errCode);
+        zend_update_property_long(swoole_client_coro_ce_ptr, getThis(), ZEND_STRL("errCode"), cli->errCode);
         swoole_php_error(E_WARNING, "connect to server[%s:%d] failed. Error: %s[%d]", host, (int )port, cli->errMsg, cli->errCode);
         RETURN_FALSE;
     }
-    zend_update_property_bool(swoole_client_coro_class_entry_ptr, getThis(), ZEND_STRL("connected"), 1);
+    zend_update_property_bool(swoole_client_coro_ce_ptr, getThis(), ZEND_STRL("connected"), 1);
     RETURN_TRUE;
 }
 
@@ -743,9 +735,12 @@ static PHP_METHOD(swoole_client_coro, send)
 {
     char *data;
     size_t data_len;
+    double timeout = COROG.socket_timeout;
 
-    ZEND_PARSE_PARAMETERS_START(1, 1)
+    ZEND_PARSE_PARAMETERS_START(1, 2)
         Z_PARAM_STRING(data, data_len)
+        Z_PARAM_OPTIONAL
+        Z_PARAM_DOUBLE(timeout)
     ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
     if (data_len <= 0)
@@ -762,12 +757,13 @@ static PHP_METHOD(swoole_client_coro, send)
 
     //clear errno
     SwooleG.error = 0;
-    sw_coro_check_bind("client", cli->has_bound(swoole::SOCKET_LOCK_WRITE));
+    sw_coro_check_bind("client", cli->has_bound());
+    cli->set_timeout(timeout, true);
     int ret = cli->send_all(data, data_len);
     if (ret < 0)
     {
         swoole_php_sys_error(E_WARNING, "failed to send(%d) %zd bytes.", cli->socket->fd, data_len);
-        zend_update_property_long(swoole_client_coro_class_entry_ptr, getThis(), ZEND_STRL("errCode"), SwooleG.error);
+        zend_update_property_long(swoole_client_coro_ce_ptr, getThis(), ZEND_STRL("errCode"), SwooleG.error);
         RETVAL_FALSE;
     }
     else
@@ -804,16 +800,15 @@ static PHP_METHOD(swoole_client_coro, sendto)
             RETURN_FALSE;
         }
         cli->socket->active = 1;
-        swoole_set_object(getThis(), cli);
     }
-    sw_coro_check_bind("client", cli->has_bound(swoole::SOCKET_LOCK_WRITE));
+    sw_coro_check_bind("client", cli->has_bound());
     SW_CHECK_RETURN(cli->sendto(ip, port, data, len));
 }
 
 static PHP_METHOD(swoole_client_coro, recvfrom)
 {
     zend_long length;
-    zval *port, *address;
+    zval *address, *port;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "lz/|z/", &length, &address, &port) == FAILURE)
     {
@@ -835,11 +830,11 @@ static PHP_METHOD(swoole_client_coro, recvfrom)
             RETURN_FALSE;
         }
         cli->socket->active = 1;
-        swoole_set_object(getThis(), cli);
     }
 
     zend_string *retval = zend_string_alloc(length + 1, 0);
-    sw_coro_check_bind("client", cli->has_bound(swoole::SOCKET_LOCK_READ));
+    sw_coro_check_bind("client", cli->has_bound());
+    // cli->set_timeout(timeout, true); TODO
     ssize_t n_bytes = cli->recvfrom(retval->val, length);
     if (n_bytes < 0)
     {
@@ -886,13 +881,13 @@ static PHP_METHOD(swoole_client_coro, sendfile)
     }
     //clear errno
     SwooleG.error = 0;
-    sw_coro_check_bind("client", cli->has_bound(swoole::SOCKET_LOCK_WRITE));
+    sw_coro_check_bind("client", cli->has_bound());
     int ret = cli->sendfile(file, offset, length);
     if (ret < 0)
     {
         SwooleG.error = errno;
         swoole_php_fatal_error(E_WARNING, "sendfile() failed. Error: %s [%d]", strerror(SwooleG.error), SwooleG.error);
-        zend_update_property_long(swoole_client_coro_class_entry_ptr, getThis(), ZEND_STRL("errCode"), SwooleG.error);
+        zend_update_property_long(swoole_client_coro_ce_ptr, getThis(), ZEND_STRL("errCode"), SwooleG.error);
         RETVAL_FALSE;
     }
     else
@@ -903,7 +898,7 @@ static PHP_METHOD(swoole_client_coro, sendfile)
 
 static PHP_METHOD(swoole_client_coro, recv)
 {
-    double timeout = 0;
+    double timeout = COROG.socket_timeout;
 
     ZEND_PARSE_PARAMETERS_START(0, 1)
         Z_PARAM_OPTIONAL
@@ -915,18 +910,15 @@ static PHP_METHOD(swoole_client_coro, recv)
     {
         RETURN_FALSE;
     }
-    sw_coro_check_bind("client", cli->has_bound(swoole::SOCKET_LOCK_READ));
-    if (timeout != 0)
-    {
-        cli->set_timeout(timeout);
-    }
+    sw_coro_check_bind("client", cli->has_bound());
+    cli->set_timeout(timeout, true);
     ssize_t retval ;
     if (cli->open_length_check || cli->open_eof_check)
     {
         retval = cli->recv_packet();
         if (retval > 0)
         {
-            RETVAL_STRINGL(cli->buffer->str, retval);
+            RETVAL_STRINGL(cli->read_buffer->str, retval);
         }
     }
     else
@@ -948,7 +940,7 @@ static PHP_METHOD(swoole_client_coro, recv)
     {
         SwooleG.error = cli->errCode;
         swoole_php_error(E_WARNING, "recv() failed. Error: %s [%d]", strerror(SwooleG.error), SwooleG.error);
-        zend_update_property_long(swoole_client_coro_class_entry_ptr, getThis(), ZEND_STRL("errCode"), SwooleG.error);
+        zend_update_property_long(swoole_client_coro_ce_ptr, getThis(), ZEND_STRL("errCode"), SwooleG.error);
         RETURN_FALSE;
     }
     else if (retval == 0)
@@ -977,12 +969,11 @@ static PHP_METHOD(swoole_client_coro, peek)
     SwooleG.error = 0;
     buf = (char *) emalloc(buf_len + 1);
     ret = cli->peek(buf, buf_len);
-
     if (ret < 0)
     {
         SwooleG.error = errno;
         swoole_php_error(E_WARNING, "recv() failed. Error: %s [%d]", strerror(SwooleG.error), SwooleG.error);
-        zend_update_property_long(swoole_client_coro_class_entry_ptr, getThis(), ZEND_STRL("errCode"),
+        zend_update_property_long(swoole_client_coro_ce_ptr, getThis(), ZEND_STRL("errCode"),
                 SwooleG.error);
         efree(buf);
         RETURN_FALSE;
@@ -1133,7 +1124,6 @@ static PHP_METHOD(swoole_client_coro, close)
     }
 #endif
 
-    sw_coro_check_bind("client", cli->has_bound(swoole::SOCKET_LOCK_RW));
     int ret = php_swoole_client_coro_socket_free(cli) ? SW_OK : SW_ERR;
     swoole_set_object(zobject, NULL);
 
@@ -1159,12 +1149,12 @@ static PHP_METHOD(swoole_client_coro, enableSSL)
         RETURN_FALSE;
     }
     cli->open_ssl = true;
-    zval *zset = sw_zend_read_property(swoole_client_coro_class_entry_ptr, getThis(), ZEND_STRL("setting"), 1);
-    if (zset && ZVAL_IS_ARRAY(zset))
+    zval *zset = sw_zend_read_property(swoole_client_coro_ce_ptr, getThis(), ZEND_STRL("setting"), 0);
+    if (ZVAL_IS_ARRAY(zset))
     {
         client_coro_check_ssl_setting(cli, zset);
     }
-    sw_coro_check_bind("client", cli->has_bound(swoole::SOCKET_LOCK_RW));
+    sw_coro_check_bind("client", cli->has_bound());
     if (cli->ssl_handshake() == false)
     {
         RETURN_FALSE;
