@@ -27,7 +27,7 @@
 
 #define SW_MAX_FDTYPE              32   //32 kinds of event
 #define SW_MAX_HOOK_TYPE           32
-#define SW_ERROR_MSG_SIZE          1024
+#define SW_ERROR_MSG_SIZE          8192
 #define SW_MAX_FILE_CONTENT        (64*1024*1024) //for swoole_file_get_contents
 #define SW_MAX_LISTEN_PORT         60000
 #define SW_MAX_CONNECTION          10000
@@ -40,15 +40,7 @@
 
 #define SW_MALLOC_TRIM_INTERVAL    1
 #define SW_MALLOC_TRIM_PAD         0
-#define SW_USE_EVENT_TIMER
-#define SW_USE_MONOTONIC_TIME
-//#define SW_USE_RINGBUFFER
-
-//#define SW_DEBUG_REMOTE_OPEN
-#define SW_DEBUG_SERVER_HOST       "127.0.0.1"
-#define SW_DEBUG_SERVER_PORT       9999
-
-#define SW_DEBUG_SERVER_DESTRUCT   0
+#define SW_USE_MONOTONIC_TIME      1
 
 #define SW_SOCKET_OVERFLOW_WAIT    100
 #define SW_SOCKET_MAX_DEFAULT      65536
@@ -59,38 +51,26 @@
 #endif
 #define SW_SYSTEMD_FDS_START       3
 
-#define SW_GLOBAL_MEMORY_PAGESIZE  (1024*1024*2) //全局内存的分页
+#define SW_GLOBAL_MEMORY_PAGESIZE  (1024*1024*2) // global memory page
 
-#define SW_MAX_THREAD_NCPU         4 // n * cpu_num
+#define SW_MAX_THREAD_NCPU         4    // n * cpu_num
 #define SW_MAX_WORKER_NCPU         1000 // n * cpu_num
-#define SW_MAX_REQUEST             5000          //最大请求包数
 
-#define SW_CORO_SCHEDUER_TIMEOUT   100           //协程强制超时回调的单位时间 100ms
-//#define SW_CONNECTION_LIST_EXPAND  (4096*2)  //动态扩容的数量
+#define SW_HOST_MAXSIZE            sizeof(((struct sockaddr_un *)NULL)->sun_path)  // Linux has 108 UNIX_PATH_MAX, but BSD/MacOS limit is only 104
 
-#define SW_HOST_MAXSIZE            104  // Linux has 108 UNIX_PATH_MAX, but BSD/MacOS limit is only 104
-
-//#define SW_DEBUG                 //debug
 #define SW_LOG_NO_SRCINFO          //no source info
-//#define SW_BUFFER_SIZE           65495 //65535 - 28 - 12(UDP最大包 - 包头 - 3个INT)
 #define SW_CLIENT_BUFFER_SIZE      65536
-//#define SW_CLIENT_RECV_AGAIN
-#define SW_CLIENT_DEFAULT_TIMEOUT  0.5
+#define SW_CLIENT_CONNECT_TIMEOUT  0.5
 #define SW_CLIENT_MAX_PORT         65535
-//#define SW_CLIENT_SOCKET_WAIT
 
 //!!!Don't modify.----------------------------------------------------------
 #ifdef __MACH__
-#define SW_IPC_MAX_SIZE            2048  //MacOS
+#define SW_IPC_MAX_SIZE            2048  // MacOS
 #else
-#define SW_IPC_MAX_SIZE            8192  //for IPC, dgram and message-queue max size
+#define SW_IPC_MAX_SIZE            8192  // for IPC, dgram and message-queue max size
 #endif
 
-#ifdef SW_USE_RINGBUFFER
-#define SW_BUFFER_SIZE             65535
-#else
 #define SW_BUFFER_SIZE             (SW_IPC_MAX_SIZE - sizeof(struct _swDataHead))
-#endif
 //!!!End.-------------------------------------------------------------------
 
 #define SW_BUFFER_SIZE_STD         8192
@@ -102,7 +82,7 @@
 #define SW_SENDFILE_MAXLEN         4194304
 
 #define SW_HASHMAP_KEY_MAXLEN      256
-#define SW_HASHMAP_INIT_BUCKET_N   32  //hashmap初始化时创建32大小的桶
+#define SW_HASHMAP_INIT_BUCKET_N   32  // hashmap bucket num (default value for init)
 
 #define SW_DATA_EOF                "\r\n\r\n"
 #define SW_DATA_EOF_MAXLEN         8
@@ -111,78 +91,73 @@
 
 #define SW_AIO_THREAD_NUM_DEFAULT        2
 #define SW_AIO_THREAD_NUM_MAX            32
-#define SW_AIO_MAX_FILESIZE              4194304  //4M
+#define SW_AIO_MAX_FILESIZE              (4*1024*1024)
 #define SW_AIO_EVENT_NUM                 128
 #define SW_AIO_DEFAULT_CHUNK_SIZE        65536
-#define SW_AIO_MAX_CHUNK_SIZE            1*1024*1024
-//#define SW_AIO_THREAD_USE_CHANNEL
+#define SW_AIO_MAX_CHUNK_SIZE            (1*1024*1024)
 #define SW_AIO_MAX_EVENTS                128
 #define SW_AIO_HANDLER_MAX_SIZE          8
-//#define SW_THREADPOOL_USE_CHANNEL
 #define SW_THREADPOOL_QUEUE_LEN          10000
 #define SW_IP_MAX_LENGTH                 46
 
-//#define SW_USE_SOCKET_LINGER
-
-#define SW_WORKER_WAIT_TIMEOUT     1000
-//#define SW_WORKER_RECV_AGAIN
+#define SW_WORKER_WAIT_TIMEOUT           1000
 
 #define SW_WORKER_USE_SIGNALFD
-#define SW_WORKER_MAX_WAIT_TIME          30           //最大等待时间
+#define SW_WORKER_MAX_WAIT_TIME          30
 
-//#define SW_WORKER_SEND_CHUNK
-
-#define SW_REACTOR_SCHEDULE              2
 #define SW_REACTOR_MAXEVENTS             4096
-#define SW_REACTOR_USE_SESSION
-#define SW_SESSION_LIST_SIZE             (1024*1024)
+#define SW_SESSION_LIST_SIZE             (1*1024*1024)
 
 #define SW_MSGMAX                        65536
 
 /**
- * 最大Reactor线程数量，默认会启动CPU核数的线程数
- * 如果超过8核，默认启动8个线程
+ * The maximum number of Reactor threads
+ * the number of the CPU cores threads will be started by default
+ * number 8 is the maximum
  */
 #define SW_REACTOR_MAX_THREAD            8
 
 /**
- * 循环从管道中读取数据，有助于缓解管道缓存塞满问题，降低进程间通信的压力
+ * Loops read data from the pipeline,
+ * helping to alleviate pipeline cache congestion
+ * reduce the pressure of interprocess communication
  */
-#define SW_REACTOR_RECV_AGAIN
-#define SW_REACTOR_SYNC_SEND            //direct send
+#define SW_REACTOR_RECV_AGAIN            1
+#define SW_REACTOR_SYNC_SEND             1    // direct send
 
-#define SW_RINGQUEUE_LEN                 1024           //RingQueue队列长度
-
-//#define SW_USE_RINGQUEUE_TS            1     //使用线程安全版本的RingQueue
-#define SW_RINGBUFFER_FREE_N_MAX         4     //when free_n > MAX, execute collect
+/**
+ * RINGBUFFER
+ */
+#define SW_RINGQUEUE_LEN                 1024
+#define SW_RINGBUFFER_FREE_N_MAX         4     // when free_n > MAX, execute collect
 #define SW_RINGBUFFER_WARNING            100
-//#define SW_RINGBUFFER_DEBUG
 
 /**
  * ringbuffer memory pool size
  */
-#define SW_BUFFER_OUTPUT_SIZE            (1024*1024*2)
-#define SW_BUFFER_INPUT_SIZE             (1024*1024*2)
+#define SW_BUFFER_OUTPUT_SIZE            (2*1024*1024)
+#define SW_BUFFER_INPUT_SIZE             (2*1024*1024)
 #define SW_BUFFER_MIN_SIZE               65536
-#define SW_PIPE_BUFFER_SIZE              (1024*1024*32)
 
 #define SW_BACKLOG                       512
 
 /**
- * 是否循环accept，可以一次性处理完全部的listen队列，用于大量并发连接的场景
+ * Whether to cycle accept
+ * you can process the full listen queue in one time
+ * for a large number of concurrent connections
  */
 #define SW_ACCEPT_AGAIN                  1
 
 /**
- * 一次循环的最大accept次数
+ * max accept times for single time
  */
 #define SW_ACCEPT_MAX_COUNT              64
 
 #define SW_TCP_KEEPCOUNT                 5
-#define SW_TCP_KEEPIDLE                  3600 //1 hour
+#define SW_TCP_KEEPIDLE                  3600 // 1 hour
 #define SW_TCP_KEEPINTERVAL              60
 
-#define SW_USE_EVENTFD                   //是否使用eventfd来做消息通知，需要Linux 2.6.22以上版本才会支持
+#define SW_USE_EVENTFD                   1 // Whether to use eventfd for message notification, Linux 2.6.22 or later is required to support
 
 #define SW_TASK_TMP_FILE                 "/tmp/swoole.task.XXXXXX"
 #define SW_TASK_TMPDIR_SIZE              128
@@ -209,6 +184,7 @@
 #define SW_STRING_BUFFER_GARBAGE_RATIO   4
 
 #define SW_SIGNO_MAX                     128
+#define SW_UNREGISTERED_SIGNAL_FMT       "Unable to find callback function for signal %s."
 
 #define SW_DNS_HOST_BUFFER_SIZE          16
 #define SW_DNS_SERVER_PORT               53
@@ -257,15 +233,22 @@
 #define SW_WEBSOCKET_KEY_LENGTH          16
 #define SW_WEBSOCKET_QUEUE_SIZE          16
 
-#define SW_MYSQL_QUERY_INIT_SIZE         8192
+/**
+ * MySQL Client
+ */
 #define SW_MYSQL_DEFAULT_PORT            3306
 #define SW_MYSQL_CONNECT_TIMEOUT         1.0
-#define SW_MYSQL_DEFAULT_CHARSET         33  //0x21, utf8_general_ci
+#define SW_MYSQL_DEFAULT_CHARSET         33  // 0x21, utf8_general_ci
 
+/**
+ * Redis Client
+ */
 #define SW_REDIS_CONNECT_TIMEOUT         1.0
-#define SW_PGSQL_CONNECT_TIMEOUT         3.0
 
-#define SW_TIMER_MAX_VALUE               86400000
+/**
+ * PGSQL Client
+ */
+#define SW_PGSQL_CONNECT_TIMEOUT         3.0
 
 /**
  * Coroutine
