@@ -56,10 +56,13 @@ static sw_inline void php_vm_stack_destroy(zend_vm_stack stack)
 
 static sw_inline void php_coro_save_vm_stack(coro_task *task)
 {
-    task->execute_data = EG(current_execute_data);
-    task->vm_stack = EG(vm_stack);
     task->vm_stack_top = EG(vm_stack_top);
     task->vm_stack_end = EG(vm_stack_end);
+    task->vm_stack = EG(vm_stack);
+    task->execute_data = EG(current_execute_data);
+    task->error_handling = EG(error_handling);
+    task->exception_class = EG(exception_class);
+    task->exception = EG(exception);
     SW_SAVE_EG_SCOPE(task->scope);
 }
 
@@ -76,10 +79,13 @@ static sw_inline coro_task* php_coro_get_current_task()
 
 static sw_inline void php_coro_restore_vm_stack(coro_task *task)
 {
-    EG(current_execute_data) = task->execute_data;
-    EG(vm_stack) = task->vm_stack;
     EG(vm_stack_top) = task->vm_stack_top;
     EG(vm_stack_end) = task->vm_stack_end;
+    EG(vm_stack) = task->vm_stack;
+    EG(current_execute_data) = task->execute_data;
+    EG(error_handling) = task->error_handling;
+    EG(exception_class) = task->exception_class;
+    EG(exception) = task->exception;
     SW_SET_EG_SCOPE(task->scope);
 }
 
@@ -174,8 +180,10 @@ static sw_inline void php_coro_og_close(coro_task *task)
 
 void coro_init(void)
 {
-    COROG.max_coro_num = DEFAULT_MAX_CORO_NUM;
-    COROG.stack_size = DEFAULT_STACK_SIZE;
+    COROG.max_coro_num = SW_DEFAULT_MAX_CORO_NUM;
+    COROG.stack_size = SW_DEFAULT_STACK_SIZE;
+    COROG.socket_connect_timeout = SW_DEFAULT_SOCKET_CONNECT_TIMEOUT;
+    COROG.socket_timeout = SW_DEFAULT_SOCKET_TIMEOUT;
     coroutine_set_onYield(internal_coro_yield);
     coroutine_set_onResume(internal_coro_resume);
     coroutine_set_onClose(sw_coro_close);
@@ -189,15 +197,15 @@ static void php_coro_create(void *arg)
     zend_function *func = fci_cache->function_handler;
     zval *argv = php_arg->argv;
     int argc = php_arg->argc;
-    zval *retval = php_arg->retval;
     coro_task *task;
     coro_task *origin_task = php_arg->origin_task;
     zend_execute_data *call;
     zval _zobject, *zobject = nullptr;
+    zval _retval, *retval = &_retval;
 
-    if (++COROG.coro_num > COROG.peak_coro_num)
+    if (swCoroG.count() > COROG.peak_coro_num)
     {
-        COROG.peak_coro_num = COROG.coro_num;
+        COROG.peak_coro_num = swCoroG.count();
     }
 
     if (fci_cache->object)
@@ -233,8 +241,6 @@ static void php_coro_create(void *arg)
 
     call->symbol_table = NULL;
 
-    // TODO: enhancement it, separate execute data is necessary, but we lose the backtrace
-    EG(current_execute_data) = NULL;
     if (func->op_array.fn_flags & ZEND_ACC_CLOSURE)
     {
         uint32_t call_info;
@@ -243,8 +249,10 @@ static void php_coro_create(void *arg)
         ZEND_ADD_CALL_FLAG(call, call_info);
     }
 
-    zend_init_execute_data(call, &func->op_array, retval);
     EG(current_execute_data) = call;
+    EG(error_handling) = EH_NORMAL;
+    EG(exception_class) = NULL;
+    EG(exception) = NULL;
 
     php_coro_save_vm_stack(task);
     task->output_ptr = nullptr;
@@ -255,8 +263,8 @@ static void php_coro_create(void *arg)
     php_coro_og_create(origin_task);
 
     swTraceLog(
-        SW_TRACE_COROUTINE, "Create coro id: %ld, origin cid: %ld, coro total count: %" PRIu64 ", heap size: %zu",
-        coroutine_get_cid(task->co), coroutine_get_cid(task->origin_task->co), COROG.coro_num, zend_memory_usage(0)
+        SW_TRACE_COROUTINE, "Create coro id: %ld, origin cid: %ld, coro total count: %zu, heap size: %zu",
+        coroutine_get_cid(task->co), coroutine_get_cid(task->origin_task->co), (uintmax_t) swCoroG.count(), (uintmax_t) zend_memory_usage(0)
     );
 
     if (SwooleG.hooks[SW_GLOBAL_HOOK_ON_CORO_START])
@@ -264,7 +272,26 @@ static void php_coro_create(void *arg)
         swoole_call_hook(SW_GLOBAL_HOOK_ON_CORO_START, task);
     }
 
-    zend_execute_ex(EG(current_execute_data));
+    if (EXPECTED(func->type == ZEND_USER_FUNCTION))
+    {
+        ZVAL_UNDEF(retval);
+        // TODO: enhancement it, separate execute data is necessary, but we lose the backtrace
+        EG(current_execute_data) = NULL;
+#if PHP_VERSION_ID >= 70200
+        zend_init_func_execute_data(call, &func->op_array, retval);
+#else
+        zend_init_execute_data(call, &func->op_array, retval);
+#endif
+        zend_execute_ex(EG(current_execute_data));
+    }
+    else /* ZEND_INTERNAL_FUNCTION */
+    {
+        ZVAL_NULL(retval);
+        call->prev_execute_data = NULL;
+        call->return_value = NULL; /* this is not a constructor call */
+        func->internal_function.handler(call, retval);
+        zend_vm_stack_free_args(call);
+    }
 
     if (task->defer_tasks)
     {
@@ -280,12 +307,14 @@ static void php_coro_create(void *arg)
         task->defer_tasks = nullptr;
     }
 
+    zval_ptr_dtor(retval);
+
     if (zobject)
     {
         zval_ptr_dtor(zobject);
     }
 
-    if (EG(exception))
+    if (UNEXPECTED(EG(exception)))
     {
         zend_exception_error(EG(exception), E_ERROR);
     }
@@ -341,7 +370,7 @@ void sw_coro_check_bind(const char *name, long bind_cid)
         sw_get_debug_print_backtrace(buffer, DEBUG_BACKTRACE_IGNORE_ARGS, 3);
         swoole_error_log(
             SW_LOG_ERROR, SW_ERROR_CO_HAS_BEEN_BOUND,
-            "%s has already been bound to another coroutine #%ld, "
+            "%s has already been bound to another coroutine#%ld, "
             "reading or writing of the same socket in multiple coroutines at the same time is not allowed.\n"
             "%.*s", name, bind_cid, (int) buffer->length, buffer->str
         );
@@ -349,19 +378,22 @@ void sw_coro_check_bind(const char *name, long bind_cid)
     }
 }
 
-long sw_coro_create(zend_fcall_info_cache *fci_cache, int argc, zval *argv, zval *retval)
+long sw_coro_create(zend_fcall_info_cache *fci_cache, int argc, zval *argv)
 {
+    zend_uchar type;
     if (unlikely(COROG.active == 0))
     {
         if (zend_get_module_started("xdebug") == SUCCESS)
         {
             swoole_php_fatal_error(E_WARNING, "Using Xdebug in coroutines is extremely dangerous, please notice that it may lead to coredump!");
         }
+        php_swoole_check_reactor();
+        // sw_enable_coroutine_hook(SW_HOOK_ALL); // TODO: enable it in version 4.3.0
         COROG.active = 1;
     }
-    if (unlikely(COROG.coro_num >= COROG.max_coro_num))
+    if (unlikely(swCoroG.count() >= COROG.max_coro_num))
     {
-        swoole_php_fatal_error(E_WARNING, "exceed max number of coroutine %" PRIu64 ".", COROG.coro_num);
+        swoole_php_fatal_error(E_WARNING, "exceed max number of coroutine %zu.", (uintmax_t) swCoroG.count());
         return CORO_LIMIT;
     }
     if (unlikely(!fci_cache || !fci_cache->function_handler))
@@ -369,7 +401,8 @@ long sw_coro_create(zend_fcall_info_cache *fci_cache, int argc, zval *argv, zval
         swoole_php_fatal_error(E_ERROR, "invalid function call info cache.");
         return CORO_INVALID;
     }
-    if (unlikely(fci_cache->function_handler->type != ZEND_USER_FUNCTION))
+    type = fci_cache->function_handler->type;
+    if (unlikely(type != ZEND_USER_FUNCTION && type != ZEND_INTERNAL_FUNCTION))
     {
         swoole_php_fatal_error(E_ERROR, "invalid function type %u.", fci_cache->function_handler->type);
         return CORO_INVALID;
@@ -379,7 +412,6 @@ long sw_coro_create(zend_fcall_info_cache *fci_cache, int argc, zval *argv, zval
     php_args.fci_cache = fci_cache;
     php_args.argv = argv;
     php_args.argc = argc;
-    php_args.retval = retval;
     php_args.origin_task = php_coro_get_current_task();
 
     return Coroutine::create(php_coro_create, (void*) &php_args);
@@ -388,10 +420,6 @@ long sw_coro_create(zend_fcall_info_cache *fci_cache, int argc, zval *argv, zval
 void sw_coro_save(zval *return_value, php_context *sw_current_context)
 {
     SWCC(current_coro_return_value_ptr) = return_value;
-    SWCC(current_execute_data) = EG(current_execute_data);
-    SWCC(current_vm_stack) = EG(vm_stack);
-    SWCC(current_vm_stack_top) = EG(vm_stack_top);
-    SWCC(current_vm_stack_end) = EG(vm_stack_end);
     SWCC(current_task) = php_coro_get_current_task();
 }
 
@@ -417,7 +445,7 @@ int sw_coro_resume(php_context *sw_current_context, zval *retval, zval *coro_ret
 
     task->co->resume_naked();
 
-    if (unlikely(EG(exception)))
+    if (UNEXPECTED(EG(exception)))
     {
         if (retval)
         {
@@ -443,11 +471,10 @@ void sw_coro_close()
 
     php_coro_close(task);
     php_vm_stack_destroy(task->vm_stack);
-    COROG.coro_num--;
 
     swTraceLog(
-        SW_TRACE_COROUTINE, "coro close cid=%ld and resume to %ld, %" PRIu64 " remained. usage size: %zu. malloc size: %zu",
-        cid, origin_cid, COROG.coro_num, zend_memory_usage(0), zend_memory_usage(1)
+        SW_TRACE_COROUTINE, "coro close cid=%ld and resume to %ld, %zu remained. usage size: %zu. malloc size: %zu",
+        cid, origin_cid, (uintmax_t) swCoroG.count(), (uintmax_t) zend_memory_usage(0), (uintmax_t) zend_memory_usage(1)
     );
 }
 
