@@ -42,7 +42,6 @@ typedef struct
 
 typedef struct
 {
-    std::unordered_map<long, Coroutine*> *_map;
     std::unordered_map<long, Coroutine*>::iterator _cursor;
     int index;
 } coroutine_iterator;
@@ -127,7 +126,7 @@ static PHP_METHOD(swoole_coroutine_util, set);
 static PHP_METHOD(swoole_coroutine_util, yield);
 static PHP_METHOD(swoole_coroutine_util, resume);
 static PHP_METHOD(swoole_coroutine_util, stats);
-static PHP_METHOD(swoole_coroutine_util, getuid);
+static PHP_METHOD(swoole_coroutine_util, getCid);
 static PHP_METHOD(swoole_coroutine_util, listCoroutines);
 static PHP_METHOD(swoole_coroutine_util, sleep);
 static PHP_METHOD(swoole_coroutine_util, fread);
@@ -153,13 +152,16 @@ static PHP_METHOD(swoole_exit_exception, getStatus);
 static std::unordered_map<int, Coroutine *> user_yield_coros;
 
 static zend_class_entry swoole_coroutine_util_ce;
-static zend_class_entry *swoole_coroutine_util_class_entry_ptr;
+static zend_class_entry *swoole_coroutine_util_ce_ptr;
+static zend_object_handlers swoole_coroutine_util_handlers;
 
 static zend_class_entry swoole_coroutine_iterator_ce;
-static zend_class_entry *swoole_coroutine_iterator_class_entry_ptr;
+static zend_class_entry *swoole_coroutine_iterator_ce_ptr;
+static zend_object_handlers swoole_coroutine_iterator_handlers;
 
 static zend_class_entry swoole_exit_exception_ce;
-static zend_class_entry *swoole_exit_exception_class_entry_ptr;
+static zend_class_entry *swoole_exit_exception_ce_ptr;
+static zend_object_handlers swoole_exit_exception_handlers;
 
 BEGIN_EXTERN_C()
 extern int swoole_coroutine_statvfs(const char *path, struct statvfs *buf);
@@ -176,7 +178,8 @@ static const zend_function_entry swoole_coroutine_util_methods[] =
     PHP_MALIAS(swoole_coroutine_util, suspend, yield, arginfo_swoole_coroutine_void, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_ME(swoole_coroutine_util, resume, arginfo_swoole_coroutine_resume, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_ME(swoole_coroutine_util, stats, arginfo_swoole_coroutine_void, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-    PHP_ME(swoole_coroutine_util, getuid, arginfo_swoole_coroutine_void, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
+    PHP_ME(swoole_coroutine_util, getCid, arginfo_swoole_coroutine_void, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
+    PHP_MALIAS(swoole_coroutine_util, getuid, getCid, arginfo_swoole_coroutine_void, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_ME(swoole_coroutine_util, sleep, arginfo_swoole_coroutine_sleep, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_ME(swoole_coroutine_util, fread, arginfo_swoole_coroutine_fread, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
     PHP_ME(swoole_coroutine_util, fgets, arginfo_swoole_coroutine_fgets, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
@@ -230,7 +233,7 @@ static int coro_exit_handler(zend_execute_data *execute_data)
     {
         flags |= SW_EXIT_IN_SERVER;
     }
-    if (flags == SW_EXIT_IN_COROUTINE && COROG.coro_num == 1)
+    if (flags == SW_EXIT_IN_COROUTINE && swCoroG.count() == 1)
     {
         php_swoole_event_exit();
     }
@@ -267,11 +270,11 @@ static int coro_exit_handler(zend_execute_data *execute_data)
             exit_status = &_exit_status;
             ZVAL_NULL(exit_status);
         }
-        obj = zend_throw_error_exception(swoole_exit_exception_class_entry_ptr, "swoole exit.", 0, E_ERROR);
+        obj = zend_throw_error_exception(swoole_exit_exception_ce_ptr, "swoole exit.", 0, E_ERROR);
         ZVAL_OBJ(&ex, obj);
-        zend_update_property_long(swoole_exit_exception_class_entry_ptr, &ex, ZEND_STRL("flags"), flags);
+        zend_update_property_long(swoole_exit_exception_ce_ptr, &ex, ZEND_STRL("flags"), flags);
         Z_TRY_ADDREF_P(exit_status);
-        zend_update_property(swoole_exit_exception_class_entry_ptr, &ex, ZEND_STRL("status"), exit_status);
+        zend_update_property(swoole_exit_exception_ce_ptr, &ex, ZEND_STRL("status"), exit_status);
     }
 
     return ZEND_USER_OPCODE_DISPATCH;
@@ -281,30 +284,20 @@ void swoole_coroutine_util_init(int module_number)
 {
     coro_init();
 
-    SWOOLE_INIT_CLASS_ENTRY(swoole_coroutine_util_ce, "swoole_coroutine", "Swoole\\Coroutine", swoole_coroutine_util_methods);
-    swoole_coroutine_util_class_entry_ptr = zend_register_internal_class(&swoole_coroutine_util_ce);
+    SWOOLE_INIT_CLASS_ENTRY(swoole_coroutine_util, "Swoole\\Coroutine", "swoole_coroutine", "Co", swoole_coroutine_util_methods);
+    SWOOLE_SET_CLASS_SERIALIZABLE(swoole_coroutine_util, zend_class_serialize_deny, zend_class_unserialize_deny);
+    SWOOLE_SET_CLASS_CLONEABLE(swoole_coroutine_util, zend_class_clone_deny);
+    SWOOLE_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_coroutine_util, zend_class_unset_property_deny);
 
-    INIT_CLASS_ENTRY(swoole_coroutine_iterator_ce, "Swoole\\Coroutine\\Iterator", iterator_methods);
-    swoole_coroutine_iterator_class_entry_ptr = zend_register_internal_class(&swoole_coroutine_iterator_ce);
-    zend_class_implements(swoole_coroutine_iterator_class_entry_ptr, 1, zend_ce_iterator);
+    SWOOLE_INIT_CLASS_ENTRY(swoole_coroutine_iterator, "Swoole\\Coroutine\\Iterator", NULL, "Co\\Iterator", iterator_methods);
+    SWOOLE_SET_CLASS_SERIALIZABLE(swoole_coroutine_iterator, zend_class_serialize_deny, zend_class_unserialize_deny);
+    SWOOLE_SET_CLASS_CLONEABLE(swoole_coroutine_iterator, zend_class_clone_deny);
+    SWOOLE_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_coroutine_iterator, zend_class_unset_property_deny);
+    zend_class_implements(swoole_coroutine_iterator_ce_ptr, 1, zend_ce_iterator);
 #ifdef SW_HAVE_COUNTABLE
-    zend_class_implements(swoole_coroutine_iterator_class_entry_ptr, 1, zend_ce_countable);
+    zend_class_implements(swoole_coroutine_iterator_ce_ptr, 1, zend_ce_countable);
 #endif
 
-    if (SWOOLE_G(use_namespace))
-    {
-        sw_zend_register_class_alias("swoole_coroutine", swoole_coroutine_util_class_entry_ptr);
-    }
-    else
-    {
-        sw_zend_register_class_alias("Swoole\\Coroutine", swoole_coroutine_util_class_entry_ptr);
-    }
-
-    if (SWOOLE_G(use_shortname))
-    {
-        sw_zend_register_class_alias("Co", swoole_coroutine_util_class_entry_ptr);
-    }
-
     SWOOLE_DEFINE(DEFAULT_MAX_CORO_NUM);
     SWOOLE_DEFINE(MAX_CORO_NUM_LIMIT);
     SWOOLE_DEFINE(MAX_CORO_NESTING_LEVEL);
@@ -314,8 +307,8 @@ void swoole_coroutine_util_init(int module_number)
     SWOOLE_DEFINE(CORO_END);
 
     //prohibit exit in coroutine
-    INIT_CLASS_ENTRY(swoole_exit_exception_ce, "Swoole\\ExitException", swoole_exit_exception_methods);
-    swoole_exit_exception_class_entry_ptr = zend_register_internal_class_ex(&swoole_exit_exception_ce, zend_exception_get_default());
+    SWOOLE_INIT_CLASS_ENTRY_EX(swoole_exit_exception, "Swoole\\ExitException", NULL, NULL, swoole_exit_exception_methods, swoole_exception);
+
     SWOOLE_DEFINE(EXIT_IN_COROUTINE);
     SWOOLE_DEFINE(EXIT_IN_SERVER);
 
@@ -368,14 +361,10 @@ static PHP_METHOD(swoole_coroutine_util, set)
     if (php_swoole_array_get_value(vht, "max_coroutine", v))
     {
         convert_to_long(v);
-        COROG.max_coro_num = (int) Z_LVAL_P(v);
+        COROG.max_coro_num = (uint64_t) Z_LVAL_P(v);
         if (COROG.max_coro_num <= 0)
         {
-            COROG.max_coro_num = DEFAULT_MAX_CORO_NUM;
-        }
-        else if (COROG.max_coro_num >= MAX_CORO_NUM_LIMIT)
-        {
-            COROG.max_coro_num = MAX_CORO_NUM_LIMIT;
+            COROG.max_coro_num = SW_DEFAULT_MAX_CORO_NUM;
         }
     }
     if (php_swoole_array_get_value(vht, "stack_size", v))
@@ -384,15 +373,33 @@ static PHP_METHOD(swoole_coroutine_util, set)
         COROG.stack_size = (uint32_t) Z_LVAL_P(v);
         sw_coro_set_stack_size(COROG.stack_size);
     }
+    if (php_swoole_array_get_value(vht, "socket_connect_timeout", v))
+    {
+        convert_to_double(v);
+        if (Z_DVAL_P(v))
+        {
+            COROG.socket_connect_timeout = (double) Z_DVAL_P(v);
+        }
+    }
+    if (php_swoole_array_get_value(vht, "socket_timeout", v))
+    {
+        convert_to_double(v);
+        if (Z_DVAL_P(v))
+        {
+            COROG.socket_timeout = (double) Z_DVAL_P(v);
+        }
+    }
     if (php_swoole_array_get_value(vht, "log_level", v))
     {
+        zend_long level;
         convert_to_long(v);
-        SwooleG.log_level = (int32_t) Z_LVAL_P(v);
+        level = Z_LVAL_P(v);
+        SwooleG.log_level = (uint32_t) (level < 0 ? UINT32_MAX : level);
     }
     if (php_swoole_array_get_value(vht, "trace_flags", v))
     {
         convert_to_long(v);
-        SwooleG.trace_flags = (int32_t) Z_LVAL_P(v);
+        SwooleG.trace_flags = (uint32_t) MAX(0, Z_LVAL_P(v));
     }
     zval_ptr_dtor(zset);
 }
@@ -417,18 +424,7 @@ PHP_FUNCTION(swoole_coroutine_create)
         }
     }
 
-    php_swoole_check_reactor();
-
-    zval *retval = NULL;
-    long cid = sw_coro_create(&fci_cache, fci.param_count, fci.params, retval);
-    if (EG(exception))
-    {
-        zend_exception_error(EG(exception), E_ERROR);
-    }
-    if (retval != NULL)
-    {
-        zval_ptr_dtor(retval);
-    }
+    long cid = sw_coro_create(&fci_cache, fci.param_count, fci.params);
     if (likely(cid > 0))
     {
         RETURN_LONG(cid);
@@ -464,18 +460,18 @@ static PHP_METHOD(swoole_coroutine_util, stats)
 {
     array_init(return_value);
     add_assoc_long_ex(return_value, ZEND_STRL("stack_size"), COROG.stack_size);
-    add_assoc_long_ex(return_value, ZEND_STRL("coroutine_num"), COROG.coro_num);
+    add_assoc_long_ex(return_value, ZEND_STRL("coroutine_num"), swCoroG.count());
     add_assoc_long_ex(return_value, ZEND_STRL("coroutine_peak_num"), COROG.peak_coro_num);
 }
 
-static PHP_METHOD(swoole_coroutine_util, getuid)
+static PHP_METHOD(swoole_coroutine_util, getCid)
 {
     RETURN_LONG(sw_get_current_cid());
 }
 
 int php_coroutine_reactor_can_exit(swReactor *reactor)
 {
-    return COROG.coro_num == 0;
+    return swCoroG.count() == 0;
 }
 
 static PHP_METHOD(swoole_coroutine_util, sleep)
@@ -495,11 +491,6 @@ static PHP_METHOD(swoole_coroutine_util, sleep)
         swoole_php_fatal_error(E_WARNING, "cannot use timer in master process.");
         return;
     }
-    if (ms > SW_TIMER_MAX_VALUE)
-    {
-        swoole_php_fatal_error(E_WARNING, "The given parameters is too big.");
-        return;
-    }
     if (ms <= 0)
     {
         swoole_php_fatal_error(E_WARNING, "Timer must be greater than 0");
@@ -1325,14 +1316,14 @@ static PHP_METHOD(swoole_coroutine_util, getBackTrace)
 static PHP_METHOD(swoole_coroutine_iterator, rewind)
 {
     coroutine_iterator *itearator = (coroutine_iterator *) swoole_get_object(getThis());
-    itearator->_cursor = itearator->_map->begin();
+    itearator->_cursor = swCoroG.coroutines.begin();
     itearator->index = 0;
 }
 
 static PHP_METHOD(swoole_coroutine_iterator, valid)
 {
     coroutine_iterator *itearator = (coroutine_iterator *) swoole_get_object(getThis());
-    RETURN_BOOL(itearator->_cursor != itearator->_map->end());
+    RETURN_BOOL(itearator->_cursor != swCoroG.coroutines.end());
 }
 
 static PHP_METHOD(swoole_coroutine_iterator, current)
@@ -1357,7 +1348,7 @@ PHP_METHOD(swoole_coroutine_iterator, key)
 
 static PHP_METHOD(swoole_coroutine_iterator, count)
 {
-    RETURN_LONG(COROG.coro_num);
+    RETURN_LONG(swCoroG.count());
 }
 
 static PHP_METHOD(swoole_coroutine_iterator, __destruct)
@@ -1369,10 +1360,9 @@ static PHP_METHOD(swoole_coroutine_iterator, __destruct)
 
 static PHP_METHOD(swoole_coroutine_util, listCoroutines)
 {
-    object_init_ex(return_value, swoole_coroutine_iterator_class_entry_ptr);
+    object_init_ex(return_value, swoole_coroutine_iterator_ce_ptr);
     coroutine_iterator *itearator = (coroutine_iterator *) emalloc(sizeof(coroutine_iterator));
     bzero(itearator, sizeof(coroutine_iterator));
-    itearator->_map = coroutine_get_map();
     swoole_set_object(return_value, itearator);
 }
 
@@ -1490,29 +1480,38 @@ PHP_FUNCTION(swoole_coroutine_exec)
     swString_free(buffer);
 }
 
+static void coro_onDefer(void *data)
+{
+    php_defer_fci *defer_fci = (php_defer_fci *) data;
+    zval _retval, *retval = &_retval;
+
+    defer_fci->fci.retval = retval;
+    if (sw_call_function_anyway(&defer_fci->fci, &defer_fci->fci_cache) == FAILURE)
+    {
+        swoole_php_fatal_error(E_WARNING, "defer callback handler error.");
+        return;
+    }
+    zval_ptr_dtor(retval);
+    sw_fci_cache_discard(&defer_fci->fci_cache);
+    efree(defer_fci);
+}
+
 PHP_FUNCTION(swoole_coroutine_defer)
 {
+    zend_fcall_info fci = empty_fcall_info;
+    zend_fcall_info_cache fci_cache = empty_fcall_info_cache;
+    php_defer_fci *defer_fci;
+
     coro_check();
 
-    zval *callback;
-    if (zend_parse_parameters(ZEND_NUM_ARGS(), "z", &callback) == FAILURE)
-    {
-        RETURN_FALSE;
-    }
+    ZEND_PARSE_PARAMETERS_START(1, -1)
+        Z_PARAM_FUNC(fci, fci_cache)
+        Z_PARAM_VARIADIC('*', fci.params, fci.param_count)
+    ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
-    char *func_name;
-    if (!sw_zend_is_callable(callback, 0, &func_name))
-    {
-        swoole_php_fatal_error(E_ERROR, "function '%s' is not callable", func_name);
-        efree(func_name);
-        RETURN_FALSE;
-    }
-    efree(func_name);
-
-    php_defer_callback *defer = (php_defer_callback *) emalloc(sizeof(php_defer_callback));
-    defer->callback = &defer->_callback;
-    memcpy(defer->callback, callback, sizeof(zval));
-    Z_TRY_ADDREF_P(callback);
-
-    sw_coro_add_defer_task(php_swoole_event_onDefer, defer);
+    defer_fci = (php_defer_fci *) emalloc(sizeof(php_defer_fci));
+    defer_fci->fci = fci;
+    defer_fci->fci_cache = fci_cache;
+    sw_fci_cache_persist(&defer_fci->fci_cache);
+    sw_coro_add_defer_task(coro_onDefer, defer_fci);
 }
