@@ -66,7 +66,7 @@ static void free_callback(void* data)
 
 static int php_swoole_event_onRead(swReactor *reactor, swEvent *event)
 {
-    zval *retval;
+    zval *retval = NULL;
     zval args[1];
     php_reactor_fd *fd = event->socket->object;
 
@@ -79,11 +79,11 @@ static int php_swoole_event_onRead(swReactor *reactor, swEvent *event)
         SwooleG.main_reactor->del(SwooleG.main_reactor, event->fd);
         return SW_ERR;
     }
-    if (EG(exception))
+    if (UNEXPECTED(EG(exception)))
     {
         zend_exception_error(EG(exception), E_ERROR);
     }
-    if (retval != NULL)
+    if (retval)
     {
         zval_ptr_dtor(retval);
     }
@@ -92,7 +92,7 @@ static int php_swoole_event_onRead(swReactor *reactor, swEvent *event)
 
 static int php_swoole_event_onWrite(swReactor *reactor, swEvent *event)
 {
-    zval *retval;
+    zval *retval = NULL;
     zval args[1];
     php_reactor_fd *fd = event->socket->object;
 
@@ -110,11 +110,11 @@ static int php_swoole_event_onWrite(swReactor *reactor, swEvent *event)
         SwooleG.main_reactor->del(SwooleG.main_reactor, event->fd);
         return SW_ERR;
     }
-    if (EG(exception))
+    if (UNEXPECTED(EG(exception)))
     {
         zend_exception_error(EG(exception), E_ERROR);
     }
-    if (retval != NULL)
+    if (retval)
     {
         zval_ptr_dtor(retval);
     }
@@ -147,18 +147,18 @@ static int php_swoole_event_onError(swReactor *reactor, swEvent *event)
 void php_swoole_event_onDefer(void *_cb)
 {
     php_defer_callback *defer = _cb;
-    zval *retval;
+    zval _retval, *retval = &_retval;
 
-    if (sw_call_user_function_anyway(NULL, defer->callback, &retval, 0, NULL, 0) == FAILURE)
+    if (sw_call_user_function_ex(EG(function_table), NULL, defer->callback, &retval, 0, NULL, 0, NULL) == FAILURE)
     {
-        swoole_php_fatal_error(E_WARNING, "defer callback handler error.");
+        swoole_php_fatal_error(E_WARNING, "swoole_event: defer handler error");
         return;
     }
-    if (EG(exception))
+    if (UNEXPECTED(EG(exception)))
     {
         zend_exception_error(EG(exception), E_ERROR);
     }
-    if (retval != NULL)
+    if (retval)
     {
         zval_ptr_dtor(retval);
     }
@@ -169,18 +169,18 @@ void php_swoole_event_onDefer(void *_cb)
 static void php_swoole_event_onEndCallback(void *_cb)
 {
     php_defer_callback *defer = _cb;
-    zval *retval;
+    zval *retval = NULL;
 
     if (sw_call_user_function_ex(EG(function_table), NULL, defer->callback, &retval, 0, NULL, 0, NULL) == FAILURE)
     {
         swoole_php_fatal_error(E_WARNING, "swoole_event: cycle callback handler error.");
         return;
     }
-    if (EG(exception))
+    if (UNEXPECTED(EG(exception)))
     {
         zend_exception_error(EG(exception), E_ERROR);
     }
-    if (retval != NULL)
+    if (retval)
     {
         zval_ptr_dtor(retval);
     }
@@ -216,16 +216,14 @@ void php_swoole_reactor_init()
             return;
         }
 
-#ifdef SW_COROUTINE
         SwooleG.main_reactor->can_exit = php_coroutine_reactor_can_exit;
-#endif
 
         //client, swoole_event_exit will set swoole_running = 0
         SwooleWG.in_client = 1;
         SwooleWG.reactor_wait_onexit = 1;
         SwooleWG.reactor_ready = 0;
         //only client side
-        php_swoole_at_shutdown("swoole_event_wait");
+        php_swoole_register_shutdown_function_prepend("swoole_event_wait");
     }
 
     SwooleG.main_reactor->setHandle(SwooleG.main_reactor, SW_FD_USER | SW_EVENT_READ, php_swoole_event_onRead);
@@ -272,11 +270,10 @@ void php_swoole_event_wait()
             }
             SW_SET_EG_SCOPE(scope);
         }
-        if (SwooleG.timer.map)
-        {
-            php_swoole_clear_all_timer();
-        }
+        php_swoole_clear_all_timer();
         SwooleWG.reactor_exit = 1;
+        SwooleG.running = 0;
+        SwooleG.main_reactor->running = 0;
     }
 }
 
@@ -340,11 +337,11 @@ int swoole_convert_to_fd(zval *zfd)
     else if (Z_TYPE_P(zfd) == IS_OBJECT)
     {
         zval *zsock = NULL;
-        if (instanceof_function(Z_OBJCE_P(zfd), swoole_client_class_entry_ptr))
+        if (instanceof_function(Z_OBJCE_P(zfd), swoole_client_ce_ptr))
         {
             zsock = sw_zend_read_property(Z_OBJCE_P(zfd), zfd, ZEND_STRL("sock"), 0);
         }
-        else if (instanceof_function(Z_OBJCE_P(zfd), swoole_process_class_entry_ptr))
+        else if (instanceof_function(Z_OBJCE_P(zfd), swoole_process_ce_ptr))
         {
             zsock = sw_zend_read_property(Z_OBJCE_P(zfd), zfd, ZEND_STRL("pipe"), 0);
         }
@@ -473,7 +470,6 @@ PHP_FUNCTION(swoole_event_add)
         if (!sw_zend_is_callable(cb_read, 0, &func_name))
         {
             swoole_php_fatal_error(E_ERROR, "function '%s' is not callable", func_name);
-            efree(func_name);
             RETURN_FALSE;
         }
         efree(func_name);
@@ -491,7 +487,6 @@ PHP_FUNCTION(swoole_event_add)
         if (!sw_zend_is_callable(cb_write, 0, &func_name))
         {
             swoole_php_fatal_error(E_ERROR, "function '%s' is not callable", func_name);
-            efree(func_name);
             RETURN_FALSE;
         }
         efree(func_name);
@@ -596,7 +591,6 @@ PHP_FUNCTION(swoole_event_set)
         if (!sw_zend_is_callable(cb_read, 0, &func_name))
         {
             swoole_php_fatal_error(E_ERROR, "function '%s' is not callable", func_name);
-            efree(func_name);
             RETURN_FALSE;
         }
         else
@@ -622,7 +616,6 @@ PHP_FUNCTION(swoole_event_set)
         if (!sw_zend_is_callable(cb_write, 0, &func_name))
         {
             swoole_php_fatal_error(E_ERROR, "function '%s' is not callable", func_name);
-            efree(func_name);
             RETURN_FALSE;
         }
         else
@@ -705,7 +698,6 @@ PHP_FUNCTION(swoole_event_defer)
     if (!sw_zend_is_callable(callback, 0, &func_name))
     {
         swoole_php_fatal_error(E_ERROR, "function '%s' is not callable", func_name);
-        efree(func_name);
         RETURN_FALSE;
     }
     efree(func_name);
@@ -754,7 +746,6 @@ PHP_FUNCTION(swoole_event_cycle)
     if (!sw_zend_is_callable(callback, 0, &func_name))
     {
         swoole_php_fatal_error(E_ERROR, "function '%s' is not callable", func_name);
-        efree(func_name);
         RETURN_FALSE;
     }
     efree(func_name);
