@@ -18,6 +18,100 @@
 #include "php_swoole.h"
 #include "swoole_http_client.h"
 
+typedef struct
+{
+    zval *onConnect;
+    zval *onError;
+    zval *onClose;
+    zval *onMessage;
+    zval *onResponse;
+
+    zval _object;
+    zval _request_body;
+    zval _request_header;
+    zval _request_upload_files;
+    zval _download_file;
+    zval _cookies;
+    zval _onConnect;
+    zval _onError;
+    zval _onClose;
+    zval _onMessage;
+
+    zval *cookies;
+    zval *request_header;
+    zval *request_body;
+    zval *request_upload_files;
+    zval *download_file;
+    off_t download_offset;
+    char *request_method;
+    int callback_index;
+
+    uint8_t error_flag;
+    uint8_t shutdown;
+
+#ifdef SW_COROUTINE
+    zend_bool defer; //0 normal 1 wait for receive
+    zend_bool defer_result;//0
+    zend_bool defer_chunk_status;// 0 1 now use rango http->complete
+    zend_bool send_yield;
+    http_client_defer_state defer_status;
+    long cid;
+    /**
+     * for websocket
+     */
+    swLinkedList *message_queue;
+#endif
+
+} http_client_property;
+
+typedef struct
+{
+    swClient *cli;
+    char *host;
+    size_t host_len;
+    long port;
+#ifdef SW_USE_OPENSSL
+    uint8_t ssl;
+#endif
+    double connect_timeout;
+    double timeout;
+    char* uri;
+    size_t uri_len;
+
+    swTimer_node *timer;
+
+    char *tmp_header_field_name;
+    int tmp_header_field_name_len;
+
+#ifdef SW_HAVE_ZLIB
+    z_stream gzip_stream;
+    swString *gzip_buffer;
+#endif
+
+    /**
+     * download page
+     */
+    int file_fd;
+
+    swoole_http_parser parser;
+
+    zval _object;
+    zval *object;
+    swString *body;
+
+    uint8_t state;       //0 wait 1 ready 2 busy
+    uint8_t keep_alive;  //0 no 1 keep
+    uint8_t upgrade;     //if upgrade successfully
+    uint8_t gzip;
+    uint8_t chunked;     //Transfer-Encoding: chunked
+    uint8_t completed;
+    uint8_t websocket_mask;
+    uint8_t download;    //save http response to file
+    uint8_t header_completed;
+    int8_t method;
+
+} http_client;
+
 swString *http_client_buffer;
 
 static void http_client_onReceive(swClient *cli, char *data, uint32_t length);
@@ -32,14 +126,25 @@ static int http_client_send_http_request(zval *zobject);
 static int http_client_execute(zval *zobject, char *uri, size_t uri_len, zval *callback);
 
 #ifdef SW_HAVE_ZLIB
-static void http_init_gzip_stream(http_client *);
 BEGIN_EXTERN_C()
-extern voidpf php_zlib_alloc(voidpf opaque, uInt items, uInt size);
-extern void php_zlib_free(voidpf opaque, voidpf address);
+static void http_init_gzip_stream(http_client*);
 extern int http_response_uncompress(z_stream *stream, swString *buffer, char *body, int length);
 END_EXTERN_C()
 #endif
 
+static void http_client_clear_response_properties(zval *zobject);
+static int http_client_parser_on_header_field(swoole_http_parser *parser, const char *at, size_t length);
+static int http_client_parser_on_header_value(swoole_http_parser *parser, const char *at, size_t length);
+static int http_client_parser_on_body(swoole_http_parser *parser, const char *at, size_t length);
+static int http_client_parser_on_headers_complete(swoole_http_parser *parser);
+static int http_client_parser_on_message_complete(swoole_http_parser *parser);
+
+static http_client* http_client_create(zval *zobject);
+static void http_client_clear(http_client *http);
+static int http_client_check_keep(http_client *http);
+static void http_client_reset(http_client *http);
+static uint8_t http_client_free(zval *zobject);
+
 static const swoole_http_parser_settings http_parser_settings =
 {
     NULL,
@@ -55,7 +160,8 @@ static const swoole_http_parser_settings http_parser_settings =
 };
 
 static zend_class_entry swoole_http_client_ce;
-static zend_class_entry *swoole_http_client_class_entry_ptr;
+static zend_class_entry *swoole_http_client_ce_ptr;
+static zend_object_handlers swoole_http_client_handlers;
 
 static PHP_METHOD(swoole_http_client, __construct);
 static PHP_METHOD(swoole_http_client, __destruct);
@@ -174,11 +280,12 @@ static const zend_function_entry swoole_http_client_methods[] =
     PHP_FE_END
 };
 
-void http_client_clear_response_properties(zval *zobject)
+static void http_client_clear_response_properties(zval *zobject)
 {
     zval *zattr;
     zend_class_entry *ce = Z_OBJCE_P(zobject);
     zend_update_property_long(ce, zobject, ZEND_STRL("errCode"), 0);
+    zend_update_property_string(ce, zobject, ZEND_STRL("errMsg"), "");
     zend_update_property_long(ce, zobject, ZEND_STRL("statusCode"), 0);
     zattr = sw_zend_read_property(ce, zobject, ZEND_STRL("headers"), 1);
     if (Z_TYPE_P(zattr) == IS_ARRAY)
@@ -323,7 +430,7 @@ static int http_client_execute(zval *zobject, char *uri, size_t uri_len, zval *c
 
     zval *ztmp;
     HashTable *vht;
-    zval *zset = sw_zend_read_property(swoole_http_client_class_entry_ptr, zobject, ZEND_STRL("setting"), 1);
+    zval *zset = sw_zend_read_property(swoole_http_client_ce_ptr, zobject, ZEND_STRL("setting"), 1);
     if (zset && ZVAL_IS_ARRAY(zset))
     {
         vht = Z_ARRVAL_P(zset);
@@ -356,7 +463,7 @@ static int http_client_execute(zval *zobject, char *uri, size_t uri_len, zval *c
 
         if (http->cli->http_proxy)
         {
-            zval *zsend_header = sw_zend_read_property(swoole_http_client_class_entry_ptr, zobject, ZEND_STRL("requestHeaders"), 1);
+            zval *zsend_header = sw_zend_read_property(swoole_http_client_ce_ptr, zobject, ZEND_STRL("requestHeaders"), 1);
             if (zsend_header == NULL || Z_TYPE_P(zsend_header) != IS_ARRAY)
             {
                 swoole_php_fatal_error (E_WARNING, "http proxy must set Host");
@@ -407,30 +514,33 @@ static int http_client_execute(zval *zobject, char *uri, size_t uri_len, zval *c
 
 void swoole_http_client_init(int module_number)
 {
-    SWOOLE_INIT_CLASS_ENTRY(swoole_http_client_ce, "swoole_http_client", "Swoole\\Http\\Client", swoole_http_client_methods);
-    swoole_http_client_class_entry_ptr = zend_register_internal_class(&swoole_http_client_ce);
-    SWOOLE_CLASS_ALIAS(swoole_http_client, "Swoole\\Http\\Client");
+    SWOOLE_INIT_CLASS_ENTRY(swoole_http_client, "Swoole\\Http\\Client", "swoole_http_client", NULL, swoole_http_client_methods);
+    SWOOLE_SET_CLASS_SERIALIZABLE(swoole_http_client, zend_class_serialize_deny, zend_class_unserialize_deny);
+    SWOOLE_SET_CLASS_CLONEABLE(swoole_http_client, zend_class_clone_deny);
+    SWOOLE_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_http_client, zend_class_unset_property_deny);
 
-    zend_declare_property_long(swoole_http_client_class_entry_ptr, ZEND_STRL("type"), 0, ZEND_ACC_PUBLIC);
-    zend_declare_property_long(swoole_http_client_class_entry_ptr, ZEND_STRL("errCode"), 0, ZEND_ACC_PUBLIC);
-    zend_declare_property_long(swoole_http_client_class_entry_ptr, ZEND_STRL("statusCode"), 0, ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_http_client_class_entry_ptr, ZEND_STRL("host"), ZEND_ACC_PUBLIC);
-    zend_declare_property_long(swoole_http_client_class_entry_ptr, ZEND_STRL("port"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_http_client_ce_ptr, ZEND_STRL("type"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_http_client_ce_ptr, ZEND_STRL("errCode"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_string(swoole_http_client_ce_ptr, ZEND_STRL("errMsg"), "", ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_http_client_ce_ptr, ZEND_STRL("statusCode"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_client_ce_ptr, ZEND_STRL("host"), ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_http_client_ce_ptr, ZEND_STRL("port"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_bool(swoole_http_client_ce_ptr, ZEND_STRL("ssl"), 0, ZEND_ACC_PUBLIC);
 
-    zend_declare_property_null(swoole_http_client_class_entry_ptr, ZEND_STRL("requestMethod"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_http_client_class_entry_ptr, ZEND_STRL("requestHeaders"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_http_client_class_entry_ptr, ZEND_STRL("requestBody"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_http_client_class_entry_ptr, ZEND_STRL("uploadFiles"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_http_client_class_entry_ptr, ZEND_STRL("set_cookie_headers"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_http_client_class_entry_ptr, ZEND_STRL("downloadFile"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_http_client_class_entry_ptr, ZEND_STRL("headers"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_http_client_class_entry_ptr, ZEND_STRL("cookies"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_http_client_class_entry_ptr, ZEND_STRL("body"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_client_ce_ptr, ZEND_STRL("requestMethod"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_client_ce_ptr, ZEND_STRL("requestHeaders"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_client_ce_ptr, ZEND_STRL("requestBody"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_client_ce_ptr, ZEND_STRL("uploadFiles"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_client_ce_ptr, ZEND_STRL("set_cookie_headers"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_client_ce_ptr, ZEND_STRL("downloadFile"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_client_ce_ptr, ZEND_STRL("headers"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_client_ce_ptr, ZEND_STRL("cookies"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_client_ce_ptr, ZEND_STRL("body"), ZEND_ACC_PUBLIC);
 
-    zend_declare_property_null(swoole_http_client_class_entry_ptr, ZEND_STRL("onConnect"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_http_client_class_entry_ptr, ZEND_STRL("onError"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_http_client_class_entry_ptr, ZEND_STRL("onMessage"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_http_client_class_entry_ptr, ZEND_STRL("onClose"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_client_ce_ptr, ZEND_STRL("onConnect"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_client_ce_ptr, ZEND_STRL("onError"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_client_ce_ptr, ZEND_STRL("onMessage"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_client_ce_ptr, ZEND_STRL("onClose"), ZEND_ACC_PUBLIC);
 
     http_client_buffer = swString_new(SW_HTTP_RESPONSE_INIT_SIZE);
     if (!http_client_buffer)
@@ -484,7 +594,7 @@ static void http_client_execute_callback(zval *zobject, enum php_swoole_client_c
         int error_code;
         if (type == SW_CLIENT_CB_onError)
         {
-            error_code = HTTP_CLIENT_ESTATUS_CONNECT_TIMEOUT;
+            error_code = HTTP_CLIENT_ESTATUS_CONNECT_FAILED;
         }
         else if (hcc->error_flag & HTTP_CLIENT_EFLAG_TIMEOUT)
         {
@@ -495,8 +605,8 @@ static void http_client_execute_callback(zval *zobject, enum php_swoole_client_c
             error_code = HTTP_CLIENT_ESTATUS_SERVER_RESET;
         }
 
-        zend_update_property_long(swoole_http_client_class_entry_ptr, zobject, ZEND_STRL("statusCode"), error_code);
-        zend_update_property_string(swoole_http_client_class_entry_ptr, zobject, ZEND_STRL("body"), "");
+        zend_update_property_long(swoole_http_client_ce_ptr, zobject, ZEND_STRL("statusCode"), error_code);
+        zend_update_property_string(swoole_http_client_ce_ptr, zobject, ZEND_STRL("body"), "");
         http_client_onResponseException(zobject);
     }
     //callback function is not set
@@ -509,12 +619,12 @@ static void http_client_execute_callback(zval *zobject, enum php_swoole_client_c
     {
         swoole_php_fatal_error(E_WARNING, "swoole_http_client->%s handler error.", callback_name);
     }
-    if (EG(exception))
+    if (UNEXPECTED(EG(exception)))
     {
         zend_exception_error(EG(exception), E_ERROR);
     }
     //free the callback return value
-    if (retval != NULL)
+    if (retval)
     {
         zval_ptr_dtor(retval);
     }
@@ -547,7 +657,7 @@ static int http_client_onMessage(swConnection *conn, char *data, uint32_t length
     swClient *cli = conn->object;
     zval *zobject = cli->object;
     zval args[2];
-    zval *retval;
+    zval *retval = NULL;
 
     zval *zframe;
     SW_MAKE_STD_ZVAL(zframe);
@@ -562,12 +672,12 @@ static int http_client_onMessage(swConnection *conn, char *data, uint32_t length
     {
         swoole_php_fatal_error(E_ERROR, "swoole_http_client->onMessage: onClose handler error");
     }
-    if (EG(exception))
+    if (UNEXPECTED(EG(exception)))
     {
         zend_exception_error(EG(exception), E_ERROR);
     }
     //free the callback return value
-    if (retval != NULL)
+    if (retval)
     {
         zval_ptr_dtor(retval);
     }
@@ -582,7 +692,7 @@ static int http_client_onMessage(swConnection *conn, char *data, uint32_t length
 static void http_client_onError(swClient *cli)
 {
     zval *zobject = cli->object;
-    zend_update_property_long(swoole_http_client_class_entry_ptr, zobject, ZEND_STRL("errCode"), SwooleG.error);
+    zend_update_property_long(swoole_http_client_ce_ptr, zobject, ZEND_STRL("errCode"), SwooleG.error);
     http_client_free(zobject);
     http_client_execute_callback(zobject, SW_CLIENT_CB_onError);
     zval_ptr_dtor(zobject);
@@ -592,8 +702,8 @@ static void http_client_onRequestTimeout(swTimer *timer, swTimer_node *tnode)
 {
     swClient *cli = (swClient *) tnode->data;
     zval *zobject = (zval *) cli->object;
-    zend_update_property_long(swoole_http_client_class_entry_ptr, zobject, ZEND_STRL("errCode"), ETIMEDOUT);
-    zend_update_property_long(swoole_http_client_class_entry_ptr, zobject, ZEND_STRL("statusCode"), HTTP_CLIENT_ESTATUS_REQUEST_TIMEOUT);
+    zend_update_property_long(swoole_http_client_ce_ptr, zobject, ZEND_STRL("errCode"), ETIMEDOUT);
+    zend_update_property_long(swoole_http_client_ce_ptr, zobject, ZEND_STRL("statusCode"), HTTP_CLIENT_ESTATUS_REQUEST_TIMEOUT);
 
     http_client_property *hcc = swoole_get_property(zobject, 0);
     if (!hcc)
@@ -604,7 +714,7 @@ static void http_client_onRequestTimeout(swTimer *timer, swTimer_node *tnode)
 
     if (cli->buffer && cli->buffer->length > 0) // received something bug not complete
     {
-        zval *zheaders = sw_zend_read_property_array(swoole_http_client_class_entry_ptr, zobject, ZEND_STRL("requestHeaders"), 1);
+        zval *zheaders = sw_zend_read_property_array(swoole_http_client_ce_ptr, zobject, ZEND_STRL("requestHeaders"), 1);
         zval *v;
         if (php_swoole_array_get_value(Z_ARRVAL_P(zheaders), "Connection", v))
         {
@@ -617,7 +727,7 @@ static void http_client_onRequestTimeout(swTimer *timer, swTimer_node *tnode)
     }
 
     zval *retval = NULL;
-    sw_zend_call_method_with_0_params(&zobject, swoole_http_client_class_entry_ptr, NULL, "close", &retval);
+    sw_zend_call_method_with_0_params(&zobject, swoole_http_client_ce_ptr, NULL, "close", &retval);
     if (retval)
     {
         zval_ptr_dtor(retval);
@@ -645,7 +755,7 @@ static void http_client_onResponseException(zval *zobject)
     {
         swoole_php_fatal_error(E_WARNING, "onResponse handler error");
     }
-    if (EG(exception))
+    if (UNEXPECTED(EG(exception)))
     {
         zend_exception_error(EG(exception), E_ERROR);
     }
@@ -753,7 +863,7 @@ static void http_client_onReceive(swClient *cli, char *data, uint32_t length)
     {
         swoole_php_fatal_error(E_WARNING, "onReactorCallback handler error");
     }
-    if (EG(exception))
+    if (UNEXPECTED(EG(exception)))
     {
         zend_exception_error(EG(exception), E_ERROR);
     }
@@ -821,8 +931,8 @@ static int http_client_send_http_request(zval *zobject)
 
     http_client_property *hcc = swoole_get_property(zobject, 0);
 
-    zval *zpost_data = sw_zend_read_property(swoole_http_client_class_entry_ptr, zobject, ZEND_STRL("requestBody"), 1);
-    zval *zsend_header = sw_zend_read_property(swoole_http_client_class_entry_ptr, zobject, ZEND_STRL("requestHeaders"), 1);
+    zval *zpost_data = sw_zend_read_property(swoole_http_client_ce_ptr, zobject, ZEND_STRL("requestBody"), 1);
+    zval *zsend_header = sw_zend_read_property(swoole_http_client_ce_ptr, zobject, ZEND_STRL("requestHeaders"), 1);
     uint32_t header_flag = 0x0;
 
     //POST
@@ -932,7 +1042,7 @@ static int http_client_send_http_request(zval *zobject)
     }
 #endif
 
-    zval *zcookies = sw_zend_read_property(swoole_http_client_class_entry_ptr, zobject, ZEND_STRL("cookies"), 1);
+    zval *zcookies = sw_zend_read_property(swoole_http_client_ce_ptr, zobject, ZEND_STRL("cookies"), 1);
     if (zcookies && Z_TYPE_P(zcookies) == IS_ARRAY)
     {
         swString_append_ptr(http_client_buffer, ZEND_STRL("Cookie: "));
@@ -1056,7 +1166,7 @@ static int http_client_send_http_request(zval *zobject)
             SW_HASHTABLE_FOREACH_END();
 
             //cleanup request body
-            zend_update_property_null(swoole_http_client_class_entry_ptr, zobject, ZEND_STRL("requestBody"));
+            zend_update_property_null(swoole_http_client_ce_ptr, zobject, ZEND_STRL("requestBody"));
         }
 
         if ((ret = http->cli->send(http->cli, http_client_buffer->str, http_client_buffer->length, 0)) < 0)
@@ -1111,7 +1221,7 @@ static int http_client_send_http_request(zval *zobject)
                 }
             SW_HASHTABLE_FOREACH_END();
 
-            zend_update_property_null(swoole_http_client_class_entry_ptr, zobject, ZEND_STRL("uploadFiles"));
+            zend_update_property_null(swoole_http_client_ce_ptr, zobject, ZEND_STRL("uploadFiles"));
             hcc->request_upload_files = NULL;
         }
 
@@ -1151,7 +1261,7 @@ static int http_client_send_http_request(zval *zobject)
                 goto send_fail;
             }
             //cleanup request body
-            zend_update_property_null(swoole_http_client_class_entry_ptr, zobject, ZEND_STRL("requestBody"));
+            zend_update_property_null(swoole_http_client_ce_ptr, zobject, ZEND_STRL("requestBody"));
         }
         else if (Z_TYPE_P(zpost_data) == IS_STRING && Z_STRLEN_P(zpost_data) > 0)
         {
@@ -1167,12 +1277,12 @@ static int http_client_send_http_request(zval *zobject)
                 goto send_fail;
             }
             //cleanup request body
-            zend_update_property_null(swoole_http_client_class_entry_ptr, zobject, ZEND_STRL("requestBody"));
+            zend_update_property_null(swoole_http_client_ce_ptr, zobject, ZEND_STRL("requestBody"));
         }
         else
         {
             //cleanup request body
-            zend_update_property_null(swoole_http_client_class_entry_ptr, zobject, ZEND_STRL("requestBody"));
+            zend_update_property_null(swoole_http_client_ce_ptr, zobject, ZEND_STRL("requestBody"));
             goto append_crlf;
         }
     }
@@ -1193,7 +1303,7 @@ static int http_client_send_http_request(zval *zobject)
             send_fail:
             SwooleG.error = errno;
             swoole_php_sys_error(E_WARNING, "send(%d) %d bytes failed.", http->cli->socket->fd, (int )http_client_buffer->length);
-            zend_update_property_long(swoole_http_client_class_entry_ptr, zobject, ZEND_STRL("errCode"), SwooleG.error);
+            zend_update_property_long(swoole_http_client_ce_ptr, zobject, ZEND_STRL("errCode"), SwooleG.error);
             return SW_ERR;
         }
     }
@@ -1208,7 +1318,7 @@ static int http_client_send_http_request(zval *zobject)
     return ret;
 }
 
-void http_client_clear(http_client *http)
+static void http_client_clear(http_client *http)
 {
     // clear timeout timer
     if (http->timer)
@@ -1240,7 +1350,7 @@ void http_client_clear(http_client *http)
 #endif
 }
 
-int http_client_check_keep(http_client *http)
+static int http_client_check_keep(http_client *http)
 {
     // not keep_alive, try close it actively, and it will destroy the http_client
     if (http->keep_alive == 0 && http->state != HTTP_CLIENT_STATE_WAIT_CLOSE && !http->upgrade)
@@ -1260,7 +1370,7 @@ int http_client_check_keep(http_client *http)
     }
 }
 
-void http_client_reset(http_client *http)
+static void http_client_reset(http_client *http)
 {
     // reset attributes
     http->completed = 0;
@@ -1268,7 +1378,7 @@ void http_client_reset(http_client *http)
     http->state = HTTP_CLIENT_STATE_READY;
 }
 
-uint8_t http_client_free(zval *zobject)
+static uint8_t http_client_free(zval *zobject)
 {
     http_client *http = swoole_get_object(zobject);
     if (!http)
@@ -1302,7 +1412,7 @@ uint8_t http_client_free(zval *zobject)
     return 1;
 }
 
-http_client* http_client_create(zval *zobject)
+static http_client* http_client_create(zval *zobject)
 {
     zval *ztmp;
     http_client *http;
@@ -1324,7 +1434,13 @@ http_client* http_client_create(zval *zobject)
     convert_to_long(ztmp);
     http->port = Z_LVAL_P(ztmp);
 
-    http->timeout = SW_CLIENT_DEFAULT_TIMEOUT;
+#ifdef SW_USE_OPENSSL
+    ztmp = sw_zend_read_property(Z_OBJCE_P(zobject), zobject, ZEND_STRL("ssl"), 0);
+    convert_to_boolean(ztmp);
+    http->ssl = Z_BVAL_P(ztmp);
+#endif
+
+    http->timeout = SW_CLIENT_CONNECT_TIMEOUT;
     http->keep_alive = 1;
     http->state = HTTP_CLIENT_STATE_READY;
     http->object = zobject;
@@ -1342,6 +1458,8 @@ static PHP_METHOD(swoole_http_client, __construct)
     long port = 80;
     zend_bool ssl = SW_FALSE;
 
+    swoole_php_fatal_error(E_DEPRECATED, "async APIs will be removed in Swoole-v4.3.0, you should be using the coroutine APIs instead.");
+
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "s|lb", &host, &host_len, &port, &ssl) == FAILURE)
     {
         RETURN_FALSE;
@@ -1359,8 +1477,9 @@ static PHP_METHOD(swoole_http_client, __construct)
         RETURN_FALSE;
     }
 
-    zend_update_property_stringl(swoole_http_client_class_entry_ptr, getThis(), ZEND_STRL("host"), host, host_len);
-    zend_update_property_long(swoole_http_client_class_entry_ptr, getThis(), ZEND_STRL("port"), port);
+    zend_update_property_stringl(swoole_http_client_ce_ptr, getThis(), ZEND_STRL("host"), host, host_len);
+    zend_update_property_long(swoole_http_client_ce_ptr, getThis(), ZEND_STRL("port"), port);
+    zend_update_property_bool(swoole_http_client_ce_ptr, getThis(), ZEND_STRL("ssl"), ssl);
 
     //init
     swoole_set_object(getThis(), NULL);
@@ -1381,7 +1500,7 @@ static PHP_METHOD(swoole_http_client, __construct)
 #endif
     }
 
-    zend_update_property_long(swoole_client_class_entry_ptr, getThis(), ZEND_STRL("type"), flags);
+    zend_update_property_long(swoole_client_ce_ptr, getThis(), ZEND_STRL("type"), flags);
 
     RETURN_TRUE;
 }
@@ -1395,7 +1514,7 @@ static PHP_METHOD(swoole_http_client, __destruct)
     {
         zval *zobject = getThis();
         zval *retval = NULL;
-        sw_zend_call_method_with_0_params(&zobject, swoole_http_client_class_entry_ptr, NULL, "close", &retval);
+        sw_zend_call_method_with_0_params(&zobject, swoole_http_client_ce_ptr, NULL, "close", &retval);
         if (retval)
         {
             zval_ptr_dtor(retval);
@@ -1426,7 +1545,7 @@ static PHP_METHOD(swoole_http_client, set)
         RETURN_FALSE;
     }
 
-    zval *zsetting = sw_zend_read_property_array(swoole_http_client_class_entry_ptr, getThis(), ZEND_STRL("setting"), 1);
+    zval *zsetting = sw_zend_read_property_array(swoole_http_client_ce_ptr, getThis(), ZEND_STRL("setting"), 1);
     php_array_merge(Z_ARRVAL_P(zsetting), Z_ARRVAL_P(zset));
 
     RETURN_TRUE;
@@ -1440,7 +1559,7 @@ static PHP_METHOD(swoole_http_client, setHeaders)
         RETURN_FALSE;
     }
 
-    zval *zheaders = sw_zend_read_property_array(swoole_http_client_class_entry_ptr, getThis(), ZEND_STRL("requestHeaders"), 1);
+    zval *zheaders = sw_zend_read_property_array(swoole_http_client_ce_ptr, getThis(), ZEND_STRL("requestHeaders"), 1);
     php_array_merge(Z_ARRVAL_P(zheaders), Z_ARRVAL_P(headers));
 
     RETURN_TRUE;
@@ -1454,7 +1573,7 @@ static PHP_METHOD(swoole_http_client, setCookies)
         RETURN_FALSE;
     }
 
-    zval *zcookies = sw_zend_read_property_array(swoole_http_client_class_entry_ptr, getThis(), ZEND_STRL("cookies"), 1);
+    zval *zcookies = sw_zend_read_property_array(swoole_http_client_ce_ptr, getThis(), ZEND_STRL("cookies"), 1);
     php_array_merge(Z_ARRVAL_P(zcookies), Z_ARRVAL_P(cookies));
 
     RETURN_TRUE;
@@ -1473,12 +1592,12 @@ static PHP_METHOD(swoole_http_client, setData)
     }
     if (Z_TYPE_P(data) == IS_ARRAY)
     {
-        zval *zbody = sw_zend_read_property_array(swoole_http_client_class_entry_ptr, getThis(), ZEND_STRL("requestBody"), 1);
+        zval *zbody = sw_zend_read_property_array(swoole_http_client_ce_ptr, getThis(), ZEND_STRL("requestBody"), 1);
         php_array_merge(Z_ARRVAL_P(zbody), Z_ARRVAL_P(data));
     }
     else
     {
-        zend_update_property(swoole_http_client_class_entry_ptr, getThis(), ZEND_STRL("requestBody"), data);
+        zend_update_property(swoole_http_client_ce_ptr, getThis(), ZEND_STRL("requestBody"), data);
     }
     RETURN_TRUE;
 }
@@ -1536,7 +1655,7 @@ static PHP_METHOD(swoole_http_client, addFile)
     }
     if (type == NULL)
     {
-        type = swoole_get_mime_type(path);
+        type = (char*) swoole_get_mime_type(path);
         l_type = strlen(type);
     }
     if (filename == NULL)
@@ -1560,10 +1679,10 @@ static PHP_METHOD(swoole_http_client, addFile)
     {
         SW_MAKE_STD_ZVAL(files);
         array_init(files);
-        zend_update_property(swoole_http_client_class_entry_ptr, getThis(), ZEND_STRL("uploadFiles"), files);
+        zend_update_property(swoole_http_client_ce_ptr, getThis(), ZEND_STRL("uploadFiles"), files);
         zval_ptr_dtor(files);
 
-        hcc->request_upload_files = sw_zend_read_property(swoole_http_client_class_entry_ptr, getThis(), ZEND_STRL("uploadFiles"), 0);
+        hcc->request_upload_files = sw_zend_read_property(swoole_http_client_ce_ptr, getThis(), ZEND_STRL("uploadFiles"), 0);
         sw_copy_to_stack(hcc->request_upload_files, hcc->_request_upload_files);
     }
 
@@ -1606,7 +1725,7 @@ static PHP_METHOD(swoole_http_client, setMethod)
 
     http_client_property *hcc = swoole_get_property(getThis(), 0);
     hcc->request_method = (char *) http_method_str;
-    zend_update_property_string(swoole_http_client_class_entry_ptr, getThis(), ZEND_STRL("requestMethod"),
+    zend_update_property_string(swoole_http_client_ce_ptr, getThis(), ZEND_STRL("requestMethod"),
             (char *) http_method_str);
     RETURN_TRUE;
 }
@@ -1677,7 +1796,6 @@ static PHP_METHOD(swoole_http_client, on)
     if (!sw_zend_is_callable(zcallback, 0, &func_name))
     {
         swoole_php_fatal_error(E_ERROR, "function '%s' is not callable", func_name);
-        efree(func_name);
         return;
     }
     efree(func_name);
@@ -1686,26 +1804,26 @@ static PHP_METHOD(swoole_http_client, on)
     http_client_property *hcc = swoole_get_property(getThis(), 0);
     if (strncasecmp("error", cb_name, cb_name_len) == 0)
     {
-        zend_update_property(swoole_http_client_class_entry_ptr, getThis(), ZEND_STRL("onError"), zcallback);
-        hcc->onError = sw_zend_read_property(swoole_http_client_class_entry_ptr,  getThis(), ZEND_STRL("onError"), 0);
+        zend_update_property(swoole_http_client_ce_ptr, getThis(), ZEND_STRL("onError"), zcallback);
+        hcc->onError = sw_zend_read_property(swoole_http_client_ce_ptr,  getThis(), ZEND_STRL("onError"), 0);
         sw_copy_to_stack(hcc->onError, hcc->_onError);
     }
     else if (strncasecmp("connect", cb_name, cb_name_len) == 0)
     {
-        zend_update_property(swoole_http_client_class_entry_ptr, getThis(), ZEND_STRL("onConnect"), zcallback);
-        hcc->onConnect = sw_zend_read_property(swoole_http_client_class_entry_ptr,  getThis(), ZEND_STRL("onConnect"), 0);
+        zend_update_property(swoole_http_client_ce_ptr, getThis(), ZEND_STRL("onConnect"), zcallback);
+        hcc->onConnect = sw_zend_read_property(swoole_http_client_ce_ptr,  getThis(), ZEND_STRL("onConnect"), 0);
         sw_copy_to_stack(hcc->onConnect, hcc->_onConnect);
     }
     else if (strncasecmp("close", cb_name, cb_name_len) == 0)
     {
-        zend_update_property(swoole_http_client_class_entry_ptr, getThis(), ZEND_STRL("onClose"), zcallback);
-        hcc->onClose = sw_zend_read_property(swoole_http_client_class_entry_ptr,  getThis(), ZEND_STRL("onClose"), 0);
+        zend_update_property(swoole_http_client_ce_ptr, getThis(), ZEND_STRL("onClose"), zcallback);
+        hcc->onClose = sw_zend_read_property(swoole_http_client_ce_ptr,  getThis(), ZEND_STRL("onClose"), 0);
         sw_copy_to_stack(hcc->onClose, hcc->_onClose);
     }
     else if (strncasecmp("message", cb_name, cb_name_len) == 0)
     {
-        zend_update_property(swoole_http_client_class_entry_ptr, getThis(), ZEND_STRL("onMessage"), zcallback);
-        hcc->onMessage = sw_zend_read_property(swoole_http_client_class_entry_ptr,  getThis(), ZEND_STRL("onMessage"), 0);
+        zend_update_property(swoole_http_client_ce_ptr, getThis(), ZEND_STRL("onMessage"), zcallback);
+        hcc->onMessage = sw_zend_read_property(swoole_http_client_ce_ptr,  getThis(), ZEND_STRL("onMessage"), 0);
         sw_copy_to_stack(hcc->onMessage, hcc->_onMessage);
     }
     else
@@ -1719,7 +1837,7 @@ static PHP_METHOD(swoole_http_client, on)
 /**
  * Http Parser Callback
  */
-int http_client_parser_on_header_field(swoole_http_parser *parser, const char *at, size_t length)
+static int http_client_parser_on_header_field(swoole_http_parser *parser, const char *at, size_t length)
 {
     http_client* http = (http_client*) parser->data;
     http->tmp_header_field_name = (char *) at;
@@ -1727,7 +1845,7 @@ int http_client_parser_on_header_field(swoole_http_parser *parser, const char *a
     return 0;
 }
 
-int http_client_parser_on_header_value(swoole_http_parser *parser, const char *at, size_t length)
+static int http_client_parser_on_header_value(swoole_http_parser *parser, const char *at, size_t length)
 {
     http_client* http = (http_client*) parser->data;
     zval* zobject = (zval*) http->object;
@@ -1859,7 +1977,7 @@ int http_response_uncompress(z_stream *stream, swString *buffer, char *body, int
 }
 #endif
 
-int http_client_parser_on_body(swoole_http_parser *parser, const char *at, size_t length)
+static int http_client_parser_on_body(swoole_http_parser *parser, const char *at, size_t length)
 {
     http_client* http = (http_client*) parser->data;
     if (swString_append_ptr(http->body, at, length) < 0)
@@ -1898,7 +2016,7 @@ enum flags
     F_CONNECTION_CLOSE = 1 << 2,
 };
 
-int http_client_parser_on_headers_complete(swoole_http_parser *parser)
+static int http_client_parser_on_headers_complete(swoole_http_parser *parser)
 {
     http_client* http = (http_client*) parser->data;
     //no content-length
@@ -1907,14 +2025,14 @@ int http_client_parser_on_headers_complete(swoole_http_parser *parser)
         http->state = HTTP_CLIENT_STATE_WAIT_CLOSE;
         parser->flags |= F_CONNECTION_CLOSE;
     }
-    if (http->method == HTTP_HEAD || parser->status_code == SW_HTTP_NO_CONTENT)
+    if (http->method == SW_HTTP_HEAD || parser->status_code == SW_HTTP_NO_CONTENT)
     {
         return 1;
     }
     return 0;
 }
 
-int http_client_parser_on_message_complete(swoole_http_parser *parser)
+static int http_client_parser_on_message_complete(swoole_http_parser *parser)
 {
     http_client* http = (http_client*) parser->data;
     zval* zobject = (zval*) http->object;
@@ -2023,8 +2141,8 @@ static PHP_METHOD(swoole_http_client, download)
         swoole_php_error(E_WARNING, "Connection failed, the server was unavailable.");
         return;
     }
-    zend_update_property(swoole_http_client_class_entry_ptr, getThis(), ZEND_STRL("downloadFile"), download_file);
-    hcc->download_file = sw_zend_read_property(swoole_http_client_class_entry_ptr, getThis(), ZEND_STRL("downloadFile"), 1);
+    zend_update_property(swoole_http_client_ce_ptr, getThis(), ZEND_STRL("downloadFile"), download_file);
+    hcc->download_file = sw_zend_read_property(swoole_http_client_ce_ptr, getThis(), ZEND_STRL("downloadFile"), 1);
     hcc->download_offset = offset;
     sw_copy_to_stack(hcc->download_file, hcc->_download_file);
     ret = http_client_execute(getThis(), uri, uri_len, finish_cb);
@@ -2056,12 +2174,12 @@ static PHP_METHOD(swoole_http_client, post)
     }
     if (Z_TYPE_P(data) == IS_ARRAY)
     {
-        zval *zbody = sw_zend_read_property_array(swoole_http_client_class_entry_ptr, getThis(), ZEND_STRL("requestBody"), 1);
+        zval *zbody = sw_zend_read_property_array(swoole_http_client_ce_ptr, getThis(), ZEND_STRL("requestBody"), 1);
         php_array_merge(Z_ARRVAL_P(zbody), Z_ARRVAL_P(data));
     }
     else
     {
-        zend_update_property(swoole_http_client_class_entry_ptr, getThis(), ZEND_STRL("requestBody"), data);
+        zend_update_property(swoole_http_client_ce_ptr, getThis(), ZEND_STRL("requestBody"), data);
     }
     ret = http_client_execute(getThis(), uri, uri_len, callback);
     SW_CHECK_RETURN(ret);
@@ -2091,7 +2209,7 @@ static PHP_METHOD(swoole_http_client, upgrade)
         return;
     }
 
-    zval *zheaders = sw_zend_read_property_array(swoole_http_client_class_entry_ptr, getThis(), ZEND_STRL("requestHeaders"), 1);
+    zval *zheaders = sw_zend_read_property_array(swoole_http_client_ce_ptr, getThis(), ZEND_STRL("requestHeaders"), 1);
 
     char buf[SW_WEBSOCKET_KEY_LENGTH + 1];
     http_client_create_token(SW_WEBSOCKET_KEY_LENGTH, buf);
