@@ -20,18 +20,18 @@
 
 extern "C"
 {
-#include <ext/standard/url.h>
-#include <ext/standard/sha1.h>
-#include <ext/standard/php_var.h>
-#include <ext/standard/php_string.h>
-#include <ext/standard/php_math.h>
-#include <ext/standard/php_array.h>
-#include <ext/date/php_date.h>
-#include <ext/standard/md5.h>
+#include "ext/standard/url.h"
+#include "ext/standard/sha1.h"
+#include "ext/standard/php_var.h"
+#include "ext/standard/php_string.h"
+#include "ext/standard/php_math.h"
+#include "ext/standard/php_array.h"
+#include "ext/date/php_date.h"
+#include "ext/standard/md5.h"
 }
 
-#include <main/rfc1867.h>
-#include <main/php_variables.h>
+#include "main/rfc1867.h"
+#include "main/php_variables.h"
 
 #include "websocket.h"
 #include "connection.h"
@@ -80,14 +80,17 @@ enum http_upload_errno
     HTTP_UPLOAD_ERR_CANT_WRITE,
 };
 
-zend_class_entry swoole_http_server_ce;
-zend_class_entry *swoole_http_server_class_entry_ptr;
+static zend_class_entry swoole_http_server_ce;
+zend_class_entry *swoole_http_server_ce_ptr;
+zend_object_handlers swoole_http_server_handlers;
 
-zend_class_entry swoole_http_response_ce;
-zend_class_entry *swoole_http_response_class_entry_ptr;
+static zend_class_entry swoole_http_response_ce;
+zend_class_entry *swoole_http_response_ce_ptr;
+static zend_object_handlers swoole_http_response_handlers;
 
-zend_class_entry swoole_http_request_ce;
-zend_class_entry *swoole_http_request_class_entry_ptr;
+static zend_class_entry swoole_http_request_ce;
+zend_class_entry *swoole_http_request_ce_ptr;
+static zend_object_handlers swoole_http_request_handlers;
 
 static int http_request_on_path(swoole_http_parser *parser, const char *at, size_t length);
 static int http_request_on_query_string(swoole_http_parser *parser, const char *at, size_t length);
@@ -240,14 +243,6 @@ static inline long http_fast_parse(swoole_http_parser *parser, char *data, size_
 }
 #endif
 
-#ifdef SW_HAVE_ZLIB
-extern "C"
-{
-voidpf php_zlib_alloc(voidpf opaque, uInt items, uInt size);
-void php_zlib_free(voidpf opaque, voidpf address);
-}
-#endif
-
 static PHP_METHOD(swoole_http_request, getData);
 static PHP_METHOD(swoole_http_request, rawcontent);
 static PHP_METHOD(swoole_http_request, __destruct);
@@ -472,11 +467,6 @@ static int http_request_on_query_string(swoole_http_parser *parser, const char *
 static int http_request_on_header_field(swoole_http_parser *parser, const char *at, size_t length)
 {
     http_context *ctx = (http_context *) parser->data;
-    if (ctx->current_header_name_allocated)
-    {
-        efree(ctx->current_header_name);
-        ctx->current_header_name_allocated = 0;
-    }
     ctx->current_header_name = (char *) at;
     ctx->current_header_name_len = length;
     return 0;
@@ -596,16 +586,10 @@ static int http_request_on_header_value(swoole_http_parser *parser, const char *
     if (strncmp(header_name, "cookie", header_len) == 0)
     {
         zval *zcookie;
-        if (length >= SW_HTTP_COOKIE_VALLEN)
-        {
-            swWarn("cookie is too large.");
-        }
-        else
-        {
-            swoole_http_server_array_init(cookie, request);
-            http_parse_cookie(zcookie, at, length);
-        }
-        goto free_memory;
+        swoole_http_server_array_init(cookie, request);
+        http_parse_cookie(zcookie, at, length);
+        efree(header_name);
+        return 0;
     }
     else if (strncmp(header_name, "upgrade", header_len) == 0 && strncasecmp(at, "websocket", length) == 0)
     {
@@ -670,14 +654,8 @@ static int http_request_on_header_value(swoole_http_parser *parser, const char *
     }
 #endif
 
-    add_assoc_stringl_ex(header, header_name, ctx->current_header_name_len, (char *) at, length);
+    add_assoc_stringl_ex(header, header_name, header_len, (char *) at, length);
 
-    free_memory:
-    if (ctx->current_header_name_allocated)
-    {
-        efree(ctx->current_header_name);
-        ctx->current_header_name_allocated = 0;
-    }
     efree(header_name);
 
     return 0;
@@ -686,11 +664,6 @@ static int http_request_on_header_value(swoole_http_parser *parser, const char *
 static int http_request_on_headers_complete(swoole_http_parser *parser)
 {
     http_context *ctx = (http_context *) parser->data;
-    if (ctx->current_header_name_allocated)
-    {
-        efree(ctx->current_header_name);
-        ctx->current_header_name_allocated = 0;
-    }
     ctx->current_header_name = NULL;
 
     return 0;
@@ -771,13 +744,12 @@ static int multipart_body_on_header_value(multipart_parser* p, const char *at, s
             }
             ctx->current_input_name = estrndup(tmp, value_len);
 
-            zval *multipart_header = NULL;
-            SW_ALLOC_INIT_ZVAL(multipart_header);
+            zval *multipart_header = sw_malloc_zval();
             array_init(multipart_header);
 
-            add_assoc_string(multipart_header, "name", "");
-            add_assoc_string(multipart_header, "type", "");
-            add_assoc_string(multipart_header, "tmp_name", "");
+            add_assoc_string(multipart_header, "name", (char *) "");
+            add_assoc_string(multipart_header, "type", (char *) "");
+            add_assoc_string(multipart_header, "tmp_name", (char *) "");
             add_assoc_long(multipart_header, "error", HTTP_UPLOAD_ERR_OK);
             add_assoc_long(multipart_header, "size", 0);
 
@@ -797,11 +769,6 @@ static int multipart_body_on_header_value(multipart_parser* p, const char *at, s
         add_assoc_stringl(ctx->current_multipart_header, "type", (char * ) at, length);
     }
 
-    if (ctx->current_header_name_allocated)
-    {
-        efree(ctx->current_header_name);
-        ctx->current_header_name_allocated = 0;
-    }
     efree(headername);
 
     return 0;
@@ -887,7 +854,7 @@ static int multipart_body_on_header_complete(multipart_parser* p)
     p->fp = fp;
     add_assoc_string(multipart_header, "tmp_name", file_path);
 
-    zval *ztmpfiles = sw_zend_read_property(swoole_http_request_class_entry_ptr, zrequest_object, ZEND_STRL("tmpfiles"), 1);
+    zval *ztmpfiles = sw_zend_read_property(swoole_http_request_ce_ptr, zrequest_object, ZEND_STRL("tmpfiles"), 1);
     if (ztmpfiles == NULL || ZVAL_IS_NULL(ztmpfiles))
     {
         swoole_http_server_array_init(tmpfiles, request);
@@ -908,7 +875,7 @@ static int multipart_body_on_data_end(multipart_parser* p)
     zval *zrequest_object = ctx->request.zobject;
     if (ctx->current_form_data_name)
     {
-        zval *zpost = sw_zend_read_property(swoole_http_request_class_entry_ptr, zrequest_object, ZEND_STRL("post"), 1);
+        zval *zpost = sw_zend_read_property(swoole_http_request_ce_ptr, zrequest_object, ZEND_STRL("post"), 1);
         if (ZVAL_IS_NULL(zpost))
         {
             swoole_http_server_array_init(post, request);
@@ -1011,7 +978,6 @@ static int http_request_message_complete(swoole_http_parser *parser)
             break;
         }
     }
-    ctx->request_read = 1;
 
     if (ctx->mt_parser)
     {
@@ -1027,7 +993,7 @@ int php_swoole_http_onReceive(swServer *serv, swEventData *req)
     int fd = req->info.fd;
     int from_fd = req->info.from_fd;
 
-    swConnection *conn = swServer_connection_verify_no_ssl(SwooleG.serv, fd);
+    swConnection *conn = swServer_connection_verify_no_ssl(serv, fd);
     if (!conn)
     {
         swoole_error_log(SW_LOG_NOTICE, SW_ERROR_SESSION_NOT_EXIST, "connection[%d] is closed.", fd);
@@ -1043,7 +1009,7 @@ int php_swoole_http_onReceive(swServer *serv, swEventData *req)
     //websocket client
     if (conn->websocket_status == WEBSOCKET_STATUS_ACTIVE)
     {
-        return swoole_websocket_onMessage(req);
+        return swoole_websocket_onMessage(serv, req);
     }
 #ifdef SW_USE_HTTP2
     if (conn->http2_stream)
@@ -1058,11 +1024,10 @@ int php_swoole_http_onReceive(swServer *serv, swEventData *req)
 
     parser->data = ctx;
 
-    zval *zdata;
-    SW_ALLOC_INIT_ZVAL(zdata);
+    zval *zdata = sw_malloc_zval();
     php_swoole_get_recv_data(zdata, req, NULL, 0);
 
-    swTrace("httpRequest %d bytes:\n---------------------------------------\n%s\n", (int)Z_STRLEN_P(zdata), Z_STRVAL_P(zdata));
+    swTrace("http request from %d with %d bytes: <<EOF\n%.*s\nEOF", fd, (int)Z_STRLEN_P(zdata), (int)Z_STRLEN_P(zdata), Z_STRVAL_P(zdata));
 
 #ifdef SW_USE_PICOHTTPPARSER
     long n = http_fast_parse(parser, Z_STRVAL_P(zdata), Z_STRLEN_P(zdata));
@@ -1077,18 +1042,13 @@ int php_swoole_http_onReceive(swServer *serv, swEventData *req)
         swWarn("swoole_http_parser_execute failed.");
         if (conn->websocket_status == WEBSOCKET_STATUS_CONNECTION)
         {
-            return swServer_tcp_close(SwooleG.serv, fd, 1);
+            return swServer_tcp_close(serv, fd, 1);
         }
     }
     else
     {
-        zval *retval = NULL;
-
-        zval *zrequest_object = ctx->request.zobject;
-        zval *zresponse_object = ctx->response.zobject;
-
-        SW_SEPARATE_ZVAL(zrequest_object);
-        SW_SEPARATE_ZVAL(zresponse_object);
+        zval _zrequest_object = *ctx->request.zobject, *zrequest_object = &_zrequest_object;
+        zval _zresponse_object = *ctx->response.zobject, *zresponse_object = &_zresponse_object;
 
         ctx->keepalive = swoole_http_should_keep_alive(parser);
         const char *method_name = http_get_method_name(parser->method);
@@ -1112,22 +1072,14 @@ int php_swoole_http_onReceive(swServer *serv, swEventData *req)
 
         swoole_set_property(zrequest_object, 0, zdata);
 
-        add_assoc_long(zserver, "server_port", swConnection_get_port(&SwooleG.serv->connection_list[conn->from_fd]));
+        add_assoc_long(zserver, "server_port", swConnection_get_port(&serv->connection_list[conn->from_fd]));
         add_assoc_long(zserver, "remote_port", swConnection_get_port(conn));
         add_assoc_string(zserver, "remote_addr", swConnection_get_ip(conn));
         add_assoc_long(zserver, "master_time", conn->last_time);
-        if (ctx->request.version == 101)
-        {
-            add_assoc_string(zserver, "server_protocol", "HTTP/1.1");
-        }
-        else
-        {
-            add_assoc_string(zserver, "server_protocol", "HTTP/1.0");
-        }
-        add_assoc_string(zserver, "server_software", SW_HTTP_SERVER_SOFTWARE);
+        add_assoc_string(zserver, "server_protocol", (char *) (ctx->request.version == 101 ? "HTTP/1.1" : "HTTP/1.0"));
+        add_assoc_string(zserver, "server_software", (char *) SW_HTTP_SERVER_SOFTWARE);
 
         // begin to check and call registerd callback
-
         zend_fcall_info_cache *fci_cache = NULL;
 
         if (conn->websocket_status == WEBSOCKET_STATUS_CONNECTION)
@@ -1135,7 +1087,7 @@ int php_swoole_http_onReceive(swServer *serv, swEventData *req)
             fci_cache = php_swoole_server_get_fci_cache(serv, from_fd, SW_SERVER_CB_onHandShake);
             if (fci_cache == NULL)
             {
-                swoole_websocket_onHandshake(port, ctx);
+                swoole_websocket_onHandshake(serv, port, ctx);
                 goto _free_object;
             }
             else
@@ -1160,45 +1112,24 @@ int php_swoole_http_onReceive(swServer *serv, swEventData *req)
 
         if (SwooleG.enable_coroutine)
         {
-            long ret = sw_coro_create(fci_cache, 2, args, retval);
-            if (ret < 0)
+            if (sw_coro_create(fci_cache, 2, args) < 0)
             {
-                if (ret < 0)
-                {
-                    serv->factory.end(&SwooleG.serv->factory, fd);
-                }
-                goto _free_object;
+                swoole_php_error(E_WARNING, "create Http onRequest coroutine error.");
+                swServer_tcp_close(serv, fd, 0);
             }
         }
         else
         {
-            if (sw_call_user_function_fast_ex(NULL, fci_cache, &retval, 2, args) == FAILURE)
+            zval _retval, *retval = &_retval;
+            if (sw_call_user_function_fast_ex(NULL, fci_cache, retval, 2, args) == FAILURE)
             {
-                swoole_php_error(E_WARNING, "onRequest handler error");
+                swoole_php_error(E_WARNING, "Http onRequest handler error.");
             }
+            zval_ptr_dtor(retval);
         }
-
-        if (EG(exception))
-        {
-            zend_exception_error(EG(exception), E_ERROR);
-        }
-        //websocket user handshake
-        if (conn->websocket_status == WEBSOCKET_STATUS_HANDSHAKE)
-        {
-            //handshake success
-            if (retval && Z_BVAL_P(retval))
-            {
-                conn->websocket_status = WEBSOCKET_STATUS_ACTIVE;
-            }
-        }
-
         _free_object:
         zval_ptr_dtor(zrequest_object);
         zval_ptr_dtor(zresponse_object);
-        if (retval)
-        {
-            zval_ptr_dtor(retval);
-        }
     }
 
     return SW_OK;
@@ -1207,7 +1138,7 @@ int php_swoole_http_onReceive(swServer *serv, swEventData *req)
 void php_swoole_http_onClose(swServer *serv, swDataHead *ev)
 {
     int fd = ev->fd;
-    swConnection *conn = swWorker_get_connection(SwooleG.serv, fd);
+    swConnection *conn = swWorker_get_connection(serv, fd);
     if (!conn)
     {
         return;
@@ -1223,52 +1154,42 @@ void php_swoole_http_onClose(swServer *serv, swDataHead *ev)
 
 void swoole_http_server_init(int module_number)
 {
-    SWOOLE_INIT_CLASS_ENTRY(swoole_http_server_ce, "swoole_http_server", "Swoole\\Http\\Server", NULL);
-    swoole_http_server_class_entry_ptr = sw_zend_register_internal_class_ex(&swoole_http_server_ce, swoole_server_class_entry_ptr, "swoole_server");
-    swoole_http_server_class_entry_ptr->serialize = zend_class_serialize_deny;
-    swoole_http_server_class_entry_ptr->unserialize = zend_class_unserialize_deny;
-    SWOOLE_CLASS_ALIAS(swoole_http_server, "Swoole\\Http\\Server");
+    SWOOLE_INIT_CLASS_ENTRY_EX(swoole_http_server, "Swoole\\Http\\Server", "swoole_http_server", NULL, NULL, swoole_server);
+    SWOOLE_SET_CLASS_SERIALIZABLE(swoole_http_server, zend_class_serialize_deny, zend_class_unserialize_deny);
+    SWOOLE_SET_CLASS_CLONEABLE(swoole_http_server, zend_class_clone_deny);
+    SWOOLE_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_http_server, zend_class_unset_property_deny);
 
-    zend_declare_property_null(swoole_http_server_class_entry_ptr, ZEND_STRL("onRequest"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_http_server_class_entry_ptr, ZEND_STRL("onHandshake"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_http_server_class_entry_ptr, ZEND_STRL("setting"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_server_ce_ptr, ZEND_STRL("onRequest"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_server_ce_ptr, ZEND_STRL("onHandshake"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_server_ce_ptr, ZEND_STRL("setting"), ZEND_ACC_PUBLIC);
 
-    SWOOLE_INIT_CLASS_ENTRY(swoole_http_response_ce, "swoole_http_response", "Swoole\\Http\\Response", swoole_http_response_methods);
-    swoole_http_response_class_entry_ptr = zend_register_internal_class(&swoole_http_response_ce);
-    swoole_http_response_class_entry_ptr->serialize = zend_class_serialize_deny;
-    swoole_http_response_class_entry_ptr->unserialize = zend_class_unserialize_deny;
-    SWOOLE_CLASS_ALIAS(swoole_http_response, "Swoole\\Http\\Response");
+    SWOOLE_INIT_CLASS_ENTRY(swoole_http_request, "Swoole\\Http\\Request", "swoole_http_request", NULL, swoole_http_request_methods);
+    SWOOLE_SET_CLASS_SERIALIZABLE(swoole_http_request, zend_class_serialize_deny, zend_class_unserialize_deny);
+    SWOOLE_SET_CLASS_CLONEABLE(swoole_http_request, zend_class_clone_deny);
+    SWOOLE_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_http_request, zend_class_unset_property_deny);
 
-    zend_declare_property_long(swoole_http_response_class_entry_ptr, ZEND_STRL("fd"), 0,  ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_http_response_class_entry_ptr, ZEND_STRL("header"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_http_response_class_entry_ptr, ZEND_STRL("cookie"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_http_response_class_entry_ptr, ZEND_STRL("trailer"), ZEND_ACC_PUBLIC);
-
-    SWOOLE_INIT_CLASS_ENTRY(swoole_http_request_ce, "swoole_http_request", "Swoole\\Http\\Request", swoole_http_request_methods);
-    swoole_http_request_class_entry_ptr = zend_register_internal_class(&swoole_http_request_ce);
-    swoole_http_request_class_entry_ptr->serialize = zend_class_serialize_deny;
-    swoole_http_request_class_entry_ptr->unserialize = zend_class_unserialize_deny;
-    SWOOLE_CLASS_ALIAS(swoole_http_request, "Swoole\\Http\\Request");
-
-    if (SWOOLE_G(use_shortname))
-    {
-        sw_zend_register_class_alias("Co\\Http\\Server", swoole_http_server_class_entry_ptr);
-        sw_zend_register_class_alias("Co\\Http\\Request", swoole_http_request_class_entry_ptr);
-        sw_zend_register_class_alias("Co\\Http\\Response", swoole_http_response_class_entry_ptr);
-    }
-
-    zend_declare_property_long(swoole_http_request_class_entry_ptr, ZEND_STRL("fd"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_http_request_ce_ptr, ZEND_STRL("fd"), 0, ZEND_ACC_PUBLIC);
 #ifdef SW_USE_HTTP2
-    zend_declare_property_long(swoole_http_request_class_entry_ptr, ZEND_STRL("streamId"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_http_request_ce_ptr, ZEND_STRL("streamId"), 0, ZEND_ACC_PUBLIC);
 #endif
-    zend_declare_property_null(swoole_http_request_class_entry_ptr, ZEND_STRL("header"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_http_request_class_entry_ptr, ZEND_STRL("server"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_http_request_class_entry_ptr, ZEND_STRL("request"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_http_request_class_entry_ptr, ZEND_STRL("cookie"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_http_request_class_entry_ptr, ZEND_STRL("get"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_http_request_class_entry_ptr, ZEND_STRL("files"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_http_request_class_entry_ptr, ZEND_STRL("post"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_http_request_class_entry_ptr, ZEND_STRL("tmpfiles"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_request_ce_ptr, ZEND_STRL("header"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_request_ce_ptr, ZEND_STRL("server"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_request_ce_ptr, ZEND_STRL("request"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_request_ce_ptr, ZEND_STRL("cookie"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_request_ce_ptr, ZEND_STRL("get"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_request_ce_ptr, ZEND_STRL("files"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_request_ce_ptr, ZEND_STRL("post"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_request_ce_ptr, ZEND_STRL("tmpfiles"), ZEND_ACC_PUBLIC);
+
+    SWOOLE_INIT_CLASS_ENTRY(swoole_http_response, "Swoole\\Http\\Response", "swoole_http_response", NULL, swoole_http_response_methods);
+    SWOOLE_SET_CLASS_SERIALIZABLE(swoole_http_response, zend_class_serialize_deny, zend_class_unserialize_deny);
+    SWOOLE_SET_CLASS_CLONEABLE(swoole_http_response, zend_class_clone_deny);
+    SWOOLE_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_http_response, zend_class_unset_property_deny);
+
+    zend_declare_property_long(swoole_http_response_ce_ptr, ZEND_STRL("fd"), 0,  ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_response_ce_ptr, ZEND_STRL("header"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_response_ce_ptr, ZEND_STRL("cookie"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_http_response_ce_ptr, ZEND_STRL("trailer"), ZEND_ACC_PUBLIC);
 }
 
 http_context* swoole_http_context_new(int fd)
@@ -1284,18 +1205,18 @@ http_context* swoole_http_context_new(int fd)
     zval *zrequest_object;
     zrequest_object = &ctx->request._zobject;
     ctx->request.zobject = zrequest_object;
-    object_init_ex(zrequest_object, swoole_http_request_class_entry_ptr);
+    object_init_ex(zrequest_object, swoole_http_request_ce_ptr);
     swoole_set_object(zrequest_object, ctx);
 
     zval *zresponse_object;
     zresponse_object = &ctx->response._zobject;
     ctx->response.zobject = zresponse_object;
-    object_init_ex(zresponse_object, swoole_http_response_class_entry_ptr);
+    object_init_ex(zresponse_object, swoole_http_response_ce_ptr);
     swoole_set_object(zresponse_object, ctx);
 
     //socket fd
-    zend_update_property_long(swoole_http_response_class_entry_ptr, zresponse_object, ZEND_STRL("fd"), fd);
-    zend_update_property_long(swoole_http_request_class_entry_ptr, zrequest_object, ZEND_STRL("fd"), fd);
+    zend_update_property_long(swoole_http_response_ce_ptr, zresponse_object, ZEND_STRL("fd"), fd);
+    zend_update_property_long(swoole_http_request_ce_ptr, zrequest_object, ZEND_STRL("fd"), fd);
 
 #if PHP_MEMORY_DEBUG
     php_vmstat.new_http_request ++;
@@ -1520,7 +1441,7 @@ static PHP_METHOD(swoole_http_request, __destruct)
 {
     SW_PREVENT_USER_DESTRUCT;
 
-    zval *ztmpfiles = sw_zend_read_property(swoole_http_request_class_entry_ptr, getThis(), ZEND_STRL("tmpfiles"), 1);
+    zval *ztmpfiles = sw_zend_read_property(swoole_http_request_ce_ptr, getThis(), ZEND_STRL("tmpfiles"), 1);
     //upload files
     if (ztmpfiles && Z_TYPE_P(ztmpfiles) == IS_ARRAY)
     {
@@ -1678,7 +1599,7 @@ static void http_build_header(http_context *ctx, zval *zobject, swString *respon
     /**
      * http header
      */
-    zval *zheader = sw_zend_read_property(swoole_http_response_class_entry_ptr, ctx->response.zobject, ZEND_STRL("header"), 1);
+    zval *zheader = sw_zend_read_property(swoole_http_response_ce_ptr, ctx->response.zobject, ZEND_STRL("header"), 1);
     uint32_t header_flag = 0x0;
     if (ZVAL_IS_ARRAY(zheader))
     {
@@ -1690,6 +1611,7 @@ static void http_build_header(http_context *ctx, zval *zobject, swString *respon
 
         SW_HASHTABLE_FOREACH_START2(ht, key, keylen, type, value)
         {
+            // TODO: numeric key name neccessary?
             if (!key)
             {
                 break;
@@ -1780,7 +1702,7 @@ static void http_build_header(http_context *ctx, zval *zobject, swString *respon
     }
 
     //http cookies
-    zval *zcookie = sw_zend_read_property(swoole_http_response_class_entry_ptr, ctx->response.zobject, ZEND_STRL("cookie"), 1);
+    zval *zcookie = sw_zend_read_property(swoole_http_response_ce_ptr, ctx->response.zobject, ZEND_STRL("cookie"), 1);
     if (ZVAL_IS_ARRAY(zcookie))
     {
         zval *value;
@@ -1811,16 +1733,6 @@ static void http_build_header(http_context *ctx, zval *zobject, swString *respon
 }
 
 #ifdef SW_HAVE_ZLIB
-voidpf php_zlib_alloc(voidpf opaque, uInt items, uInt size)
-{
-    return (voidpf) safe_emalloc(items, size, 0);
-}
-
-void php_zlib_free(voidpf opaque, voidpf address)
-{
-    efree((void* )address);
-}
-
 void swoole_http_get_compression_method(http_context *ctx, const char *accept_encoding, size_t length)
 {
 #ifdef SW_HAVE_BROTLI
@@ -1992,19 +1904,19 @@ static PHP_METHOD(swoole_http_response, initHeader)
         RETURN_FALSE;
     }
     zval *zresponse_object = ctx->response.zobject;
-    zval *zheader = sw_zend_read_property(swoole_http_response_class_entry_ptr, zresponse_object, ZEND_STRL("header"), 1);
+    zval *zheader = sw_zend_read_property(swoole_http_response_ce_ptr, zresponse_object, ZEND_STRL("header"), 1);
     if (!ZVAL_IS_ARRAY(zheader))
     {
         swoole_http_server_array_init(header, response);
     }
 
-    zval *zcookie = sw_zend_read_property(swoole_http_response_class_entry_ptr, zresponse_object, ZEND_STRL("cookie"), 1);
+    zval *zcookie = sw_zend_read_property(swoole_http_response_ce_ptr, zresponse_object, ZEND_STRL("cookie"), 1);
     if (!ZVAL_IS_ARRAY(zcookie))
     {
         swoole_http_server_array_init(cookie, response);
     }
 
-    zval *ztrailer = sw_zend_read_property(swoole_http_response_class_entry_ptr, zresponse_object, ZEND_STRL("trailer"), 1);
+    zval *ztrailer = sw_zend_read_property(swoole_http_response_ce_ptr, zresponse_object, ZEND_STRL("trailer"), 1);
     if (!ZVAL_IS_ARRAY(ztrailer))
     {
         swoole_http_server_array_init(trailer, response);
@@ -2217,7 +2129,7 @@ static PHP_METHOD(swoole_http_response, cookie)
     }
 
     zval *zresponse_object = ctx->response.zobject;
-    zval *zcookie = sw_zend_read_property(swoole_http_response_class_entry_ptr, zresponse_object, ZEND_STRL("cookie"), 1);
+    zval *zcookie = sw_zend_read_property(swoole_http_response_ce_ptr, zresponse_object, ZEND_STRL("cookie"), 1);
     if (!ZVAL_IS_ARRAY(zcookie))
     {
         swoole_http_server_array_init(cookie, response);
@@ -2328,7 +2240,7 @@ static PHP_METHOD(swoole_http_response, rawcookie)
     }
 
     zval *zresponse_object = ctx->response.zobject;
-    zval *zcookie = sw_zend_read_property(swoole_http_response_class_entry_ptr, zresponse_object, ZEND_STRL("cookie"), 1);
+    zval *zcookie = sw_zend_read_property(swoole_http_response_ce_ptr, zresponse_object, ZEND_STRL("cookie"), 1);
     if (!ZVAL_IS_ARRAY(zcookie))
     {
         swoole_http_server_array_init(cookie, response);
@@ -2459,7 +2371,7 @@ static PHP_METHOD(swoole_http_response, header)
     }
 
     zval *zresponse_object = ctx->response.zobject;
-    zval *zheader = sw_zend_read_property(swoole_http_response_class_entry_ptr, zresponse_object, ZEND_STRL("header"), 1);
+    zval *zheader = sw_zend_read_property(swoole_http_response_ce_ptr, zresponse_object, ZEND_STRL("header"), 1);
     if (!ZVAL_IS_ARRAY(zheader))
     {
         swoole_http_server_array_init(header, response);
@@ -2518,7 +2430,7 @@ static PHP_METHOD(swoole_http_response, trailer)
     }
 
     zval *zresponse_object = ctx->response.zobject;
-    zval *ztrailer = sw_zend_read_property(swoole_http_response_class_entry_ptr, zresponse_object, ZEND_STRL("trailer"), 1);
+    zval *ztrailer = sw_zend_read_property(swoole_http_response_ce_ptr, zresponse_object, ZEND_STRL("trailer"), 1);
     if (!ZVAL_IS_ARRAY(ztrailer))
     {
         swoole_http_server_array_init(trailer, response);
@@ -2592,12 +2504,12 @@ static PHP_METHOD(swoole_http_response, create)
     bzero(ctx, sizeof(http_context));
     ctx->fd = (int) fd;
 
-    object_init_ex(return_value, swoole_http_response_class_entry_ptr);
+    object_init_ex(return_value, swoole_http_response_ce_ptr);
     swoole_set_object(return_value, ctx);
     ctx->response.zobject = return_value;
     sw_copy_to_stack(ctx->response.zobject, ctx->response._zobject);
 
-    zend_update_property_long(swoole_http_response_class_entry_ptr, return_value, ZEND_STRL("fd"), ctx->fd);
+    zend_update_property_long(swoole_http_response_ce_ptr, return_value, ZEND_STRL("fd"), ctx->fd);
 }
 
 static PHP_METHOD(swoole_http_response, redirect)
@@ -2656,11 +2568,14 @@ static PHP_METHOD(swoole_http_response, __destruct)
         }
         else
         {
-            context->response.status = 500;
+            if (context->response.status == 0)
+            {
+                context->response.status = 500;
+            }
 
             zval *zobject = getThis();
             zval *retval = NULL;
-            sw_zend_call_method_with_0_params(&zobject, swoole_http_response_class_entry_ptr, NULL, "end", &retval);
+            sw_zend_call_method_with_0_params(&zobject, swoole_http_response_ce_ptr, NULL, "end", &retval);
             if (retval)
             {
                 zval_ptr_dtor(retval);
