@@ -21,7 +21,8 @@
 #include "swoole_coroutine.h"
 
 #include "http2.h"
-#include <main/php_variables.h>
+#include "main/php_variables.h"
+
 #include <unordered_map>
 
 extern swString *swoole_http_buffer;
@@ -114,7 +115,7 @@ static int http_build_trailer(http_context *ctx, uchar *buffer)
 {
     int ret;
     size_t index = 0;
-    zval *ztrailer = sw_zend_read_property(swoole_http_response_class_entry_ptr, ctx->response.zobject, ZEND_STRL("trailer"), 0);
+    zval *ztrailer = sw_zend_read_property(swoole_http_response_ce_ptr, ctx->response.zobject, ZEND_STRL("trailer"), 0);
     uint32_t nv_size = ZVAL_IS_ARRAY(ztrailer) ? php_swoole_array_length(ztrailer) : 0;
 
     if (nv_size > 0)
@@ -177,13 +178,10 @@ static int http_build_trailer(http_context *ctx, uchar *buffer)
 
 static sw_inline void http2_onRequest(http_context *ctx, int from_fd)
 {
-    zval *retval = NULL;
     swServer *serv = SwooleG.serv;
     int fd = ctx->fd;
-    zval *zrequest_object = ctx->request.zobject;
-    zval *zresponse_object = ctx->response.zobject;
-    SW_SEPARATE_ZVAL(zrequest_object);
-    SW_SEPARATE_ZVAL(zresponse_object);
+    zval _zrequest_object = *ctx->request.zobject, *zrequest_object = &_zrequest_object;
+    zval _zresponse_object = *ctx->response.zobject, *zresponse_object = &_zresponse_object;
 
     zend_fcall_info_cache *fci_cache = php_swoole_server_get_fci_cache(serv, from_fd, SW_SERVER_CB_onRequest);
     zval args[2];
@@ -192,36 +190,24 @@ static sw_inline void http2_onRequest(http_context *ctx, int from_fd)
 
     if (SwooleG.enable_coroutine)
     {
-        long ret = sw_coro_create(fci_cache, 2, args, retval);
-        if (ret < 0)
+        if (sw_coro_create(fci_cache, 2, args) < 0)
         {
-            if (ret < 0)
-            {
-                serv->factory.end(&SwooleG.serv->factory, fd);
-            }
-            goto _free_object;
+            swoole_php_error(E_WARNING, "create Http2 onRequest coroutine error.");
+            serv->factory.end(&serv->factory, fd);
         }
     }
     else
     {
-        if (sw_call_user_function_fast_ex(NULL, fci_cache, &retval, 2, args) == FAILURE)
+        zval _retval, *retval = &_retval;
+        if (sw_call_user_function_fast_ex(NULL, fci_cache, retval, 2, args) == FAILURE)
         {
-            swoole_php_error(E_WARNING, "Http2 onRequest handler error");
+            swoole_php_error(E_WARNING, "Http2 onRequest handler error.");
         }
-    }
-
-    if (EG(exception))
-    {
-        zend_exception_error(EG(exception), E_ERROR);
-    }
-
-    _free_object:
-    zval_ptr_dtor(zrequest_object);
-    zval_ptr_dtor(zresponse_object);
-    if (retval)
-    {
         zval_ptr_dtor(retval);
     }
+
+    zval_ptr_dtor(zrequest_object);
+    zval_ptr_dtor(zresponse_object);
 }
 
 static int http2_build_header(http_context *ctx, uchar *buffer, size_t body_length)
@@ -231,7 +217,7 @@ static int http2_build_header(http_context *ctx, uchar *buffer, size_t body_leng
     char intbuf[2][16];
     int ret;
     size_t index = 0;
-    zval *zheader = sw_zend_read_property(swoole_http_response_class_entry_ptr, ctx->response.zobject, ZEND_STRL("header"), 1);
+    zval *zheader = sw_zend_read_property(swoole_http_response_ce_ptr, ctx->response.zobject, ZEND_STRL("header"), 1);
     nghttp2_nv *nv = (nghttp2_nv *) ecalloc(sizeof(nghttp2_nv), 8 + (ZVAL_IS_ARRAY(zheader) ? php_swoole_array_length(zheader) : 0));
 
     assert(ctx->send_header == 0);
@@ -308,7 +294,7 @@ static int http2_build_header(http_context *ctx, uchar *buffer, size_t body_leng
     }
 
     // cookies
-    zval *zcookie = sw_zend_read_property(swoole_http_response_class_entry_ptr, ctx->response.zobject, ZEND_STRL("cookie"), 1);
+    zval *zcookie = sw_zend_read_property(swoole_http_response_ce_ptr, ctx->response.zobject, ZEND_STRL("cookie"), 1);
     if (ZVAL_IS_ARRAY(zcookie))
     {
         zval *value;
@@ -419,7 +405,7 @@ int swoole_http2_do_response(http_context *ctx, swString *body)
      +---------------------------------------------------------------+
      */
     char frame_header[SW_HTTP2_FRAME_HEADER_SIZE];
-    zval *ztrailer = sw_zend_read_property(swoole_http_response_class_entry_ptr, ctx->response.zobject, ZEND_STRL("trailer"), 1);
+    zval *ztrailer = sw_zend_read_property(swoole_http_response_ce_ptr, ctx->response.zobject, ZEND_STRL("trailer"), 1);
     if (!ZVAL_IS_ARRAY(ztrailer))
     {
         ztrailer = NULL;
