@@ -100,14 +100,16 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_mysql_coro_statement_nextResult, 0, 0, 0)
 ZEND_END_ARG_INFO()
 
 static zend_class_entry swoole_mysql_coro_ce;
-static zend_class_entry *swoole_mysql_coro_class_entry_ptr;
+static zend_class_entry *swoole_mysql_coro_ce_ptr;
 static zend_object_handlers swoole_mysql_coro_handlers;
 
 static zend_class_entry swoole_mysql_coro_exception_ce;
-static zend_class_entry *swoole_mysql_coro_exception_class_entry_ptr;
+static zend_class_entry *swoole_mysql_coro_exception_ce_ptr;
+static zend_object_handlers swoole_mysql_coro_exception_handlers;
 
 static zend_class_entry swoole_mysql_coro_statement_ce;
-static zend_class_entry *swoole_mysql_coro_statement_class_entry_ptr;
+static zend_class_entry *swoole_mysql_coro_statement_ce_ptr;
+static zend_object_handlers swoole_mysql_coro_statement_handlers;
 
 static const zend_function_entry swoole_mysql_coro_methods[] =
 {
@@ -146,8 +148,8 @@ static int swoole_mysql_coro_onError(swReactor *reactor, swEvent *event);
 static void swoole_mysql_coro_onConnect(mysql_client *client);
 static void swoole_mysql_coro_onTimeout(swTimer *timer, swTimer_node *tnode);
 
-static void swoole_mysql_coro_free_storage(zend_object *object);
-static zend_object *swoole_mysql_coro_create(zend_class_entry *ce)
+static void swoole_mysql_coro_free_object(zend_object *object);
+static zend_object *swoole_mysql_coro_create_object(zend_class_entry *ce)
 {
     zend_object *object;
     object = zend_objects_new(ce);
@@ -158,54 +160,43 @@ static zend_object *swoole_mysql_coro_create(zend_class_entry *ce)
 
     mysql_client *client = (mysql_client *) emalloc(sizeof(mysql_client));
     bzero(client, sizeof(mysql_client));
-
-    zval _zobject, *zobject = &_zobject;
-    ZVAL_OBJ(zobject, object);
-    swoole_set_object(zobject, client);
+    swoole_set_object_by_handle(object->handle, client);
 
     return object;
 }
 
 void swoole_mysql_coro_init(int module_number)
 {
-    INIT_CLASS_ENTRY(swoole_mysql_coro_ce, "Swoole\\Coroutine\\MySQL", swoole_mysql_coro_methods);
-    swoole_mysql_coro_class_entry_ptr = zend_register_internal_class(&swoole_mysql_coro_ce);
-    swoole_mysql_coro_class_entry_ptr->create_object = swoole_mysql_coro_create;
-    swoole_mysql_coro_class_entry_ptr->serialize = zend_class_serialize_deny;
-    swoole_mysql_coro_class_entry_ptr->unserialize = zend_class_unserialize_deny;
-    memcpy(&swoole_mysql_coro_handlers, zend_get_std_object_handlers(), sizeof(swoole_mysql_coro_handlers));
-    swoole_mysql_coro_handlers.free_obj = swoole_mysql_coro_free_storage;
+    SWOOLE_INIT_CLASS_ENTRY(swoole_mysql_coro, "Swoole\\Coroutine\\MySQL", NULL, "Co\\MySQL", swoole_mysql_coro_methods);
+    SWOOLE_SET_CLASS_SERIALIZABLE(swoole_mysql_coro, zend_class_serialize_deny, zend_class_unserialize_deny);
+    SWOOLE_SET_CLASS_CLONEABLE(swoole_mysql_coro, zend_class_clone_deny);
+    SWOOLE_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_mysql_coro, zend_class_unset_property_deny);
+    SWOOLE_SET_CLASS_CREATE_AND_FREE(swoole_mysql_coro, swoole_mysql_coro_create_object, swoole_mysql_coro_free_object);
 
-    INIT_CLASS_ENTRY(swoole_mysql_coro_statement_ce, "Swoole\\Coroutine\\MySQL\\Statement", swoole_mysql_coro_statement_methods);
-    swoole_mysql_coro_statement_class_entry_ptr = zend_register_internal_class(&swoole_mysql_coro_statement_ce);
-    swoole_mysql_coro_statement_class_entry_ptr->serialize = zend_class_serialize_deny;
-    swoole_mysql_coro_statement_class_entry_ptr->unserialize = zend_class_unserialize_deny;
+    SWOOLE_INIT_CLASS_ENTRY(swoole_mysql_coro_statement, "Swoole\\Coroutine\\MySQL\\Statement", NULL, "Co\\MySQL\\Statement", swoole_mysql_coro_statement_methods);
+    SWOOLE_SET_CLASS_SERIALIZABLE(swoole_mysql_coro_statement, zend_class_serialize_deny, zend_class_unserialize_deny);
+    SWOOLE_SET_CLASS_CLONEABLE(swoole_mysql_coro_statement, zend_class_clone_deny);
+    SWOOLE_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_mysql_coro_statement, zend_class_unset_property_deny);
 
-    INIT_CLASS_ENTRY(swoole_mysql_coro_exception_ce, "Swoole\\Coroutine\\MySQL\\Exception", NULL);
-    swoole_mysql_coro_exception_class_entry_ptr = sw_zend_register_internal_class_ex(&swoole_mysql_coro_exception_ce, zend_exception_get_default(), NULL);
+    SWOOLE_INIT_CLASS_ENTRY_EX(swoole_mysql_coro_exception, "Swoole\\Coroutine\\MySQL\\Exception", NULL, "Co\\MySQL\\Exception", NULL, swoole_exception);
+    SWOOLE_SET_CLASS_SERIALIZABLE(swoole_mysql_coro_exception, zend_class_serialize_deny, zend_class_unserialize_deny);
+    SWOOLE_SET_CLASS_CLONEABLE(swoole_mysql_coro_exception, zend_class_clone_deny);
+    SWOOLE_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_mysql_coro_exception, zend_class_unset_property_deny);
 
-    /** ============================================= ALIAS ================================================**/
-    if (SWOOLE_G(use_shortname))
-    {
-        sw_zend_register_class_alias("Co\\MySQL", swoole_mysql_coro_class_entry_ptr);
-        sw_zend_register_class_alias("Co\\MySQL\\Statement", swoole_mysql_coro_statement_class_entry_ptr);
-        sw_zend_register_class_alias("Co\\MySQL\\Exception", swoole_mysql_coro_exception_class_entry_ptr);
-    }
-    /** ============================================= DECLARES =============================================**/
-    zend_declare_property_string(swoole_mysql_coro_class_entry_ptr, ZEND_STRL("serverInfo"), "", ZEND_ACC_PRIVATE);
-    zend_declare_property_long(swoole_mysql_coro_class_entry_ptr, ZEND_STRL("sock"), 0, ZEND_ACC_PUBLIC);
-    zend_declare_property_bool(swoole_mysql_coro_class_entry_ptr, ZEND_STRL("connected"), 0, ZEND_ACC_PUBLIC);
-    zend_declare_property_string(swoole_mysql_coro_class_entry_ptr, ZEND_STRL("connect_error"), "", ZEND_ACC_PUBLIC);
-    zend_declare_property_long(swoole_mysql_coro_class_entry_ptr, ZEND_STRL("connect_errno"), 0, ZEND_ACC_PUBLIC);
-    zend_declare_property_long(swoole_mysql_coro_class_entry_ptr, ZEND_STRL("affected_rows"), 0, ZEND_ACC_PUBLIC);
-    zend_declare_property_long(swoole_mysql_coro_class_entry_ptr, ZEND_STRL("insert_id"), 0, ZEND_ACC_PUBLIC);
-    zend_declare_property_string(swoole_mysql_coro_class_entry_ptr, ZEND_STRL("error"), "", ZEND_ACC_PUBLIC);
-    zend_declare_property_long(swoole_mysql_coro_class_entry_ptr, ZEND_STRL("errno"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_string(swoole_mysql_coro_ce_ptr, ZEND_STRL("serverInfo"), "", ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_mysql_coro_ce_ptr, ZEND_STRL("sock"), -1, ZEND_ACC_PUBLIC);
+    zend_declare_property_bool(swoole_mysql_coro_ce_ptr, ZEND_STRL("connected"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_string(swoole_mysql_coro_ce_ptr, ZEND_STRL("connect_error"), "", ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_mysql_coro_ce_ptr, ZEND_STRL("connect_errno"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_mysql_coro_ce_ptr, ZEND_STRL("affected_rows"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_mysql_coro_ce_ptr, ZEND_STRL("insert_id"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_string(swoole_mysql_coro_ce_ptr, ZEND_STRL("error"), "", ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_mysql_coro_ce_ptr, ZEND_STRL("errno"), 0, ZEND_ACC_PUBLIC);
 
-    zend_declare_property_long(swoole_mysql_coro_statement_class_entry_ptr, ZEND_STRL("affected_rows"), 0, ZEND_ACC_PUBLIC);
-    zend_declare_property_long(swoole_mysql_coro_statement_class_entry_ptr, ZEND_STRL("insert_id"), 0, ZEND_ACC_PUBLIC);
-    zend_declare_property_string(swoole_mysql_coro_statement_class_entry_ptr, ZEND_STRL("error"), "", ZEND_ACC_PUBLIC);
-    zend_declare_property_long(swoole_mysql_coro_statement_class_entry_ptr, ZEND_STRL("errno"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_mysql_coro_statement_ce_ptr, ZEND_STRL("affected_rows"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_mysql_coro_statement_ce_ptr, ZEND_STRL("insert_id"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_string(swoole_mysql_coro_statement_ce_ptr, ZEND_STRL("error"), "", ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_mysql_coro_statement_ce_ptr, ZEND_STRL("errno"), 0, ZEND_ACC_PUBLIC);
 }
 
 int mysql_query(zval *zobject, mysql_client *client, swString *sql, zval *callback);
@@ -352,9 +343,9 @@ static int swoole_mysql_coro_execute(zval *zobject, mysql_client *client, zval *
         //connection is closed
         if (swConnection_error(errno) == SW_CLOSE)
         {
-            zend_update_property_bool(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("connected"), 0);
-            zend_update_property_long(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("errno"), 2013);
-            zend_update_property_string(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("error"), "Lost connection to MySQL server during query");
+            zend_update_property_bool(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("connected"), 0);
+            zend_update_property_long(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("errno"), 2013);
+            zend_update_property_string(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("error"), "Lost connection to MySQL server during query");
         }
         return SW_ERR;
     }
@@ -368,34 +359,67 @@ static int swoole_mysql_coro_execute(zval *zobject, mysql_client *client, zval *
 static int swoole_mysql_coro_parse_response(mysql_client *client, zval **result, int from_next_result)
 {
     zval *zobject = client->object;
+    int ret = mysql_response(client);
 
-    if (mysql_response(client) < 0)
+    if (ret < 0)
     {
-        return SW_ERR;
+        if (ret == SW_AGAIN)
+        {
+            return SW_AGAIN;
+        }
+        else // handler error
+        {
+            static const char* errmsg = "mysql response packet parse error.";
+            client->response.response_type = SW_MYSQL_PACKET_ERR;
+            client->response.error_code = ret;
+            client->response.server_msg = (char *) errmsg;
+            client->response.l_server_msg = strlen(errmsg);
+            if (client->response.result_array)
+            {
+                sw_zval_free(client->response.result_array);
+                client->response.result_array = nullptr;
+            }
+            if (client->cmd == SW_MYSQL_COM_STMT_EXECUTE)
+            {
+                if (client->statement && client->statement->result)
+                {
+                    sw_zval_free(client->statement->result);
+                    client->statement->result = NULL;
+                }
+            }
+        }
     }
 
     //remove from eventloop
     //reactor->del(reactor, event->fd);
 
-    zend_update_property_long(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("affected_rows"),
-            client->response.affected_rows);
-    zend_update_property_long(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("insert_id"),
-            client->response.insert_id);
+    zend_update_property_long(
+        swoole_mysql_coro_ce_ptr, zobject,
+        ZEND_STRL("affected_rows"), client->response.affected_rows
+    );
+    zend_update_property_long(
+        swoole_mysql_coro_ce_ptr, zobject,
+        ZEND_STRL("insert_id"), client->response.insert_id
+    );
 
     if (client->cmd == SW_MYSQL_COM_STMT_EXECUTE)
     {
-        zend_update_property_long(swoole_mysql_coro_statement_class_entry_ptr, client->statement->object,
-                ZEND_STRL("affected_rows"), client->response.affected_rows);
-        zend_update_property_long(swoole_mysql_coro_statement_class_entry_ptr, client->statement->object,
-                ZEND_STRL("insert_id"), client->response.insert_id);
+        zend_update_property_long(
+            swoole_mysql_coro_statement_ce_ptr, client->statement->object,
+            ZEND_STRL("affected_rows"), client->response.affected_rows
+        );
+        zend_update_property_long(
+            swoole_mysql_coro_statement_ce_ptr, client->statement->object,
+            ZEND_STRL("insert_id"), client->response.insert_id
+        );
     }
 
     client->state = SW_MYSQL_STATE_QUERY;
 
-    //OK
+    // OK
     if (client->response.response_type == SW_MYSQL_PACKET_OK)
     {
-        SW_ALLOC_INIT_ZVAL(*result);
+        *result = sw_malloc_zval();
         // prepare finished and create statement
         if (client->cmd == SW_MYSQL_COM_STMT_PREPARE)
         {
@@ -404,7 +428,7 @@ static int swoole_mysql_coro_parse_response(mysql_client *client, zval **result,
                 client->statement_list = swLinkedList_new(0, NULL);
             }
             swLinkedList_append(client->statement_list, client->statement);
-            object_init_ex(*result, swoole_mysql_coro_statement_class_entry_ptr);
+            object_init_ex(*result, swoole_mysql_coro_statement_ce_ptr);
             swoole_set_object(*result, client->statement);
             client->statement->object = sw_zval_dup(*result);
         }
@@ -421,26 +445,34 @@ static int swoole_mysql_coro_parse_response(mysql_client *client, zval **result,
             }
         }
     }
-    //ERROR
+    // ERROR
     else if (client->response.response_type == SW_MYSQL_PACKET_ERR)
     {
-        SW_ALLOC_INIT_ZVAL(*result);
+        *result = sw_malloc_zval();
         ZVAL_FALSE(*result);
 
-        zend_update_property_stringl(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("error"),
-                client->response.server_msg, client->response.l_server_msg);
-        zend_update_property_long(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("errno"),
-                client->response.error_code);
+        zend_update_property_stringl(
+            swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("error"),
+            client->response.server_msg, client->response.l_server_msg
+        );
+        zend_update_property_long(
+            swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("errno"),
+            client->response.error_code
+        );
 
         if (client->cmd == SW_MYSQL_COM_STMT_EXECUTE)
         {
-            zend_update_property_stringl(swoole_mysql_coro_statement_class_entry_ptr, client->statement->object,
-                    ZEND_STRL("error"), client->response.server_msg, client->response.l_server_msg);
-            zend_update_property_long(swoole_mysql_coro_statement_class_entry_ptr, client->statement->object,
-                    ZEND_STRL("errno"), client->response.error_code);
+            zend_update_property_stringl(
+                swoole_mysql_coro_statement_ce_ptr, client->statement->object,
+                ZEND_STRL("error"), client->response.server_msg, client->response.l_server_msg
+            );
+            zend_update_property_long(
+                swoole_mysql_coro_statement_ce_ptr, client->statement->object,
+                ZEND_STRL("errno"), client->response.error_code
+            );
         }
     }
-    //ResultSet
+    // ResultSet
     else
     {
         if (client->connector.fetch_mode && client->cmd == SW_MYSQL_COM_STMT_EXECUTE)
@@ -455,7 +487,7 @@ static int swoole_mysql_coro_parse_response(mysql_client *client, zval **result,
             client->statement->result = client->response.result_array;
             client->response.result_array = NULL;
             // return true (success)]
-            SW_ALLOC_INIT_ZVAL(*result);
+            *result = sw_malloc_zval();
             ZVAL_TRUE(*result);
         }
         else
@@ -464,7 +496,7 @@ static int swoole_mysql_coro_parse_response(mysql_client *client, zval **result,
         }
     }
 
-    return SW_OK;
+    return ret;
 }
 
 static void swoole_mysql_coro_parse_end(mysql_client *client, swString *buffer)
@@ -506,7 +538,7 @@ static int swoole_mysql_coro_statement_free(mysql_statement *stmt)
 
 static int swoole_mysql_coro_statement_close(mysql_statement *stmt)
 {
-    // WARNING: it's wrong operation, we send the close statement package silently, don't change any property in the client!
+    // WARNING: it's wrong operation, we send the close statement packet silently, don't change any property in the client!
     // stmt->client->cmd = SW_MYSQL_COM_STMT_CLOSE;
 
     // call mysql-server to destruct this statement
@@ -554,9 +586,10 @@ static int swoole_mysql_coro_close(zval *zobject)
         mysql_request_buffer->length = 5;
         mysql_pack_length(mysql_request_buffer->length - 4, mysql_request_buffer->str);
         SwooleG.main_reactor->write(SwooleG.main_reactor, client->fd, mysql_request_buffer->str, mysql_request_buffer->length);
+        client->connected = 0;
     }
 
-    zend_update_property_bool(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("connected"), 0);
+    zend_update_property_bool(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("connected"), 0);
     SwooleG.main_reactor->del(SwooleG.main_reactor, client->fd);
 
     swConnection *_socket = swReactor_get(SwooleG.main_reactor, client->fd);
@@ -607,7 +640,6 @@ static PHP_METHOD(swoole_mysql_coro, __destruct)
 static PHP_METHOD(swoole_mysql_coro, connect)
 {
     zval *server_info;
-    char buf[2048];
 
     ZEND_PARSE_PARAMETERS_START(1, 1)
         Z_PARAM_ARRAY(server_info)
@@ -635,7 +667,7 @@ static PHP_METHOD(swoole_mysql_coro, connect)
     }
     else
     {
-        zend_throw_exception(swoole_mysql_coro_exception_class_entry_ptr, "HOST parameter is required.", 11);
+        zend_throw_exception(swoole_mysql_coro_exception_ce_ptr, "HOST parameter is required.", 11);
         zval_ptr_dtor(server_info);
         RETURN_FALSE;
     }
@@ -656,7 +688,7 @@ static PHP_METHOD(swoole_mysql_coro, connect)
     }
     else
     {
-        zend_throw_exception(swoole_mysql_coro_exception_class_entry_ptr, "USER parameter is required.", 11);
+        zend_throw_exception(swoole_mysql_coro_exception_ce_ptr, "USER parameter is required.", 11);
         zval_ptr_dtor(server_info);
         RETURN_FALSE;
     }
@@ -668,7 +700,7 @@ static PHP_METHOD(swoole_mysql_coro, connect)
     }
     else
     {
-        zend_throw_exception(swoole_mysql_coro_exception_class_entry_ptr, "PASSWORD parameter is required.", 11);
+        zend_throw_exception(swoole_mysql_coro_exception_ce_ptr, "PASSWORD parameter is required.", 11);
         zval_ptr_dtor(server_info);
         RETURN_FALSE;
     }
@@ -680,7 +712,7 @@ static PHP_METHOD(swoole_mysql_coro, connect)
     }
     else
     {
-        zend_throw_exception(swoole_mysql_coro_exception_class_entry_ptr, "DATABASE parameter is required.", 11);
+        zend_throw_exception(swoole_mysql_coro_exception_ce_ptr, "DATABASE parameter is required.", 11);
         zval_ptr_dtor(server_info);
         RETURN_FALSE;
     }
@@ -691,7 +723,7 @@ static PHP_METHOD(swoole_mysql_coro, connect)
     }
     else
     {
-        connector->timeout = SW_MYSQL_CONNECT_TIMEOUT;
+        connector->timeout = COROG.socket_connect_timeout;
     }
     if (php_swoole_array_get_value(_ht, "charset", value))
     {
@@ -699,8 +731,9 @@ static PHP_METHOD(swoole_mysql_coro, connect)
         connector->character_set = mysql_get_charset(Z_STRVAL_P(value));
         if (connector->character_set < 0)
         {
+            char buf[64];
             snprintf(buf, sizeof(buf), "unknown charset [%s].", Z_STRVAL_P(value));
-            zend_throw_exception(swoole_mysql_coro_exception_class_entry_ptr, buf, 11);
+            zend_throw_exception(swoole_mysql_coro_exception_ce_ptr, buf, 11);
             zval_ptr_dtor(server_info);
             RETURN_FALSE;
         }
@@ -748,8 +781,8 @@ static PHP_METHOD(swoole_mysql_coro, connect)
     {
         swoole_php_fatal_error(E_WARNING, "swClient_create() failed. Error: %s [%d]", strerror(errno), errno);
         _failed:
-        zend_update_property_string(swoole_mysql_coro_class_entry_ptr, getThis(), ZEND_STRL("connect_error"), strerror(errno));
-        zend_update_property_long(swoole_mysql_coro_class_entry_ptr, getThis(), ZEND_STRL("connect_errno"), errno);
+        zend_update_property_string(swoole_mysql_coro_ce_ptr, getThis(), ZEND_STRL("connect_error"), strerror(errno));
+        zend_update_property_long(swoole_mysql_coro_ce_ptr, getThis(), ZEND_STRL("connect_errno"), errno);
         efree(cli);
         zval_ptr_dtor(server_info);
         RETURN_FALSE;
@@ -778,9 +811,9 @@ static PHP_METHOD(swoole_mysql_coro, connect)
         goto _failed;
     }
 
-    zend_update_property(swoole_mysql_coro_class_entry_ptr, getThis(), ZEND_STRL("serverInfo"), server_info);
+    zend_update_property(swoole_mysql_coro_ce_ptr, getThis(), ZEND_STRL("serverInfo"), server_info);
     zval_ptr_dtor(server_info);
-    zend_update_property_long(swoole_mysql_coro_class_entry_ptr, getThis(), ZEND_STRL("sock"), cli->socket->fd);
+    zend_update_property_long(swoole_mysql_coro_ce_ptr, getThis(), ZEND_STRL("sock"), cli->socket->fd);
 
     if (!client->buffer)
     {
@@ -827,7 +860,7 @@ static PHP_METHOD(swoole_mysql_coro, query)
     if (!client || client->state == SW_MYSQL_STATE_CLOSED)
     {
         SwooleG.error = SW_ERROR_CLIENT_NO_CONNECTION;
-        zend_update_property_long(swoole_mysql_coro_class_entry_ptr, getThis(), ZEND_STRL("errCode"), SwooleG.error);
+        zend_update_property_long(swoole_mysql_coro_ce_ptr, getThis(), ZEND_STRL("errCode"), SwooleG.error);
         swoole_php_fatal_error(E_WARNING, "The MySQL connection is not established.");
         RETURN_FALSE;
     }
@@ -840,7 +873,7 @@ static PHP_METHOD(swoole_mysql_coro, query)
 
     sw_coro_check_bind("mysql client", client->cid);
 
-    double timeout = -1;
+    double timeout = COROG.socket_timeout;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "s|d", &sql.str, &sql.length, &timeout) == FAILURE)
     {
@@ -873,6 +906,7 @@ static PHP_METHOD(swoole_mysql_coro, query)
         client->iowait = SW_MYSQL_CORO_STATUS_WAIT;
         RETURN_TRUE;
     }
+    client->suspending = 1;
     client->cid = sw_get_current_cid();
     sw_coro_save(return_value, context);
     sw_coro_yield();
@@ -927,19 +961,23 @@ static void swoole_mysql_coro_query_transcation(const char* command, uint8_t in_
     // to make sure they know what they are doing
     if (unlikely(client->defer))
     {
-        swoole_php_fatal_error(E_DEPRECATED, "you should not use defer to handle transaction, if you want, please use `query` instead.");
-        client->defer = 0;
+        swoole_php_fatal_error(
+            E_DEPRECATED,
+            "you should not use defer to handle transaction, "
+            "if you want, please use `query` instead."
+        );
+        RETURN_FALSE;
     }
 
     if (in_transaction && client->transaction)
     {
-        zend_throw_exception(swoole_mysql_coro_exception_class_entry_ptr, "There is already an active transaction.", 21);
+        zend_throw_exception(swoole_mysql_coro_exception_ce_ptr, "There is already an active transaction.", 21);
         RETURN_FALSE;
     }
 
     if (!in_transaction && !client->transaction)
     {
-        zend_throw_exception(swoole_mysql_coro_exception_class_entry_ptr, "There is no active transaction.", 22);
+        zend_throw_exception(swoole_mysql_coro_exception_ce_ptr, "There is no active transaction.", 22);
         RETURN_FALSE;
     }
 
@@ -952,7 +990,7 @@ static void swoole_mysql_coro_query_transcation(const char* command, uint8_t in_
     }
     else
     {
-        double timeout = -1;
+        double timeout = COROG.socket_timeout;
         if (zend_parse_parameters(ZEND_NUM_ARGS(), "|d", &timeout) == FAILURE)
         {
             RETURN_FALSE;
@@ -1056,7 +1094,7 @@ static PHP_METHOD(swoole_mysql_coro, prepare)
     if (!client || client->state == SW_MYSQL_STATE_CLOSED)
     {
         SwooleG.error = SW_ERROR_CLIENT_NO_CONNECTION;
-        zend_update_property_long(swoole_mysql_coro_class_entry_ptr, getThis(), ZEND_STRL("errCode"), SwooleG.error);
+        zend_update_property_long(swoole_mysql_coro_ce_ptr, getThis(), ZEND_STRL("errCode"), SwooleG.error);
         swoole_php_fatal_error(E_WARNING, "The MySQL connection is not established.");
         RETURN_FALSE;
     }
@@ -1069,7 +1107,7 @@ static PHP_METHOD(swoole_mysql_coro, prepare)
 
     sw_coro_check_bind("mysql client", client->cid);
 
-    double timeout = -1;
+    double timeout = COROG.socket_timeout;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "s|d", &sql.str, &sql.length, &timeout) == FAILURE)
     {
@@ -1100,9 +1138,9 @@ static PHP_METHOD(swoole_mysql_coro, prepare)
         if (swConnection_error(errno) == SW_CLOSE)
         {
             zval *zobject = getThis();
-            zend_update_property_bool(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("connected"), 0);
-            zend_update_property_long(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("errno"), 2013);
-            zend_update_property_string(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("error"), "Lost connection to MySQL server during query");
+            zend_update_property_bool(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("connected"), 0);
+            zend_update_property_long(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("errno"), 2013);
+            zend_update_property_string(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("error"), "Lost connection to MySQL server during query");
         }
         RETURN_FALSE;
     }
@@ -1141,7 +1179,7 @@ static PHP_METHOD(swoole_mysql_coro_statement, execute)
         RETURN_FALSE;
     }
 
-    double timeout = -1;
+    double timeout = COROG.socket_timeout;
 
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "|ad", &params, &timeout) == FAILURE)
     {
@@ -1375,13 +1413,14 @@ static PHP_METHOD(swoole_mysql_coro, close)
     RETURN_TRUE;
 }
 
-static void swoole_mysql_coro_free_storage(zend_object *object)
+static void swoole_mysql_coro_free_object(zend_object *object)
 {
     // as __destruct
+    uint32_t handle = object->handle;
     zval _zobject, *zobject = &_zobject;
     ZVAL_OBJ(zobject, object);
 
-    mysql_client *client = (mysql_client *) swoole_get_object(zobject);
+    mysql_client *client = (mysql_client *) swoole_get_object_by_handle(handle);
     if (client)
     {
         if (client->state != SW_MYSQL_STATE_CLOSED && client->cli)
@@ -1393,14 +1432,14 @@ static void swoole_mysql_coro_free_storage(zend_object *object)
             swString_free(client->buffer);
         }
         efree(client);
-        swoole_set_object(zobject, NULL);
+        swoole_set_object_by_handle(handle, NULL);
     }
 
-    php_context *context = (php_context *) swoole_get_property(zobject, 0);
+    php_context *context = (php_context *) swoole_get_property_by_handle(handle, 0);
     if (context)
     {
         efree(context);
-        swoole_set_property(zobject, 0, NULL);
+        swoole_set_property_by_handle(handle, 0, NULL);
     }
 
     zend_object_std_dtor(object);
@@ -1408,15 +1447,14 @@ static void swoole_mysql_coro_free_storage(zend_object *object)
 
 static int swoole_mysql_coro_onError(swReactor *reactor, swEvent *event)
 {
-    zval *retval = NULL, *result;
+    zval *retval = NULL, *result = sw_malloc_zval();;
     mysql_client *client = (mysql_client *) event->socket->object;
     zval *zobject = client->object;
 
     swoole_mysql_coro_close(zobject);
 
-    SW_ALLOC_INIT_ZVAL(result);
-    zend_update_property_string(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("connect_error"), "EPOLLERR/EPOLLHUP/EPOLLRDHUP happen!");
-    zend_update_property_long(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("connect_errno"), 104);
+    zend_update_property_string(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("connect_error"), "EPOLLERR/EPOLLHUP/EPOLLRDHUP happen!");
+    zend_update_property_long(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("connect_errno"), ECONNRESET);
     ZVAL_BOOL(result, 0);
     if (client->defer && !client->suspending)
     {
@@ -1456,8 +1494,8 @@ static void swoole_mysql_coro_onConnect(mysql_client *client)
 
     if (client->connector.error_code > 0)
     {
-        zend_update_property_stringl(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("connect_error"), client->connector.error_msg, client->connector.error_length);
-        zend_update_property_long(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("connect_errno"), client->connector.error_code);
+        zend_update_property_stringl(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("connect_error"), client->connector.error_msg, client->connector.error_length);
+        zend_update_property_long(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("connect_errno"), client->connector.error_code);
 
         ZVAL_BOOL(result, 0);
 
@@ -1467,7 +1505,7 @@ static void swoole_mysql_coro_onConnect(mysql_client *client)
     {
         client->state = SW_MYSQL_STATE_QUERY;
         client->iowait = SW_MYSQL_CORO_STATUS_READY;
-        zend_update_property_bool(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("connected"), 1);
+        zend_update_property_bool(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("connected"), 1);
         client->connected = 1;
         ZVAL_BOOL(result, 1);
     }
@@ -1485,28 +1523,26 @@ static void swoole_mysql_coro_onConnect(mysql_client *client)
 
 static void swoole_mysql_coro_onTimeout(swTimer *timer, swTimer_node *tnode)
 {
-    zval *result;
+    zval *result = sw_malloc_zval();;
     zval *retval = NULL;
-
     php_context *ctx = (php_context *) tnode->data;
-
-    SW_ALLOC_INIT_ZVAL(result);
-    ZVAL_BOOL(result, 0);
     zval _zobject = ctx->coro_params;
     zval *zobject = & _zobject;
 
+    ZVAL_BOOL(result, 0);
+
     mysql_client *client = (mysql_client *) swoole_get_object(zobject);
 
     if (client->iowait == SW_MYSQL_CORO_STATUS_CLOSED)
     {
-        zend_update_property_string(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("error"), "connect timeout");
+        zend_update_property_string(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("error"), "connect timeout");
     }
     else
     {
-        zend_update_property_string(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("error"), "query timeout");
+        zend_update_property_string(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("error"), "query timeout");
     }
 
-    zend_update_property_long(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("errno"), ETIMEDOUT);
+    zend_update_property_long(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("errno"), ETIMEDOUT);
 
     //timeout close conncttion
     client->timer = NULL;
@@ -1629,7 +1665,7 @@ static int swoole_mysql_coro_onHandShake(mysql_client *client)
             }
             else
             {
-                // clear for the new package
+                // clear for the new packet
                 swString_clear(buffer);
                 // mysql_handshake will return the next state flag
                 client->handshake = ret;
@@ -1648,7 +1684,7 @@ static int swoole_mysql_coro_onHandShake(mysql_client *client)
         case SW_AGAIN:
             return SW_OK;
         case SW_ERR:
-            // not the switch package, go to the next
+            // not the switch packet, go to the next
             goto _again;
         default:
             ret = next_state;
@@ -1677,7 +1713,7 @@ static int swoole_mysql_coro_onHandShake(mysql_client *client)
         }
         }
 
-        // may be more packages
+        // may be more packets
         if ((size_t) buffer->offset < buffer->length)
         {
             goto _again;
@@ -1692,7 +1728,7 @@ static int swoole_mysql_coro_onHandShake(mysql_client *client)
     {
         // encode by RSA
 #ifdef SW_MYSQL_RSA_SUPPORT
-        switch (mysql_parse_rsa(connector, SWSTRING_CURRENT_VL(buffer)))
+        switch (mysql_parse_rsa(connector, SW_STRINGCVL(buffer)))
         {
         case SW_AGAIN:
             return SW_OK;
@@ -1713,7 +1749,7 @@ static int swoole_mysql_coro_onHandShake(mysql_client *client)
     }
     default:
     {
-        ret = mysql_get_result(connector, SWSTRING_CURRENT_VL(buffer));
+        ret = mysql_get_result(connector, SW_STRINGCVL(buffer));
         if (ret < 0)
         {
             _error:
@@ -1787,18 +1823,9 @@ static int swoole_mysql_coro_onRead(swReactor *reactor, swEvent *event)
                     swSysError("Read from socket[%d] failed.", event->fd);
                     return SW_ERR;
                 case SW_CLOSE:
-                    goto close_fd;
+                    goto _close_fd;
                 case SW_WAIT:
-                    if ((size_t) client->check_offset == buffer->length)
-                    {
-                        // check all of the package but not complete
-                        return SW_OK;
-                    }
-                    else
-                    {
-                        // have already check all of the data
-                        goto parse_response;
-                    }
+                    return SW_OK;
                 default:
                     return SW_ERR;
                 }
@@ -1806,21 +1833,23 @@ static int swoole_mysql_coro_onRead(swReactor *reactor, swEvent *event)
         }
         else if (ret == 0)
         {
-            close_fd:
+            _close_fd:
             if (client->state == SW_MYSQL_STATE_READ_END)
             {
-                goto parse_response;
+                goto _parse_response;
             }
 
 
-            zend_update_property_long(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("connect_errno"), 111);
-            zend_update_property_string(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("connect_error"), "connection close by peer");
+            zend_update_property_long(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("connect_errno"), 111);
+            zend_update_property_string(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("connect_error"), "connection close by peer");
             if (client->connected)
             {
                 client->connected = 0;
-                zend_update_property_long(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("errno"), 2006);
-                zend_update_property_string(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("error"), "MySQL server has gone away");
+                zend_update_property_long(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("errno"), 2006);
+                zend_update_property_string(swoole_mysql_coro_ce_ptr, zobject, ZEND_STRL("error"), "MySQL server has gone away");
             }
+
+            _active_close:
             swoole_mysql_coro_close(zobject);
 
             if (!client->cid)
@@ -1828,7 +1857,7 @@ static int swoole_mysql_coro_onRead(swReactor *reactor, swEvent *event)
                 return SW_OK;
             }
 
-            SW_ALLOC_INIT_ZVAL(result);
+            result = sw_malloc_zval();
             ZVAL_BOOL(result, 0);
             if (client->defer && !client->suspending)
             {
@@ -1863,30 +1892,50 @@ static int swoole_mysql_coro_onRead(swReactor *reactor, swEvent *event)
                 continue;
             }
 
-            parse_response:
+            _parse_response:
 
-            // always check that is package complete
-            // and maybe more responses has already received in buffer, we check it now.
-            if ((client->cmd == SW_MYSQL_COM_QUERY || client->cmd == SW_MYSQL_COM_STMT_EXECUTE) && mysql_is_over(client) != SW_OK)
+            if (client->tmp_result)
             {
-                // the **last** sever status flag shows that more results exist but we hasn't received.
-                swTraceLog(SW_TRACE_MYSQL_CLIENT, "need more");
-                continue;
+                _check_over:
+                // maybe more responses has already received in buffer, we check it now.
+                if (mysql_is_over(client) != SW_OK)
+                {
+                    // the **last** sever status flag shows that more results exist but we hasn't received.
+                    return SW_OK;
+                }
+                else
+                {
+                    result = client->tmp_result;
+                    client->tmp_result = NULL;
+                }
             }
-
-            if (swoole_mysql_coro_parse_response(client, &result, 0) != SW_OK)
+            else
             {
-                return SW_OK;//parse error
+                ret = swoole_mysql_coro_parse_response(client, &result, 0);
+                if (ret == SW_AGAIN)
+                {
+                    return SW_OK; // parse error or need again
+                }
+                if (client->response.status_code & SW_MYSQL_SERVER_MORE_RESULTS_EXISTS)
+                {
+                    client->tmp_result = result;
+                    goto _check_over;
+                }
             }
             swoole_mysql_coro_parse_end(client, buffer); // ending tidy up
 
-
             if (client->defer && !client->suspending)
             {
                 client->iowait = SW_MYSQL_CORO_STATUS_DONE;
                 client->result = result;
                 return SW_OK;
             }
+
+            if (!client->cid)
+            {
+                goto _active_close; // error
+            }
+
             client->suspending = 0;
             client->iowait = SW_MYSQL_CORO_STATUS_READY;
             client->cid = 0;
