@@ -116,21 +116,19 @@ static const zend_function_entry swoole_postgresql_coro_methods[] =
 };
 
 static zend_class_entry swoole_postgresql_coro_ce;
-static zend_class_entry *swoole_postgresql_coro_class_entry_ptr;
+static zend_class_entry *swoole_postgresql_coro_ce_ptr;
+static zend_object_handlers swoole_postgresql_coro_handlers;
 static int le_result;
 
 void swoole_postgresql_coro_init(int module_number)
 {
 
-    INIT_CLASS_ENTRY(swoole_postgresql_coro_ce, "Swoole\\Coroutine\\PostgreSQL", swoole_postgresql_coro_methods);
+    SWOOLE_INIT_CLASS_ENTRY(swoole_postgresql_coro, "Swoole\\Coroutine\\PostgreSQL", NULL, "Co\\PostgreSQL", swoole_postgresql_coro_methods);
+    SWOOLE_SET_CLASS_SERIALIZABLE(swoole_postgresql_coro, zend_class_serialize_deny, zend_class_unserialize_deny);
+    SWOOLE_SET_CLASS_CLONEABLE(swoole_postgresql_coro, zend_class_clone_deny);
+    SWOOLE_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_postgresql_coro, zend_class_unset_property_deny);
     le_result = zend_register_list_destructors_ex(_free_result, NULL, "pgsql result", module_number);
-    swoole_postgresql_coro_class_entry_ptr = zend_register_internal_class(&swoole_postgresql_coro_ce TSRMLS_CC);
-    zend_declare_property_null(swoole_postgresql_coro_class_entry_ptr, "error", 5, ZEND_ACC_PUBLIC);
-
-    if (SWOOLE_G(use_shortname))
-    {
-        sw_zend_register_class_alias("Co\\PostgreSQL", swoole_postgresql_coro_class_entry_ptr);
-    }
+    zend_declare_property_null(swoole_postgresql_coro_ce_ptr, "error", 5, ZEND_ACC_PUBLIC);
 
     REGISTER_LONG_CONSTANT("SW_PGSQL_ASSOC", PGSQL_ASSOC, CONST_CS | CONST_PERSISTENT);
     REGISTER_LONG_CONSTANT("SW_PGSQL_NUM", PGSQL_NUM, CONST_CS | CONST_PERSISTENT);
@@ -256,7 +254,7 @@ static void swoole_pgsql_coro_onTimeout(swTimer *timer, swTimer_node *tnode)
         PQfinish(pgsql);
     }
 
-    zend_update_property_string(swoole_postgresql_coro_class_entry_ptr, zobject, "error", 5, "ontimeout");
+    zend_update_property_string(swoole_postgresql_coro_ce_ptr, zobject, "error", 5, "ontimeout");
     int ret = sw_coro_resume(ctx, result, retval);
     if (ret == CORO_END && retval)
     {
@@ -322,7 +320,7 @@ static int swoole_pgsql_coro_onWrite(swReactor *reactor, swEvent *event)
             {
                 err_msg = PQerrorMessage(object->conn);
                 swWarn("error:[%s] please cofirm that the connection configuration is correct \n",err_msg);
-                zend_update_property_string(swoole_postgresql_coro_class_entry_ptr, object->object, "error", 5, err_msg);
+                zend_update_property_string(swoole_postgresql_coro_ce_ptr, object->object, "error", 5, err_msg);
                 break;
             }
         }
@@ -340,7 +338,7 @@ static int swoole_pgsql_coro_onWrite(swReactor *reactor, swEvent *event)
     ZVAL_BOOL(&return_value, success);
     if (success == 1)
     {
-        zend_update_property_null(swoole_postgresql_coro_class_entry_ptr, object->object, "error", 5);
+        zend_update_property_null(swoole_postgresql_coro_ce_ptr, object->object, "error", 5);
     }
 
     int ret = sw_coro_resume(context, &return_value, retval);
@@ -428,7 +426,7 @@ static  int meta_data_result_parse(pg_object *object)
 
     }
     php_context *context = (php_context *) swoole_get_property(object->object, 0);
-    zend_update_property_null(swoole_postgresql_coro_class_entry_ptr, object->object, "error", 5);
+    zend_update_property_null(swoole_postgresql_coro_ce_ptr, object->object, "error", 5);
     int ret = sw_coro_resume(context, &return_value, retval);
     if (ret == CORO_END && retval)
     {
@@ -465,7 +463,7 @@ static  int query_result_parse(pg_object *object)
 
             PQclear(pgsql_result);
             ZVAL_FALSE(&return_value);
-            zend_update_property_string(swoole_postgresql_coro_class_entry_ptr, object->object, "error", 5, err_msg);
+            zend_update_property_string(swoole_postgresql_coro_ce_ptr, object->object, "error", 5, err_msg);
             ret = sw_coro_resume(context, &return_value, retval);
             if (ret == CORO_END && retval)
             {
@@ -480,7 +478,7 @@ static  int query_result_parse(pg_object *object)
             res = PQflush(object->conn);
 
             ZVAL_RES(&return_value, zend_register_resource(object, le_result));
-            zend_update_property_null(swoole_postgresql_coro_class_entry_ptr, object->object, "error", 5);
+            zend_update_property_null(swoole_postgresql_coro_ce_ptr, object->object, "error", 5);
             ret = sw_coro_resume(context, &return_value, retval);
             if (ret == CORO_END && retval)
             {
@@ -1030,7 +1028,7 @@ static int swoole_pgsql_coro_onError(swReactor *reactor, swEvent *event)
     ZVAL_BOOL(result, 0);
 
     php_context *context = (php_context *) swoole_get_property(zobject, 0);
-    zend_update_property_string(swoole_postgresql_coro_class_entry_ptr, zobject, "error", 5, "onerror");
+    zend_update_property_string(swoole_postgresql_coro_ce_ptr, zobject, "error", 5, "onerror");
     int ret = sw_coro_resume(context, result, retval);
     zval_ptr_dtor(result);
 
