@@ -105,7 +105,8 @@ static void swoole_redis_event_DelWrite(void *privdata);
 static void swoole_redis_event_Cleanup(void *privdata);
 
 static zend_class_entry swoole_redis_ce;
-static zend_class_entry *swoole_redis_class_entry_ptr;
+static zend_class_entry *swoole_redis_ce_ptr;
+static zend_object_handlers swoole_redis_handlers;
 
 static const zend_function_entry swoole_redis_methods[] =
 {
@@ -150,7 +151,7 @@ static sw_inline void redis_execute_connect_callback(swRedisClient *redis, int s
     ZVAL_BOOL(result, success);
 
     zval args[2];
-    zval *zcallback = sw_zend_read_property(swoole_redis_class_entry_ptr, redis->object, ZEND_STRL("onConnect"), 0);
+    zval *zcallback = sw_zend_read_property(swoole_redis_ce_ptr, redis->object, ZEND_STRL("onConnect"), 0);
     args[0] = *redis->object;
     args[1] = *result;
     redis->connecting = 1;
@@ -158,11 +159,11 @@ static sw_inline void redis_execute_connect_callback(swRedisClient *redis, int s
     {
         swoole_php_fatal_error(E_WARNING, "swoole_async_redis connect_callback handler error.");
     }
-    if (EG(exception))
+    if (UNEXPECTED(EG(exception)))
     {
         zend_exception_error(EG(exception), E_ERROR);
     }
-    if (retval != NULL)
+    if (retval)
     {
         zval_ptr_dtor(retval);
     }
@@ -172,31 +173,35 @@ static sw_inline void redis_execute_connect_callback(swRedisClient *redis, int s
 
 void swoole_redis_init(int module_number)
 {
-    SWOOLE_INIT_CLASS_ENTRY(swoole_redis_ce, "swoole_redis", "Swoole\\Redis", swoole_redis_methods);
-    swoole_redis_class_entry_ptr = zend_register_internal_class(&swoole_redis_ce);
-    SWOOLE_CLASS_ALIAS(swoole_redis, "Swoole\\Redis");
+    SWOOLE_INIT_CLASS_ENTRY(swoole_redis, "Swoole\\Redis", "swoole_redis", NULL, swoole_redis_methods);
+    SWOOLE_SET_CLASS_SERIALIZABLE(swoole_redis, zend_class_serialize_deny, zend_class_unserialize_deny);
+    SWOOLE_SET_CLASS_CLONEABLE(swoole_redis, zend_class_clone_deny);
+    SWOOLE_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_redis, zend_class_unset_property_deny);
 
-    zend_declare_property_null(swoole_redis_class_entry_ptr, ZEND_STRL("onConnect"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_redis_class_entry_ptr, ZEND_STRL("onClose"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_redis_class_entry_ptr, ZEND_STRL("onMessage"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_redis_ce_ptr, ZEND_STRL("onConnect"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_redis_ce_ptr, ZEND_STRL("onClose"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_redis_ce_ptr, ZEND_STRL("onMessage"), ZEND_ACC_PUBLIC);
 
-    zend_declare_property_null(swoole_redis_class_entry_ptr, ZEND_STRL("setting"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_redis_class_entry_ptr, ZEND_STRL("host"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_redis_class_entry_ptr, ZEND_STRL("port"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_redis_class_entry_ptr, ZEND_STRL("sock"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_redis_class_entry_ptr, ZEND_STRL("errCode"), ZEND_ACC_PUBLIC);
-    zend_declare_property_null(swoole_redis_class_entry_ptr, ZEND_STRL("errMsg"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_redis_ce_ptr, ZEND_STRL("setting"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_redis_ce_ptr, ZEND_STRL("host"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_redis_ce_ptr, ZEND_STRL("port"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_redis_ce_ptr, ZEND_STRL("sock"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_redis_ce_ptr, ZEND_STRL("errCode"), ZEND_ACC_PUBLIC);
+    zend_declare_property_null(swoole_redis_ce_ptr, ZEND_STRL("errMsg"), ZEND_ACC_PUBLIC);
 
-    zend_declare_class_constant_long(swoole_redis_class_entry_ptr, ZEND_STRL("STATE_CONNECT"), SWOOLE_REDIS_STATE_CONNECT);
-    zend_declare_class_constant_long(swoole_redis_class_entry_ptr, ZEND_STRL("STATE_READY"), SWOOLE_REDIS_STATE_READY);
-    zend_declare_class_constant_long(swoole_redis_class_entry_ptr, ZEND_STRL("STATE_WAIT_RESULT"), SWOOLE_REDIS_STATE_WAIT_RESULT);
-    zend_declare_class_constant_long(swoole_redis_class_entry_ptr, ZEND_STRL("STATE_SUBSCRIBE"), SWOOLE_REDIS_STATE_SUBSCRIBE);
-    zend_declare_class_constant_long(swoole_redis_class_entry_ptr, ZEND_STRL("STATE_CLOSED"), SWOOLE_REDIS_STATE_CLOSED);
+    zend_declare_class_constant_long(swoole_redis_ce_ptr, ZEND_STRL("STATE_CONNECT"), SWOOLE_REDIS_STATE_CONNECT);
+    zend_declare_class_constant_long(swoole_redis_ce_ptr, ZEND_STRL("STATE_READY"), SWOOLE_REDIS_STATE_READY);
+    zend_declare_class_constant_long(swoole_redis_ce_ptr, ZEND_STRL("STATE_WAIT_RESULT"), SWOOLE_REDIS_STATE_WAIT_RESULT);
+    zend_declare_class_constant_long(swoole_redis_ce_ptr, ZEND_STRL("STATE_SUBSCRIBE"), SWOOLE_REDIS_STATE_SUBSCRIBE);
+    zend_declare_class_constant_long(swoole_redis_ce_ptr, ZEND_STRL("STATE_CLOSED"), SWOOLE_REDIS_STATE_CLOSED);
 }
 
 static PHP_METHOD(swoole_redis, __construct)
 {
     zval *zset = NULL;
+
+    swoole_php_fatal_error(E_DEPRECATED, "async APIs will be removed in Swoole-v4.3.0, you should be using the coroutine APIs instead.");
+
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "|z", &zset) == FAILURE)
     {
         RETURN_FALSE;
@@ -212,7 +217,7 @@ static PHP_METHOD(swoole_redis, __construct)
     if (zset && ZVAL_IS_ARRAY(zset))
     {
         php_swoole_array_separate(zset);
-        zend_update_property(swoole_redis_class_entry_ptr, getThis(), ZEND_STRL("setting"), zset);
+        zend_update_property(swoole_redis_ce_ptr, getThis(), ZEND_STRL("setting"), zset);
         zval_ptr_dtor(zset);
 
         HashTable *vht;
@@ -283,12 +288,12 @@ static PHP_METHOD(swoole_redis, on)
 
     if (strncasecmp("close", name, len) == 0)
     {
-        zend_update_property(swoole_redis_class_entry_ptr, getThis(), ZEND_STRL("onClose"), cb);
+        zend_update_property(swoole_redis_ce_ptr, getThis(), ZEND_STRL("onClose"), cb);
     }
     else if (strncasecmp("message", name, len) == 0)
     {
-        zend_update_property(swoole_redis_class_entry_ptr, getThis(), ZEND_STRL("onMessage"), cb);
-        redis->message_callback = sw_zend_read_property(swoole_redis_class_entry_ptr, getThis(), ZEND_STRL("onMessage"), 0);
+        zend_update_property(swoole_redis_ce_ptr, getThis(), ZEND_STRL("onMessage"), cb);
+        redis->message_callback = sw_zend_read_property(swoole_redis_ce_ptr, getThis(), ZEND_STRL("onMessage"), 0);
         sw_copy_to_stack(redis->message_callback, redis->_message_callback);
 
         redis->subscribe = 1;
@@ -313,9 +318,9 @@ static PHP_METHOD(swoole_redis, connect)
         RETURN_FALSE;
     }
 
-    if (host_len <= 0)
+    if (host_len == 0)
     {
-        swoole_php_error(E_WARNING, "redis server host is empty.");
+        swoole_php_error(E_ERROR, "redis server host is empty.");
         RETURN_FALSE;
     }
 
@@ -360,8 +365,8 @@ static PHP_METHOD(swoole_redis, connect)
     redisAsyncSetConnectCallback(context, swoole_redis_onConnect);
     redisAsyncSetDisconnectCallback(context, swoole_redis_onClose);
 
-    zend_update_property_long(swoole_redis_class_entry_ptr, getThis(), ZEND_STRL("sock"), context->c.fd);
-    zend_update_property(swoole_redis_class_entry_ptr, getThis(), ZEND_STRL("onConnect"), callback);
+    zend_update_property_long(swoole_redis_ce_ptr, getThis(), ZEND_STRL("sock"), context->c.fd);
+    zend_update_property(swoole_redis_ce_ptr, getThis(), ZEND_STRL("onConnect"), callback);
 
     redis->context = context;
     context->ev.addRead = swoole_redis_event_AddRead;
@@ -371,8 +376,8 @@ static PHP_METHOD(swoole_redis, connect)
     context->ev.cleanup = swoole_redis_event_Cleanup;
     context->ev.data = redis;
 
-    zend_update_property_string(swoole_redis_class_entry_ptr, getThis(), ZEND_STRL("host"), host);
-    zend_update_property_long(swoole_redis_class_entry_ptr, getThis(), ZEND_STRL("port"), port);
+    zend_update_property_string(swoole_redis_ce_ptr, getThis(), ZEND_STRL("host"), host);
+    zend_update_property_long(swoole_redis_ce_ptr, getThis(), ZEND_STRL("port"), port);
 
     if (SwooleG.main_reactor->add(SwooleG.main_reactor, redis->context->c.fd, PHP_SWOOLE_FD_REDIS | SW_EVENT_WRITE) < 0)
     {
@@ -639,19 +644,15 @@ static void swoole_redis_set_error(swRedisClient *redis, zval* return_value, red
     str[reply->len] = 0;
 
     ZVAL_FALSE(return_value);
-    zend_update_property_long(swoole_redis_class_entry_ptr, redis->object, ZEND_STRL("errCode"), -1);
-    zend_update_property_string(swoole_redis_class_entry_ptr, redis->object, ZEND_STRL("errMsg"), str);
+    zend_update_property_long(swoole_redis_ce_ptr, redis->object, ZEND_STRL("errCode"), -1);
+    zend_update_property_string(swoole_redis_ce_ptr, redis->object, ZEND_STRL("errMsg"), str);
     free(str);
 }
 
 static void swoole_redis_parse_result(swRedisClient *redis, zval* return_value, redisReply* reply)
 {
-    zval *val;
     int j;
-
-    zval _val;
-    val = &_val;
-    bzero(val, sizeof(zval));
+    zval _val, *val = &_val;
 
     switch (reply->type)
     {
@@ -677,8 +678,8 @@ static void swoole_redis_parse_result(swRedisClient *redis, zval* return_value,
         }
         else
         {
-            zend_update_property_long(swoole_redis_class_entry_ptr, redis->object, ZEND_STRL("errCode"), redis->context->err);
-            zend_update_property_string(swoole_redis_class_entry_ptr, redis->object, ZEND_STRL("errMsg"), redis->context->errstr);
+            zend_update_property_long(swoole_redis_ce_ptr, redis->object, ZEND_STRL("errCode"), redis->context->err);
+            zend_update_property_string(swoole_redis_ce_ptr, redis->object, ZEND_STRL("errMsg"), redis->context->errstr);
         }
         break;
 
@@ -706,8 +707,8 @@ static void swoole_redis_onTimeout(swTimer *timer, swTimer_node *tnode)
 {
     swRedisClient *redis = tnode->data;
     redis->timer = NULL;
-    zend_update_property_long(swoole_redis_class_entry_ptr, redis->object, ZEND_STRL("errCode"), ETIMEDOUT);
-    zend_update_property_string(swoole_redis_class_entry_ptr, redis->object, ZEND_STRL("errMsg"), strerror(ETIMEDOUT));
+    zend_update_property_long(swoole_redis_ce_ptr, redis->object, ZEND_STRL("errCode"), ETIMEDOUT);
+    zend_update_property_string(swoole_redis_ce_ptr, redis->object, ZEND_STRL("errMsg"), strerror(ETIMEDOUT));
     redis->state = SWOOLE_REDIS_STATE_CLOSED;
     redis_execute_connect_callback(redis, 0);
     if (redis->context)
@@ -731,8 +732,8 @@ static void swoole_redis_onCompleted(redisAsyncContext *c, void *r, void *privda
         switch (reply->type)
         {
         case REDIS_REPLY_ERROR:
-            zend_update_property_long(swoole_redis_class_entry_ptr, redis->object, ZEND_STRL("errCode"), 0);
-            zend_update_property_stringl(swoole_redis_class_entry_ptr, redis->object, ZEND_STRL("errMsg"), reply->str,
+            zend_update_property_long(swoole_redis_ce_ptr, redis->object, ZEND_STRL("errCode"), 0);
+            zend_update_property_stringl(swoole_redis_ce_ptr, redis->object, ZEND_STRL("errMsg"), reply->str,
                     reply->len);
             redis->failure = 1;
             break;
@@ -744,9 +745,9 @@ static void swoole_redis_onCompleted(redisAsyncContext *c, void *r, void *privda
             }
             else
             {
-                zend_update_property_long(swoole_redis_class_entry_ptr, redis->object, ZEND_STRL("errCode"),
+                zend_update_property_long(swoole_redis_ce_ptr, redis->object, ZEND_STRL("errCode"),
                         redis->context->err);
-                zend_update_property_string(swoole_redis_class_entry_ptr, redis->object, ZEND_STRL("errMsg"),
+                zend_update_property_string(swoole_redis_ce_ptr, redis->object, ZEND_STRL("errMsg"),
                         redis->context->errstr);
                 redis->failure = 1;
             }
@@ -763,7 +764,7 @@ static void swoole_redis_onCompleted(redisAsyncContext *c, void *r, void *privda
             redis->connecting = 0;
             zval *retval = NULL;
             zval *zobject = redis->object;
-            sw_zend_call_method_with_0_params(&zobject, swoole_redis_class_entry_ptr, NULL, "close", &retval);
+            sw_zend_call_method_with_0_params(&zobject, swoole_redis_ce_ptr, NULL, "close", &retval);
             if (retval)
             {
                 zval_ptr_dtor(retval);
@@ -819,11 +820,11 @@ static void swoole_redis_onResult(redisAsyncContext *c, void *r, void *privdata)
     {
         swoole_php_fatal_error(E_WARNING, "swoole_redis callback[%s] handler error.", callback_type);
     }
-    if (EG(exception))
+    if (UNEXPECTED(EG(exception)))
     {
         zend_exception_error(EG(exception), E_ERROR);
     }
-    if (retval != NULL)
+    if (retval)
     {
         zval_ptr_dtor(retval);
     }
@@ -846,8 +847,8 @@ void swoole_redis_onConnect(const redisAsyncContext *c, int status)
 
     if (status != REDIS_OK)
     {
-        zend_update_property_long(swoole_redis_class_entry_ptr, redis->object, ZEND_STRL("errCode"), errno);
-        zend_update_property_string(swoole_redis_class_entry_ptr, redis->object, ZEND_STRL("errMsg"), c->errstr);
+        zend_update_property_long(swoole_redis_ce_ptr, redis->object, ZEND_STRL("errCode"), errno);
+        zend_update_property_string(swoole_redis_ce_ptr, redis->object, ZEND_STRL("errMsg"), c->errstr);
         redis->state = SWOOLE_REDIS_STATE_CLOSED;
         redis_execute_connect_callback(redis, 0);
         SwooleG.main_reactor->defer(SwooleG.main_reactor, redis_free_object, redis->object);
@@ -881,21 +882,21 @@ void swoole_redis_onClose(const redisAsyncContext *c, int status)
     redis->state = SWOOLE_REDIS_STATE_CLOSED;
     redis->context = NULL;
 
-    zval *zcallback = sw_zend_read_property(swoole_redis_class_entry_ptr, redis->object, ZEND_STRL("onClose"), 1);
+    zval *zcallback = sw_zend_read_property(swoole_redis_ce_ptr, redis->object, ZEND_STRL("onClose"), 1);
     if (zcallback && !ZVAL_IS_NULL(zcallback))
     {
-        zval *retval;
+        zval *retval = NULL;
         zval args[1];
         args[0] = *redis->object;
         if (sw_call_user_function_ex(EG(function_table), NULL, zcallback, &retval, 1, args, 0, NULL) != SUCCESS)
         {
             swoole_php_fatal_error(E_WARNING, "swoole_async_redis close_callback handler error.");
         }
-        if (EG(exception))
+        if (UNEXPECTED(EG(exception)))
         {
             zend_exception_error(EG(exception), E_ERROR);
         }
-        if (retval != NULL)
+        if (retval)
         {
             zval_ptr_dtor(retval);
         }
@@ -907,7 +908,7 @@ void swoole_redis_onClose(const redisAsyncContext *c, int status)
 static int swoole_redis_onError(swReactor *reactor, swEvent *event)
 {
     swRedisClient *redis = event->socket->object;
-    zval *zcallback = sw_zend_read_property(swoole_redis_class_entry_ptr, redis->object, ZEND_STRL("onConnect"), 0);
+    zval *zcallback = sw_zend_read_property(swoole_redis_ce_ptr, redis->object, ZEND_STRL("onConnect"), 0);
 
     if (!ZVAL_IS_NULL(zcallback))
     {
@@ -915,8 +916,8 @@ static int swoole_redis_onError(swReactor *reactor, swEvent *event)
         zval *result;
         SW_MAKE_STD_ZVAL(result);
         ZVAL_BOOL(result, 0);
-        zend_update_property_long(swoole_redis_class_entry_ptr, redis->object, ZEND_STRL("errCode"), c->err);
-        zend_update_property_string(swoole_redis_class_entry_ptr, redis->object, ZEND_STRL("errMsg"), c->errstr);
+        zend_update_property_long(swoole_redis_ce_ptr, redis->object, ZEND_STRL("errCode"), c->err);
+        zend_update_property_string(swoole_redis_ce_ptr, redis->object, ZEND_STRL("errMsg"), c->errstr);
 
         redis->state = SWOOLE_REDIS_STATE_CLOSED;
 
@@ -929,11 +930,11 @@ static int swoole_redis_onError(swReactor *reactor, swEvent *event)
         {
             swoole_php_fatal_error(E_WARNING, "swoole_async_redis connect_callback handler error.");
         }
-        if (EG(exception))
+        if (UNEXPECTED(EG(exception)))
         {
             zend_exception_error(EG(exception), E_ERROR);
         }
-        if (retval != NULL)
+        if (retval)
         {
             zval_ptr_dtor(retval);
         }
@@ -942,7 +943,7 @@ static int swoole_redis_onError(swReactor *reactor, swEvent *event)
         redis->connecting = 0;
         retval = NULL;
         zval *zobject = redis->object;
-        sw_zend_call_method_with_0_params(&zobject, swoole_redis_class_entry_ptr, NULL, "close", &retval);
+        sw_zend_call_method_with_0_params(&zobject, swoole_redis_ce_ptr, NULL, "close", &retval);
         if (retval)
         {
             zval_ptr_dtor(retval);
