@@ -20,20 +20,6 @@
 #include <unordered_map>
 #include <initializer_list>
 
-enum hook_type
-{
-    SW_HOOK_FILE = 1u << 1,
-    SW_HOOK_SLEEP = 1u << 2,
-    SW_HOOK_TCP = 1u << 3,
-    SW_HOOK_UDP = 1u << 4,
-    SW_HOOK_UNIX = 1u << 5,
-    SW_HOOK_UDG = 1u << 6,
-    SW_HOOK_SSL = 1u << 7,
-    SW_HOOK_TLS = 1u << 8,
-    SW_HOOK_BLOCKING_FUNCTION = 1u << 9,
-    SW_HOOK_ALL = 0x7fffffff,
-};
-
 using namespace swoole;
 using namespace std;
 
@@ -63,7 +49,9 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_runtime_enableCoroutine, 0, 0, 0)
     ZEND_ARG_INFO(0, flags)
 ZEND_END_ARG_INFO()
 
-static zend_class_entry *ce;
+static zend_class_entry swoole_runtime_ce;
+static zend_class_entry *swoole_runtime_ce_ptr;
+static zend_object_handlers swoole_runtime_handlers;
 
 static php_stream_ops socket_ops
 {
@@ -104,6 +92,7 @@ static struct
     nullptr,
 #ifdef SW_USE_OPENSSL
     nullptr,
+    nullptr,
 #endif
 };
 
@@ -138,9 +127,10 @@ static const zend_function_entry swoole_runtime_methods[] =
 
 void swoole_runtime_init(int module_number)
 {
-    static zend_class_entry _ce;
-    INIT_CLASS_ENTRY(_ce, "Swoole\\Runtime", swoole_runtime_methods);
-    ce = zend_register_internal_class(&_ce);
+    SWOOLE_INIT_CLASS_ENTRY(swoole_runtime, "Swoole\\Runtime", "swoole_runtime", NULL, swoole_runtime_methods);
+    SWOOLE_SET_CLASS_SERIALIZABLE(swoole_runtime, zend_class_serialize_deny, zend_class_unserialize_deny);
+    SWOOLE_SET_CLASS_CLONEABLE(swoole_runtime, zend_class_clone_deny);
+    SWOOLE_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_runtime, zend_class_unset_property_deny);
 
     SWOOLE_DEFINE(HOOK_FILE);
     SWOOLE_DEFINE(HOOK_SLEEP);
@@ -154,12 +144,26 @@ void swoole_runtime_init(int module_number)
     SWOOLE_DEFINE(HOOK_ALL);
 }
 
-static auto block_io_functions =
-{ "sleep", "usleep", "time_nanosleep", "time_sleep_until", "file_get_contents", "curl_init", "stream_select",
-        "socket_select", "gethostbyname", };
+static auto block_io_functions = {
+    "sleep",
+    "usleep",
+    "time_nanosleep",
+    "time_sleep_until",
+    "file_get_contents",
+    "curl_init",
+    "stream_select",
+    "stream_create",
+    "pcntl_fork",
+    "popen",
+    "socket_select",
+    "gethostbyname",
+};
 
-static auto block_io_classes =
-{ "redis", "mysqli", };
+static auto block_io_classes = {
+    "redis", "pdo", "mysqli",
+};
+
+static bool enable_strict_mode = false;
 
 static PHP_METHOD(swoole_runtime, enableStrictMode)
 {
@@ -811,13 +815,57 @@ static int socket_set_option(php_stream *stream, int option, int value, void *pt
     return 0;
 }
 
-static php_stream *socket_create(const char *proto, size_t protolen, const char *resourcename, size_t resourcenamelen,
-        const char *persistent_id, int options, int flags, struct timeval *timeout, php_stream_context *context
-        STREAMS_DC)
+static php_stream *php_socket_create(
+    const char *proto, size_t protolen, const char *resourcename, size_t resourcenamelen,
+    const char *persistent_id, int options, int flags, struct timeval *timeout, php_stream_context *context
+    STREAMS_DC
+)
+{
+    php_stream_transport_factory ori_call;
+
+    if (strncmp(proto, "unix", protolen) == 0)
+    {
+        ori_call = ori_factory._unix;
+    }
+    else if (strncmp(proto, "udp", protolen) == 0)
+    {
+        ori_call = ori_factory.udp;
+    }
+    else if (strncmp(proto, "udg", protolen) == 0)
+    {
+        ori_call = ori_factory.udg;
+    }
+#ifdef SW_USE_OPENSSL
+    else if (strncmp(proto, "ssl", protolen) == 0)
+    {
+        ori_call = ori_factory.ssl;
+    }
+    else if (strncmp(proto, "tls", protolen) == 0)
+    {
+        ori_call = ori_factory.tls;
+    }
+#endif
+    else
+    {
+        ori_call = ori_factory.tcp;
+    }
+    return ori_call(proto, protolen, resourcename, resourcenamelen, persistent_id, options, flags, timeout, context STREAMS_CC);
+}
+
+static php_stream *socket_create(
+    const char *proto, size_t protolen, const char *resourcename, size_t resourcenamelen,
+    const char *persistent_id, int options, int flags, struct timeval *timeout, php_stream_context *context
+    STREAMS_DC
+)
 {
     php_stream *stream = NULL;
     Socket *sock;
 
+    if (unlikely(SwooleG.main_reactor == nullptr || !coroutine_get_current()))
+    {
+        return php_socket_create(proto, protolen, resourcename, resourcenamelen, persistent_id, options, flags, timeout, context STREAMS_CC);
+    }
+
     php_swoole_check_reactor();
 
     if (strncmp(proto, "unix", protolen) == 0)
@@ -833,7 +881,7 @@ static php_stream *socket_create(const char *proto, size_t protolen, const char
         sock = new Socket(SW_SOCK_UNIX_DGRAM);
     }
 #ifdef SW_USE_OPENSSL
-    else if (strncmp(proto, "ssl", protolen) == 0)
+    else if (strncmp(proto, "ssl", protolen) == 0 || strncmp(proto, "tls", protolen) == 0)
     {
         sock = new Socket(SW_SOCK_TCP);
         sock->open_ssl = true;
@@ -873,6 +921,161 @@ static php_stream *socket_create(const char *proto, size_t protolen, const char
     return stream;
 }
 
+bool sw_enable_coroutine_hook(int flags)
+{
+    if (unlikely(enable_strict_mode))
+    {
+        swoole_php_fatal_error(E_ERROR, "unable to enable the coroutine mode after you enable the strict mode.");
+    }
+    if (hook_init)
+    {
+        return false;
+    }
+    hook_flags = flags;
+    hook_init = true;
+    HashTable *xport_hash = php_stream_xport_get_hash();
+
+    if (flags & SW_HOOK_FILE)
+    {
+        memcpy((void*) &ori_php_plain_files_wrapper, &php_plain_files_wrapper, sizeof(php_plain_files_wrapper));
+        memcpy((void*) &php_plain_files_wrapper, &sw_php_plain_files_wrapper, sizeof(php_plain_files_wrapper));
+    }
+    if (flags & SW_HOOK_SLEEP)
+    {
+        ori_sleep = (zend_function *) zend_hash_str_find_ptr(EG(function_table), ZEND_STRL("sleep"));
+        if (ori_sleep)
+        {
+            ori_sleep_handler =  ori_sleep->internal_function.handler;
+            ori_sleep->internal_function.handler = PHP_FN(_sleep);
+        }
+        ori_usleep = (zend_function *) zend_hash_str_find_ptr(EG(function_table), ZEND_STRL("usleep"));
+        if (ori_usleep)
+        {
+            ori_usleep_handler =  ori_usleep->internal_function.handler;
+            ori_usleep->internal_function.handler = PHP_FN(_usleep);
+        }
+        ori_time_nanosleep = (zend_function *) zend_hash_str_find_ptr(EG(function_table), ZEND_STRL("time_nanosleep"));
+        if (ori_time_nanosleep)
+        {
+            ori_time_nanosleep_handler =  ori_time_nanosleep->internal_function.handler;
+            ori_time_nanosleep->internal_function.handler = PHP_FN(_time_nanosleep);
+        }
+        ori_time_sleep_until = (zend_function *) zend_hash_str_find_ptr(EG(function_table), ZEND_STRL("time_sleep_until"));
+        if (ori_time_sleep_until)
+        {
+            ori_time_sleep_until_handler =  ori_time_sleep_until->internal_function.handler;
+            ori_time_sleep_until->internal_function.handler = PHP_FN(_time_sleep_until);
+        }
+    }
+    if (flags & SW_HOOK_BLOCKING_FUNCTION)
+    {
+        ori_gethostbyname = (zend_function *) zend_hash_str_find_ptr(EG(function_table), ZEND_STRL("gethostbyname"));
+        if (ori_gethostbyname)
+        {
+            ori_gethostbyname_handler =  ori_gethostbyname->internal_function.handler;
+            ori_gethostbyname->internal_function.handler = PHP_FN(swoole_coroutine_gethostbyname);
+        }
+    }
+    if (flags & SW_HOOK_TCP)
+    {
+        ori_factory.tcp = (php_stream_transport_factory) zend_hash_str_find_ptr(xport_hash, ZEND_STRL("tcp"));
+        php_stream_xport_register("tcp", socket_create);
+    }
+    if (flags & SW_HOOK_UNIX)
+    {
+        ori_factory._unix = (php_stream_transport_factory) zend_hash_str_find_ptr(xport_hash, ZEND_STRL("unix"));
+        php_stream_xport_register("unix", socket_create);
+    }
+    if (flags & SW_HOOK_UDG)
+    {
+        ori_factory._unix = (php_stream_transport_factory) zend_hash_str_find_ptr(xport_hash, ZEND_STRL("udg"));
+        php_stream_xport_register("udg", socket_create);
+    }
+    if (flags & SW_HOOK_UDP)
+    {
+        ori_factory._unix = (php_stream_transport_factory) zend_hash_str_find_ptr(xport_hash, ZEND_STRL("udp"));
+        php_stream_xport_register("udp", socket_create);
+    }
+#ifdef SW_USE_OPENSSL
+    if (flags & SW_HOOK_SSL)
+    {
+        ori_factory.ssl = (php_stream_transport_factory) zend_hash_str_find_ptr(xport_hash, ZEND_STRL("ssl"));
+        php_stream_xport_register("ssl", socket_create);
+    }
+    if (flags & SW_HOOK_TLS)
+    {
+        ori_factory.tls = (php_stream_transport_factory) zend_hash_str_find_ptr(xport_hash, ZEND_STRL("tls"));
+        php_stream_xport_register("tls", socket_create);
+    }
+#endif
+    return true;
+}
+
+bool sw_disable_coroutine_hook()
+{
+    if (!hook_init)
+    {
+        return false;
+    }
+    if (hook_flags & SW_HOOK_FILE)
+    {
+        memcpy((void*) &php_plain_files_wrapper, &ori_php_plain_files_wrapper, sizeof(php_plain_files_wrapper));
+    }
+    if (hook_flags & SW_HOOK_SLEEP)
+    {
+        if (ori_sleep)
+        {
+            ori_sleep->internal_function.handler = ori_sleep_handler;
+        }
+        if (ori_usleep)
+        {
+            ori_usleep->internal_function.handler = ori_usleep_handler;
+        }
+        if (ori_time_nanosleep)
+        {
+            ori_time_nanosleep->internal_function.handler = ori_time_nanosleep_handler;
+        }
+        if (ori_time_sleep_until)
+        {
+            ori_time_sleep_until->internal_function.handler = ori_time_sleep_until_handler;
+        }
+    }
+    if (hook_flags & SW_HOOK_BLOCKING_FUNCTION)
+    {
+        if (ori_gethostbyname)
+        {
+            ori_gethostbyname->internal_function.handler = ori_gethostbyname_handler;
+        }
+    }
+    if (hook_flags & SW_HOOK_TCP)
+    {
+        php_stream_xport_register("tcp", ori_factory.tcp);
+    }
+    if (hook_flags & SW_HOOK_UNIX)
+    {
+        php_stream_xport_register("unix", ori_factory._unix);
+    }
+    if (hook_flags & SW_HOOK_UDP)
+    {
+        php_stream_xport_register("udp", ori_factory.udp);
+    }
+    if (hook_flags & SW_HOOK_UDG)
+    {
+        php_stream_xport_register("udg", ori_factory.udg);
+    }
+#ifdef SW_USE_OPENSSL
+    if (hook_flags & SW_HOOK_SSL)
+    {
+        php_stream_xport_register("ssl", ori_factory.ssl);
+    }
+    if (hook_flags & SW_HOOK_TLS)
+    {
+        php_stream_xport_register("tls", ori_factory.tls);
+    }
+#endif
+    return true;
+}
+
 static PHP_METHOD(swoole_runtime, enableCoroutine)
 {
     zend_bool enable = 1;
@@ -889,86 +1092,7 @@ static PHP_METHOD(swoole_runtime, enableCoroutine)
         {
             RETURN_FALSE;
         }
-        hook_flags = flags;
-        hook_init = true;
-        HashTable *xport_hash = php_stream_xport_get_hash();
-
-        if (flags & SW_HOOK_FILE)
-        {
-            memcpy((void*) &ori_php_plain_files_wrapper, &php_plain_files_wrapper, sizeof(php_plain_files_wrapper));
-            memcpy((void*) &php_plain_files_wrapper, &sw_php_plain_files_wrapper, sizeof(php_plain_files_wrapper));
-        }
-        if (flags & SW_HOOK_SLEEP)
-        {
-            ori_sleep = (zend_function *) zend_hash_str_find_ptr(EG(function_table), ZEND_STRL("sleep"));
-            if (ori_sleep)
-            {
-                ori_sleep_handler =  ori_sleep->internal_function.handler;
-                ori_sleep->internal_function.handler = PHP_FN(_sleep);
-            }
-            ori_usleep = (zend_function *) zend_hash_str_find_ptr(EG(function_table), ZEND_STRL("usleep"));
-            if (ori_usleep)
-            {
-                ori_usleep_handler =  ori_usleep->internal_function.handler;
-                ori_usleep->internal_function.handler = PHP_FN(_usleep);
-            }
-            ori_time_nanosleep = (zend_function *) zend_hash_str_find_ptr(EG(function_table), ZEND_STRL("time_nanosleep"));
-            if (ori_time_nanosleep)
-            {
-                ori_time_nanosleep_handler =  ori_time_nanosleep->internal_function.handler;
-                ori_time_nanosleep->internal_function.handler = PHP_FN(_time_nanosleep);
-            }
-            ori_time_sleep_until = (zend_function *) zend_hash_str_find_ptr(EG(function_table), ZEND_STRL("time_sleep_until"));
-            if (ori_time_sleep_until)
-            {
-                ori_time_sleep_until_handler =  ori_time_sleep_until->internal_function.handler;
-                ori_time_sleep_until->internal_function.handler = PHP_FN(_time_sleep_until);
-            }
-        }
-        if (flags & SW_HOOK_BLOCKING_FUNCTION)
-        {
-            /**
-             * gethostbyname
-             */
-            ori_gethostbyname = (zend_function *) zend_hash_str_find_ptr(EG(function_table), ZEND_STRL("gethostbyname"));
-            if (ori_time_sleep_until)
-            {
-                ori_gethostbyname_handler =  ori_gethostbyname->internal_function.handler;
-                ori_gethostbyname->internal_function.handler = PHP_FN(swoole_coroutine_gethostbyname);
-            }
-        }
-        if (flags & SW_HOOK_TCP)
-        {
-            ori_factory.tcp = (php_stream_transport_factory) zend_hash_str_find_ptr(xport_hash, ZEND_STRL("tcp"));
-            php_stream_xport_register("tcp", socket_create);
-        }
-        if (flags & SW_HOOK_UNIX)
-        {
-            ori_factory._unix = (php_stream_transport_factory) zend_hash_str_find_ptr(xport_hash, ZEND_STRL("unix"));
-            php_stream_xport_register("unix", socket_create);
-        }
-        if (flags & SW_HOOK_UDG)
-        {
-            ori_factory._unix = (php_stream_transport_factory) zend_hash_str_find_ptr(xport_hash, ZEND_STRL("udg"));
-            php_stream_xport_register("udg", socket_create);
-        }
-        if (flags & SW_HOOK_UDP)
-        {
-            ori_factory._unix = (php_stream_transport_factory) zend_hash_str_find_ptr(xport_hash, ZEND_STRL("udp"));
-            php_stream_xport_register("udp", socket_create);
-        }
-#ifdef SW_USE_OPENSSL
-        if (flags & SW_HOOK_SSL)
-        {
-            ori_factory.ssl = (php_stream_transport_factory) zend_hash_str_find_ptr(xport_hash, ZEND_STRL("ssl"));
-            php_stream_xport_register("ssl", socket_create);
-        }
-        if (flags & SW_HOOK_TLS)
-        {
-            ori_factory.tls = (php_stream_transport_factory) zend_hash_str_find_ptr(xport_hash, ZEND_STRL("tls"));
-            php_stream_xport_register("tls", socket_create);
-        }
-#endif
+        sw_enable_coroutine_hook(flags);
     }
     else
     {
@@ -976,59 +1100,7 @@ static PHP_METHOD(swoole_runtime, enableCoroutine)
         {
             RETURN_FALSE;
         }
-        if (hook_flags & SW_HOOK_FILE)
-        {
-            memcpy((void*) &php_plain_files_wrapper, &ori_php_plain_files_wrapper, sizeof(php_plain_files_wrapper));
-        }
-        if (hook_flags & SW_HOOK_SLEEP)
-        {
-            ori_sleep = (zend_function *) zend_hash_str_find_ptr(EG(function_table), ZEND_STRL("sleep"));
-            if (ori_sleep)
-            {
-                ori_sleep->internal_function.handler = ori_sleep_handler;
-            }
-            ori_usleep = (zend_function *) zend_hash_str_find_ptr(EG(function_table), ZEND_STRL("usleep"));
-            if (ori_usleep)
-            {
-                ori_usleep->internal_function.handler = ori_usleep_handler;
-            }
-            ori_time_nanosleep = (zend_function *) zend_hash_str_find_ptr(EG(function_table), ZEND_STRL("time_nanosleep"));
-            if (ori_time_nanosleep)
-            {
-                ori_time_nanosleep->internal_function.handler = ori_time_nanosleep_handler;
-            }
-            ori_time_sleep_until = (zend_function *) zend_hash_str_find_ptr(EG(function_table), ZEND_STRL("time_sleep_until"));
-            if (ori_time_sleep_until)
-            {
-                ori_time_sleep_until->internal_function.handler = ori_time_sleep_until_handler;
-            }
-        }
-        if (flags & SW_HOOK_TCP)
-        {
-            php_stream_xport_register("tcp", ori_factory.tcp);
-        }
-        if (flags & SW_HOOK_UNIX)
-        {
-            php_stream_xport_register("unix", ori_factory._unix);
-        }
-        if (flags & SW_HOOK_UDP)
-        {
-            php_stream_xport_register("udp", ori_factory.udp);
-        }
-        if (flags & SW_HOOK_UDG)
-        {
-            php_stream_xport_register("udg", ori_factory.udg);
-        }
-#ifdef SW_USE_OPENSSL
-        if (flags & SW_HOOK_SSL)
-        {
-            php_stream_xport_register("ssl", ori_factory.ssl);
-        }
-        if (flags & SW_HOOK_TLS)
-        {
-            php_stream_xport_register("tls", ori_factory.tls);
-        }
-#endif
+        sw_disable_coroutine_hook();
     }
 }
 
