@@ -49,8 +49,9 @@ static const zend_function_entry swoole_serialize_methods[] = {
     PHP_FE_END
 };
 
-zend_class_entry swoole_serialize_ce;
-zend_class_entry *swoole_serialize_class_entry_ptr;
+static zend_class_entry swoole_serialize_ce;
+zend_class_entry *swoole_serialize_ce_ptr;
+static zend_object_handlers swoole_serialize_handlers;
 
 #define SWOOLE_SERI_EOF "EOF"
 #define CHECK_STEP if(buffer>unseri_buffer_end){ php_error_docref(NULL, E_ERROR, "illegal unserialize data"); return NULL;}
@@ -60,9 +61,10 @@ void *unseri_buffer_end = NULL;
 
 void swoole_serialize_init(int module_number)
 {
-    SWOOLE_INIT_CLASS_ENTRY(swoole_serialize_ce, "swoole_serialize", "Swoole\\Serialize", swoole_serialize_methods);
-    swoole_serialize_class_entry_ptr = zend_register_internal_class(&swoole_serialize_ce);
-    SWOOLE_CLASS_ALIAS(swoole_serialize, "Swoole\\Serialize");
+    SWOOLE_INIT_CLASS_ENTRY(swoole_serialize, "Swoole\\Serialize", "swoole_serialize", NULL, swoole_serialize_methods);
+    SWOOLE_SET_CLASS_SERIALIZABLE(swoole_serialize, zend_class_serialize_deny, zend_class_unserialize_deny);
+    SWOOLE_SET_CLASS_CLONEABLE(swoole_serialize, zend_class_clone_deny);
+    SWOOLE_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_serialize, zend_class_unset_property_deny);
 
     //    ZVAL_STRING(&swSeriaG.sleep_fname, "__sleep");
     zend_string *zstr_sleep = zend_string_init("__sleep", sizeof ("__sleep") - 1, 1);
@@ -1130,7 +1132,7 @@ static void swoole_serialize_object(seriaString *buffer, zval *obj, size_t start
         zval retval;
         if (call_user_function_ex(NULL, obj, &swSeriaG.sleep_fname, &retval, 0, 0, 1, NULL) == SUCCESS)
         {
-            if (EG(exception))
+            if (UNEXPECTED(EG(exception)))
             {
                 zval_dtor(&retval);
                 return;
@@ -1556,6 +1558,8 @@ static PHP_METHOD(swoole_serialize, pack)
     zval *zvalue;
     size_t is_fast = 0;
 
+    swoole_php_fatal_error(E_DEPRECATED, "swoole serialize will be removed, you should be using the php serialize instead.");
+
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "z|l", &zvalue, &is_fast) == FAILURE)
     {
         RETURN_FALSE;
@@ -1573,6 +1577,8 @@ static PHP_METHOD(swoole_serialize, unpack)
     zval *args = NULL; //for object
     long flag = 0;
 
+    swoole_php_fatal_error(E_DEPRECATED, "swoole serialize will be removed, you should be using the php serialize instead.");
+
     if (zend_parse_parameters(ZEND_NUM_ARGS(), "s|la", &buffer, &arg_len, &flag, &args) == FAILURE)
     {
         RETURN_FALSE;
