@@ -26,15 +26,15 @@
 using namespace swoole;
 
 static zend_class_entry swoole_socket_coro_ce;
-static zend_class_entry *swoole_socket_coro_class_entry_ptr;
+static zend_class_entry *swoole_socket_coro_ce_ptr;
 static zend_object_handlers swoole_socket_coro_handlers;
 
 static zend_class_entry swoole_socket_coro_exception_ce;
-static zend_class_entry *swoole_socket_coro_exception_class_entry_ptr;
+static zend_class_entry *swoole_socket_coro_exception_ce_ptr;
+static zend_object_handlers swoole_socket_coro_exception_handlers;
 
 typedef struct
 {
-    zval object;
     Socket *socket;
     int domain;
 #ifdef SWOOLE_SOCKETS_SUPPORT
@@ -128,22 +128,35 @@ static const zend_function_entry swoole_socket_coro_methods[] =
     PHP_FE_END
 };
 
-static inline socket_coro * sw_socket_coro_fetch_object(zend_object *obj)
+static sw_inline socket_coro * swoole_socket_coro_fetch_object(zend_object *obj)
 {
-    return (socket_coro *) ((char *) obj - XtOffsetOf(socket_coro, std));
+    return (socket_coro *) ((char *) obj - swoole_socket_coro_handlers.offset);
 }
 
-#define Z_SOCKET_CORO_OBJ_P(zv) sw_socket_coro_fetch_object(Z_OBJ_P(zv))
-
-
-static void swoole_socket_coro_free_storage(zend_object *object)
+static sw_inline socket_coro * swoole_get_socket_coro(zval *zobject)
 {
-    socket_coro *sock = (socket_coro *) sw_socket_coro_fetch_object(object);
+    socket_coro *sock = swoole_socket_coro_fetch_object(Z_OBJ_P(zobject));
+    if (UNEXPECTED(!sock->socket))
+    {
+        swoole_php_fatal_error(E_ERROR, "you must call Socket constructor first.");
+    }
+    return sock;
+}
+
+static void swoole_socket_coro_free_object(zend_object *object)
+{
+    socket_coro *sock = (socket_coro *) swoole_socket_coro_fetch_object(object);
+#ifdef SWOOLE_SOCKETS_SUPPORT
+    if (sock->resource)
+    {
+        sw_zval_free(sock->resource);
+    }
+#endif
     delete sock->socket;
     zend_object_std_dtor(&sock->std);
 }
 
-static zend_object *swoole_socket_coro_create(zend_class_entry *ce)
+static zend_object *swoole_socket_coro_create_object(zend_class_entry *ce)
 {
     socket_coro *sock = (socket_coro *) ecalloc(1, sizeof(socket_coro) + zend_object_properties_size(ce));
     zend_object_std_init(&sock->std, ce);
@@ -157,29 +170,15 @@ static zend_object *swoole_socket_coro_create(zend_class_entry *ce)
 
 void swoole_socket_coro_init(int module_number)
 {
-    INIT_CLASS_ENTRY(swoole_socket_coro_ce, "Swoole\\Coroutine\\Socket", swoole_socket_coro_methods);
+    SWOOLE_INIT_CLASS_ENTRY(swoole_socket_coro, "Swoole\\Coroutine\\Socket", NULL, "Co\\Socket", swoole_socket_coro_methods);
+    SWOOLE_SET_CLASS_SERIALIZABLE(swoole_socket_coro, zend_class_serialize_deny, zend_class_unserialize_deny);
+    SWOOLE_SET_CLASS_CLONEABLE(swoole_socket_coro, zend_class_clone_deny);
+    SWOOLE_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_socket_coro, zend_class_unset_property_deny);
+    SWOOLE_SET_CLASS_CUSTOM_OBJECT(swoole_socket_coro, swoole_socket_coro_create_object, swoole_socket_coro_free_object, socket_coro, std);
 
-    swoole_socket_coro_class_entry_ptr = zend_register_internal_class(&swoole_socket_coro_ce);
-    swoole_socket_coro_class_entry_ptr->ce_flags |= ZEND_ACC_FINAL;
-    swoole_socket_coro_class_entry_ptr->create_object = swoole_socket_coro_create;
-    swoole_socket_coro_class_entry_ptr->serialize = zend_class_serialize_deny;
-    swoole_socket_coro_class_entry_ptr->unserialize = zend_class_unserialize_deny;
-    zend_declare_property_long(swoole_socket_coro_class_entry_ptr, ZEND_STRL("errCode"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_socket_coro_ce_ptr, ZEND_STRL("errCode"), 0, ZEND_ACC_PUBLIC);
 
-    memcpy(&swoole_socket_coro_handlers, zend_get_std_object_handlers(), sizeof(swoole_socket_coro_handlers));
-    swoole_socket_coro_handlers.free_obj = swoole_socket_coro_free_storage;
-    swoole_socket_coro_handlers.clone_obj = NULL;
-    swoole_socket_coro_handlers.offset = XtOffsetOf(socket_coro, std);
-
-    INIT_CLASS_ENTRY(swoole_socket_coro_exception_ce, "Swoole\\Coroutine\\Socket\\Exception", NULL);
-    swoole_socket_coro_exception_class_entry_ptr = sw_zend_register_internal_class_ex(&swoole_socket_coro_exception_ce,
-            zend_exception_get_default(), NULL);
-
-    if (SWOOLE_G(use_shortname))
-    {
-        sw_zend_register_class_alias("Co\\Socket", swoole_socket_coro_class_entry_ptr);
-        sw_zend_register_class_alias("Co\\Socket\\Exception", swoole_socket_coro_exception_class_entry_ptr);
-    }
+    SWOOLE_INIT_CLASS_ENTRY_EX(swoole_socket_coro_exception, "Swoole\\Coroutine\\Socket\\Exception", NULL, "Co\\Socket\\Exception", NULL, swoole_exception);
 }
 
 static PHP_METHOD(swoole_socket_coro, __construct)
@@ -194,19 +193,19 @@ static PHP_METHOD(swoole_socket_coro, __construct)
 
     php_swoole_check_reactor();
 
-    socket_coro *sock = (socket_coro *) Z_SOCKET_CORO_OBJ_P(getThis()) ;
+    socket_coro *sock = (socket_coro *) swoole_socket_coro_fetch_object(Z_OBJ_P(getThis()));
     sock->socket = new Socket(get_socket_type(domain, type, protocol));
     if (unlikely(sock->socket->socket == nullptr))
     {
         delete sock->socket;
         zend_throw_exception_ex(
-            swoole_socket_coro_exception_class_entry_ptr, errno,
+            swoole_socket_coro_exception_ce_ptr, errno,
             "new Socket() failed. Error: %s [%d]", strerror(errno), errno
         );
         RETURN_FALSE;
     }
     sock->domain = domain;
-    sock->object = *getThis();
+    sock->socket->set_timeout(COROG.socket_timeout);
 }
 
 static PHP_METHOD(swoole_socket_coro, bind)
@@ -221,10 +220,10 @@ static PHP_METHOD(swoole_socket_coro, bind)
         Z_PARAM_LONG(port)
     ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
-    socket_coro *sock = (socket_coro *) Z_SOCKET_CORO_OBJ_P(getThis());
+    socket_coro *sock = swoole_get_socket_coro(getThis());
     if (!sock->socket->bind(std::string(address, l_address), port))
     {
-        zend_update_property_long(swoole_socket_coro_class_entry_ptr, getThis(), ZEND_STRL("errCode"), sock->socket->errCode);
+        zend_update_property_long(swoole_socket_coro_ce_ptr, getThis(), ZEND_STRL("errCode"), sock->socket->errCode);
         RETURN_FALSE;
     }
     RETURN_TRUE;
@@ -239,10 +238,10 @@ static PHP_METHOD(swoole_socket_coro, listen)
         Z_PARAM_LONG(backlog)
     ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
-    socket_coro *sock = (socket_coro *) Z_SOCKET_CORO_OBJ_P(getThis());
+    socket_coro *sock = swoole_get_socket_coro(getThis());
     if (!sock->socket->listen(backlog))
     {
-        zend_update_property_long(swoole_socket_coro_class_entry_ptr, getThis(), ZEND_STRL("errCode"), sock->socket->errCode);
+        zend_update_property_long(swoole_socket_coro_ce_ptr, getThis(), ZEND_STRL("errCode"), sock->socket->errCode);
         RETURN_FALSE;
     }
     RETURN_TRUE;
@@ -250,29 +249,27 @@ static PHP_METHOD(swoole_socket_coro, listen)
 
 static PHP_METHOD(swoole_socket_coro, accept)
 {
-    double timeout = -1;
+    double timeout = COROG.socket_timeout;
 
     ZEND_PARSE_PARAMETERS_START(0, 1)
         Z_PARAM_OPTIONAL
         Z_PARAM_DOUBLE(timeout)
     ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
-    socket_coro *sock = (socket_coro *) Z_SOCKET_CORO_OBJ_P(getThis());
-    if (timeout != 0)
-    {
-        sock->socket->set_timeout(timeout);
-    }
+    socket_coro *sock = swoole_get_socket_coro(getThis());
+
+    sock->socket->set_timeout(timeout, true);
     Socket *conn = sock->socket->accept();
     if (conn)
     {
-        zend_object *client = swoole_socket_coro_create(swoole_socket_coro_class_entry_ptr);
-        socket_coro *client_sock = (socket_coro *) sw_socket_coro_fetch_object(client);
+        zend_object *client = swoole_socket_coro_create_object(swoole_socket_coro_ce_ptr);
+        socket_coro *client_sock = (socket_coro *) swoole_socket_coro_fetch_object(client);
         client_sock->socket = conn;
         ZVAL_OBJ(return_value, &client_sock->std);
     }
     else
     {
-        zend_update_property_long(swoole_socket_coro_class_entry_ptr, getThis(), ZEND_STRL("errCode"), sock->socket->errCode);
+        zend_update_property_long(swoole_socket_coro_ce_ptr, getThis(), ZEND_STRL("errCode"), sock->socket->errCode);
         RETURN_FALSE;
     }
 }
@@ -282,7 +279,7 @@ static PHP_METHOD(swoole_socket_coro, recv)
     coro_check();
 
     zend_long length = SW_BUFFER_SIZE_BIG;
-    double timeout = -1;
+    double timeout = COROG.socket_timeout;
 
     ZEND_PARSE_PARAMETERS_START(0, 2)
         Z_PARAM_OPTIONAL
@@ -295,14 +292,14 @@ static PHP_METHOD(swoole_socket_coro, recv)
         length = SW_BUFFER_SIZE_BIG;
     }
 
-    socket_coro *sock = (socket_coro *) Z_SOCKET_CORO_OBJ_P(getThis());
-    sock->socket->set_timeout(timeout);
+    socket_coro *sock = swoole_get_socket_coro(getThis());
 
+    sock->socket->set_timeout(timeout, true);
     zend_string *buf = zend_string_alloc(length, 0);
     ssize_t bytes = sock->socket->recv(ZSTR_VAL(buf), length);
     if (bytes < 0)
     {
-        zend_update_property_long(swoole_socket_coro_class_entry_ptr, getThis(), ZEND_STRL("errCode"), sock->socket->errCode);
+        zend_update_property_long(swoole_socket_coro_ce_ptr, getThis(), ZEND_STRL("errCode"), sock->socket->errCode);
         zend_string_free(buf);
         RETURN_FALSE;
     }
@@ -322,7 +319,7 @@ static PHP_METHOD(swoole_socket_coro, recv)
 static PHP_METHOD(swoole_socket_coro, recvfrom)
 {
     zval *peername;
-    double timeout = -1;
+    double timeout = COROG.socket_timeout;
 
     ZEND_PARSE_PARAMETERS_START(1, 2)
         Z_PARAM_ZVAL_EX(peername, 0, 1)
@@ -330,14 +327,14 @@ static PHP_METHOD(swoole_socket_coro, recvfrom)
         Z_PARAM_DOUBLE(timeout)
     ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
-    socket_coro *sock = (socket_coro *) Z_SOCKET_CORO_OBJ_P(getThis());
-    sock->socket->set_timeout(timeout);
+    socket_coro *sock = swoole_get_socket_coro(getThis());
 
     zend_string *buf = zend_string_alloc(SW_BUFFER_SIZE_BIG, 0);
+    sock->socket->set_timeout(timeout, true);
     ssize_t bytes = sock->socket->recvfrom(ZSTR_VAL(buf), SW_BUFFER_SIZE_BIG);
     if (bytes < 0)
     {
-        zend_update_property_long(swoole_socket_coro_class_entry_ptr, getThis(), ZEND_STRL("errCode"), sock->socket->errCode);
+        zend_update_property_long(swoole_socket_coro_ce_ptr, getThis(), ZEND_STRL("errCode"), sock->socket->errCode);
         zend_string_free(buf);
         RETURN_FALSE;
     }
@@ -373,7 +370,7 @@ static PHP_METHOD(swoole_socket_coro, recvfrom)
 
 static PHP_METHOD(swoole_socket_coro, send)
 {
-    double timeout = -1;
+    double timeout = COROG.socket_timeout;
     char *data;
     size_t l_data;
 
@@ -383,11 +380,13 @@ static PHP_METHOD(swoole_socket_coro, send)
         Z_PARAM_DOUBLE(timeout)
     ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
-    socket_coro *sock = (socket_coro *) Z_SOCKET_CORO_OBJ_P(getThis());
+    socket_coro *sock = swoole_get_socket_coro(getThis());
+
+    sock->socket->set_timeout(timeout, true);
     ssize_t retval = sock->socket->send(data, l_data);
     if (retval < 0)
     {
-        zend_update_property_long(swoole_socket_coro_class_entry_ptr, getThis(), ZEND_STRL("errCode"), sock->socket->errCode);
+        zend_update_property_long(swoole_socket_coro_ce_ptr, getThis(), ZEND_STRL("errCode"), sock->socket->errCode);
         RETURN_FALSE;
     }
     else
@@ -410,11 +409,11 @@ static PHP_METHOD(swoole_socket_coro, sendto)
         Z_PARAM_STRING(data, l_data)
     ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
-    socket_coro *sock = (socket_coro *) Z_SOCKET_CORO_OBJ_P(getThis());
+    socket_coro *sock = swoole_get_socket_coro(getThis());
     ssize_t retval = sock->socket->sendto(addr, port, data, l_data);
     if (retval < 0)
     {
-        zend_update_property_long(swoole_socket_coro_class_entry_ptr, getThis(), ZEND_STRL("errCode"), sock->socket->errCode);
+        zend_update_property_long(swoole_socket_coro_ce_ptr, getThis(), ZEND_STRL("errCode"), sock->socket->errCode);
         RETURN_FALSE;
     }
     else
@@ -427,11 +426,11 @@ static PHP_METHOD(swoole_socket_coro, close)
 {
     coro_check();
 
-    socket_coro *sock = (socket_coro *) Z_SOCKET_CORO_OBJ_P(getThis());
+    socket_coro *sock = swoole_get_socket_coro(getThis());
 
     if (!sock->socket->close())
     {
-        zend_update_property_long(swoole_socket_coro_class_entry_ptr, getThis(), ZEND_STRL("errCode"), sock->socket->errCode);
+        zend_update_property_long(swoole_socket_coro_ce_ptr, getThis(), ZEND_STRL("errCode"), sock->socket->errCode);
         RETURN_FALSE;
     }
     else
@@ -442,16 +441,17 @@ static PHP_METHOD(swoole_socket_coro, close)
 
 static PHP_METHOD(swoole_socket_coro, getsockname)
 {
-    socket_coro *sock = (socket_coro *) Z_SOCKET_CORO_OBJ_P(getThis());
+    socket_coro *sock = swoole_get_socket_coro(getThis());
     array_init(return_value);
 
     swSocketAddress info;
     memset(&info, 0, sizeof(info));
+    info.len = sizeof(info.addr);
     char addr_str[INET6_ADDRSTRLEN + 1];
 
-    if (getsockname(sock->socket->get_fd(), (struct sockaddr *) &info.addr.inet_v4, &info.len) != 0)
+    if (getsockname(sock->socket->get_fd(), (struct sockaddr *) &info.addr, &info.len) != 0)
     {
-        zend_update_property_long(swoole_socket_coro_class_entry_ptr, getThis(), ZEND_STRL("errCode"), errno);
+        zend_update_property_long(swoole_socket_coro_ce_ptr, getThis(), ZEND_STRL("errCode"), errno);
         RETURN_FALSE;
     }
 
@@ -478,7 +478,7 @@ static PHP_METHOD(swoole_socket_coro, getsockname)
 
 static PHP_METHOD(swoole_socket_coro, getpeername)
 {
-    socket_coro *sock = (socket_coro *) Z_SOCKET_CORO_OBJ_P(getThis());
+    socket_coro *sock = swoole_get_socket_coro(getThis());
     array_init(return_value);
 
     swSocketAddress info;
@@ -487,7 +487,7 @@ static PHP_METHOD(swoole_socket_coro, getpeername)
 
     if (getpeername(sock->socket->get_fd(), (struct sockaddr *) &info.addr, &info.len) != 0)
     {
-        zend_update_property_long(swoole_socket_coro_class_entry_ptr, getThis(), ZEND_STRL("errCode"), errno);
+        zend_update_property_long(swoole_socket_coro_ce_ptr, getThis(), ZEND_STRL("errCode"), errno);
         RETURN_FALSE;
     }
 
@@ -516,11 +516,11 @@ static PHP_METHOD(swoole_socket_coro, connect)
 {
     coro_check();
 
-    socket_coro *sock = (socket_coro *) Z_SOCKET_CORO_OBJ_P(getThis());
+    socket_coro *sock = swoole_get_socket_coro(getThis());
     char *host;
     size_t l_host;
     zend_long port = 0;
-    double timeout = SW_CLIENT_DEFAULT_TIMEOUT;
+    double timeout = COROG.socket_connect_timeout;
 
     ZEND_PARSE_PARAMETERS_START(1, 3)
         Z_PARAM_STRING(host, l_host)
@@ -542,7 +542,8 @@ static PHP_METHOD(swoole_socket_coro, connect)
             RETURN_FALSE;
         }
     }
-    sock->socket->set_timeout(timeout);
+
+    sock->socket->set_timeout(timeout, true);
     if (sock->socket->connect(std::string(host, l_host), port))
     {
         RETURN_TRUE;
@@ -556,7 +557,7 @@ static PHP_METHOD(swoole_socket_coro, connect)
 #ifdef SWOOLE_SOCKETS_SUPPORT
 static PHP_METHOD(swoole_socket_coro, getSocket)
 {
-    socket_coro *sock = (socket_coro *) Z_SOCKET_CORO_OBJ_P(getThis());
+    socket_coro *sock = swoole_get_socket_coro(getThis());
     php_socket *socket_object = swoole_convert_to_socket(sock->socket->get_fd());
     if (!socket_object)
     {
