@@ -28,13 +28,14 @@ enum swoole_timer_type
 
 typedef struct _swTimer_callback
 {
+    int type;
+    int interval;
     zval* callback;
+    zend_fcall_info_cache *fci_cache;
     zval* data;
     zval _callback;
+    zend_fcall_info_cache _fci_cache;
     zval _data;
-    zend_fcall_info_cache *func_cache;
-    int interval;
-    int type;
 } swTimer_callback;
 
 static int php_swoole_del_timer(swTimer_node *tnode);
@@ -63,59 +64,41 @@ void php_swoole_clear_all_timer()
     }
 }
 
-long php_swoole_add_timer(int ms, zval *callback, zval *param, int persistent)
+long php_swoole_add_timer(long ms, zval *callback, zval *param, int persistent)
 {
-    if (ms > SW_TIMER_MAX_VALUE)
-    {
-        swoole_php_fatal_error(E_WARNING, "The given parameters is too big.");
-        return SW_ERR;
-    }
     if (ms <= 0)
     {
         swoole_php_fatal_error(E_WARNING, "Timer must be greater than 0");
         return SW_ERR;
     }
 
-    char *func_name = NULL;
-    zend_fcall_info_cache *func_cache = (zend_fcall_info_cache *) emalloc(sizeof(zend_fcall_info_cache));
-    if (!sw_zend_is_callable_ex(callback, NULL, 0, &func_name, NULL, func_cache, NULL))
-    {
-        swoole_php_fatal_error(E_ERROR, "function '%s' is not callable", func_name);
-        efree(func_cache);
-        efree(func_name);
-        return SW_ERR;
-    }
-    efree(func_name);
-
     if (!(SwooleG.serv && swIsTaskWorker() && SwooleG.serv->task_async == 0))
     {
         php_swoole_check_reactor();
     }
 
     swTimer_callback *cb = (swTimer_callback *) emalloc(sizeof(swTimer_callback));
+    char *func_name = NULL;
+    if (!sw_zend_is_callable_ex(callback, NULL, 0, &func_name, NULL, &cb->_fci_cache, NULL))
+    {
+        swoole_php_fatal_error(E_ERROR, "function '%s' is not callable", func_name);
+        return SW_ERR;
+    }
+    efree(func_name);
 
-    cb->data = &cb->_data;
+    cb->_callback = *callback;
     cb->callback = &cb->_callback;
-    memcpy(cb->callback, callback, sizeof(zval));
+    cb->fci_cache = &cb->_fci_cache;
     if (param)
     {
-        memcpy(cb->data, param, sizeof(zval));
+        cb->_data = *param;
+        cb->data = &cb->_data;
     }
     else
     {
         cb->data = NULL;
     }
 
-
-    if (SwooleG.enable_coroutine)
-    {
-        cb->func_cache = func_cache;
-    }
-    else
-    {
-        efree(func_cache);
-    }
-
     swTimerCallback timer_func;
     if (persistent)
     {
@@ -162,10 +145,6 @@ static int php_swoole_del_timer(swTimer_node *tnode)
     {
         zval_ptr_dtor(cb->data);
     }
-    if (SwooleG.enable_coroutine && cb->func_cache)
-    {
-        efree(cb->func_cache);
-    }
     efree(cb);
     return SW_OK;
 }
@@ -173,8 +152,6 @@ static int php_swoole_del_timer(swTimer_node *tnode)
 void php_swoole_onTimeout(swTimer *timer, swTimer_node *tnode)
 {
     swTimer_callback *cb = (swTimer_callback *) tnode->data;
-    zval *retval = NULL;
-
     zval args[1];
     int argc = 0;
 
@@ -186,36 +163,26 @@ void php_swoole_onTimeout(swTimer *timer, swTimer_node *tnode)
 
     if (SwooleG.enable_coroutine)
     {
-        long ret = sw_coro_create(cb->func_cache, argc, args, retval);
-        if (ret < 0)
+        if (sw_coro_create(cb->fci_cache, argc, args) < 0)
         {
-            swoole_php_fatal_error(E_WARNING, "swoole timer onTimeout failed, create coroutine failed.");
-            return;
+            swoole_php_fatal_error(E_WARNING, "create onTimer coroutine error.");
         }
     }
     else
     {
-        if (sw_call_user_function_ex(EG(function_table), NULL, cb->callback, &retval, argc, args, 0, NULL) == FAILURE)
+        zval _retval, *retval = &_retval;
+        if (sw_call_user_function_fast_ex(NULL, cb->fci_cache, retval, argc, args) == FAILURE)
         {
-            swoole_php_fatal_error(E_WARNING, "swoole_timer: onTimeout handler error");
-            return;
+            swoole_php_fatal_error(E_WARNING, "onTimeout handler error.");
         }
-    }
-
-    if (EG(exception))
-    {
-        zend_exception_error(EG(exception), E_ERROR);
-    }
-    if (retval)
-    {
         zval_ptr_dtor(retval);
     }
+
     php_swoole_del_timer(tnode);
 }
 
 void php_swoole_onInterval(swTimer *timer, swTimer_node *tnode)
 {
-    zval *retval = NULL;
     zval *ztimer_id;
     swTimer_callback *cb = (swTimer_callback *) tnode->data;
 
@@ -234,31 +201,21 @@ void php_swoole_onInterval(swTimer *timer, swTimer_node *tnode)
 
     if (SwooleG.enable_coroutine)
     {
-        long ret = sw_coro_create(cb->func_cache, argc, args, retval);
-        if (ret < 0)
+        if (sw_coro_create(cb->fci_cache, argc, args) < 0)
         {
-            swoole_php_fatal_error(E_WARNING, "swoole timer onInterval failed, create coroutine failed.");
+            swoole_php_fatal_error(E_WARNING, "create onInterval coroutine error.");
             return;
         }
     }
     else
     {
-        if (sw_call_user_function_ex(EG(function_table), NULL, cb->callback, &retval, argc, args, 0, NULL) == FAILURE)
+        zval _retval, *retval = &_retval;
+        if (sw_call_user_function_fast_ex(NULL, cb->fci_cache, retval, argc, args) == FAILURE)
         {
-            swoole_php_fatal_error(E_WARNING, "swoole timer onInterval handler error.");
-            return;
+            swoole_php_fatal_error(E_WARNING, "onInterval handler error.");
         }
-    }
-
-    if (EG(exception))
-    {
-        zend_exception_error(EG(exception), E_ERROR);
-    }
-    if (retval != NULL)
-    {
         zval_ptr_dtor(retval);
     }
-    zval_ptr_dtor(ztimer_id);
 
     if (tnode->remove)
     {
