@@ -20,12 +20,12 @@
 
 extern "C"
 {
-#include <ext/standard/url.h>
-#include <ext/standard/sha1.h>
-#include <ext/standard/php_var.h>
-#include <ext/standard/php_string.h>
-#include <ext/date/php_date.h>
-#include <main/php_variables.h>
+#include "ext/standard/url.h"
+#include "ext/standard/sha1.h"
+#include "ext/standard/php_var.h"
+#include "ext/standard/php_string.h"
+#include "ext/date/php_date.h"
+#include "main/php_variables.h"
 }
 
 #include "websocket.h"
@@ -34,13 +34,16 @@ extern "C"
 #include "thirdparty/swoole_http_parser.h"
 
 static zend_class_entry swoole_websocket_server_ce;
-zend_class_entry *swoole_websocket_server_class_entry_ptr;
+zend_class_entry *swoole_websocket_server_ce_ptr;
+static zend_object_handlers swoole_websocket_server_handlers;
 
 static zend_class_entry swoole_websocket_frame_ce;
-zend_class_entry *swoole_websocket_frame_class_entry_ptr;
+zend_class_entry *swoole_websocket_frame_ce_ptr;
+static zend_object_handlers swoole_websocket_frame_handlers;
 
-static zend_class_entry swoole_websocket_close_frame_ce;
-static zend_class_entry *swoole_websocket_close_frame_class_entry_ptr;
+static zend_class_entry swoole_websocket_closeframe_ce;
+static zend_class_entry *swoole_websocket_closeframe_ce_ptr;
+static zend_object_handlers swoole_websocket_closeframe_handlers;
 
 static int websocket_handshake(swListenPort *, http_context *);
 
@@ -111,19 +114,19 @@ static void php_swoole_websocket_construct_frame(zval *zframe, zend_long opcode,
 {
     if (opcode == WEBSOCKET_OPCODE_CLOSE)
     {
-        object_init_ex(zframe, swoole_websocket_close_frame_class_entry_ptr);
+        object_init_ex(zframe, swoole_websocket_closeframe_ce_ptr);
         if (payload_length >= SW_WEBSOCKET_CLOSE_CODE_LEN)
         {
             // WebSocket Close code
             zend_update_property_long(
-                swoole_websocket_close_frame_class_entry_ptr, zframe, ZEND_STRL("code"),
+                swoole_websocket_closeframe_ce_ptr, zframe, ZEND_STRL("code"),
                 (payload[0] << 8) ^ (payload[1] & 0xFF)
             );
             if (payload_length > SW_WEBSOCKET_CLOSE_CODE_LEN)
             {
                 // WebSocket Close reason message
                 zend_update_property_stringl(
-                    swoole_websocket_close_frame_class_entry_ptr, zframe, ZEND_STRL("reason"),
+                    swoole_websocket_closeframe_ce_ptr, zframe, ZEND_STRL("reason"),
                     payload + SW_WEBSOCKET_CLOSE_CODE_LEN, payload_length - SW_WEBSOCKET_CLOSE_CODE_LEN
                 );
             }
@@ -131,11 +134,11 @@ static void php_swoole_websocket_construct_frame(zval *zframe, zend_long opcode,
     }
     else
     {
-        object_init_ex(zframe, swoole_websocket_frame_class_entry_ptr);
-        zend_update_property_stringl(swoole_websocket_frame_class_entry_ptr, zframe, ZEND_STRL("data"), payload, payload_length);
+        object_init_ex(zframe, swoole_websocket_frame_ce_ptr);
+        zend_update_property_stringl(swoole_websocket_frame_ce_ptr, zframe, ZEND_STRL("data"), payload, payload_length);
     }
-    zend_update_property_bool(swoole_websocket_frame_class_entry_ptr, zframe, ZEND_STRL("finish"), finish);
-    zend_update_property_long(swoole_websocket_frame_class_entry_ptr, zframe, ZEND_STRL("opcode"), opcode);
+    zend_update_property_bool(swoole_websocket_frame_ce_ptr, zframe, ZEND_STRL("finish"), finish);
+    zend_update_property_long(swoole_websocket_frame_ce_ptr, zframe, ZEND_STRL("opcode"), opcode);
 }
 
 void php_swoole_websocket_frame_unpack(swString *data, zval *zframe)
@@ -157,38 +160,38 @@ int php_swoole_websocket_frame_pack(swString *buffer, zval *zdata, zend_bool opc
     char *data = NULL;
     size_t length = 0;
     zend_long code = WEBSOCKET_CLOSE_NORMAL;
-    if (Z_TYPE_P(zdata) == IS_OBJECT && instanceof_function(Z_OBJCE_P(zdata), swoole_websocket_frame_class_entry_ptr))
+    if (Z_TYPE_P(zdata) == IS_OBJECT && instanceof_function(Z_OBJCE_P(zdata), swoole_websocket_frame_ce_ptr))
     {
         zval *zframe = zdata;
         zval *ztmp = NULL;
         zdata = NULL;
-        if ((ztmp = sw_zend_read_property(swoole_websocket_frame_class_entry_ptr, zframe, ZEND_STRL("opcode"), 1)))
+        if ((ztmp = sw_zend_read_property(swoole_websocket_frame_ce_ptr, zframe, ZEND_STRL("opcode"), 1)))
         {
             convert_to_long(ztmp);
             opcode = Z_LVAL_P(ztmp);
         }
         if (opcode == WEBSOCKET_OPCODE_CLOSE)
         {
-            if ((ztmp = sw_zend_read_property_not_null(swoole_websocket_frame_class_entry_ptr, zframe, ZEND_STRL("code"), 1)))
+            if ((ztmp = sw_zend_read_property_not_null(swoole_websocket_frame_ce_ptr, zframe, ZEND_STRL("code"), 1)))
             {
                 convert_to_long(ztmp);
                 code = Z_LVAL_P(ztmp);
             }
-            if ((ztmp = sw_zend_read_property_not_null(swoole_websocket_frame_class_entry_ptr, zframe, ZEND_STRL("reason"), 1)))
+            if ((ztmp = sw_zend_read_property_not_null(swoole_websocket_frame_ce_ptr, zframe, ZEND_STRL("reason"), 1)))
             {
                 zdata = ztmp;
             }
         }
-        if (!zdata && (ztmp = sw_zend_read_property(swoole_websocket_frame_class_entry_ptr, zframe, ZEND_STRL("data"), 1)))
+        if (!zdata && (ztmp = sw_zend_read_property(swoole_websocket_frame_ce_ptr, zframe, ZEND_STRL("data"), 1)))
         {
             zdata = ztmp;
         }
-        if ((ztmp = sw_zend_read_property(swoole_websocket_frame_class_entry_ptr, zframe, ZEND_STRL("finish"), 1)))
+        if ((ztmp = sw_zend_read_property(swoole_websocket_frame_ce_ptr, zframe, ZEND_STRL("finish"), 1)))
         {
             convert_to_boolean(ztmp);
             fin = Z_BVAL_P(ztmp);
         }
-        if ((ztmp = sw_zend_read_property(swoole_websocket_frame_class_entry_ptr, zframe, ZEND_STRL("mask"), 1)))
+        if ((ztmp = sw_zend_read_property(swoole_websocket_frame_ce_ptr, zframe, ZEND_STRL("mask"), 1)))
         {
             convert_to_boolean(ztmp);
             mask = Z_BVAL_P(ztmp);
@@ -215,55 +218,47 @@ int php_swoole_websocket_frame_pack(swString *buffer, zval *zdata, zend_bool opc
     return SW_OK;
 }
 
-
-void swoole_websocket_onOpen(http_context *ctx)
+void swoole_websocket_onOpen(swServer *serv, http_context *ctx)
 {
     int fd = ctx->fd;
 
-    swConnection *conn = swWorker_get_connection(SwooleG.serv, fd);
+    swConnection *conn = swWorker_get_connection(serv, fd);
     if (!conn)
     {
         swoole_error_log(SW_LOG_NOTICE, SW_ERROR_SESSION_CLOSED, "session[%d] is closed.", fd);
         return;
     }
 
-    zend_fcall_info_cache *fci_cache = php_swoole_server_get_fci_cache(SwooleG.serv, conn->from_fd, SW_SERVER_CB_onOpen);
-    if (fci_cache)
+    zend_fcall_info_cache *fci_cache = php_swoole_server_get_fci_cache(serv, conn->from_fd, SW_SERVER_CB_onOpen);
+    if (!fci_cache)
     {
-        swServer *serv = SwooleG.serv;
-        zval *zserv = (zval *) serv->ptr2;
-        zval *zrequest_object = ctx->request.zobject;
-        zval *retval = NULL;
+        return;
+    }
 
-        zval args[2];
-        args[0] = *zserv;
-        args[1] = *zrequest_object;
+    zval *zserv = (zval *) serv->ptr2;
+    zval *zrequest_object = ctx->request.zobject;
 
-        if (SwooleG.enable_coroutine)
-        {
-            long ret = sw_coro_create(fci_cache, 2, args, retval);
-            if (ret < 0)
-            {
-                SwooleG.serv->factory.end(&SwooleG.serv->factory, fd);
-                return;
-            }
-        }
-        else
-        {
-            if (sw_call_user_function_fast_ex(NULL, fci_cache, &retval, 2, args) == FAILURE)
-            {
-                swoole_php_error(E_WARNING, "onOpen handler error");
-            }
-        }
+    zval args[2];
+    args[0] = *zserv;
+    args[1] = *zrequest_object;
 
-        if (EG(exception))
+    if (SwooleG.enable_coroutine)
+    {
+        if (sw_coro_create(fci_cache, 2, args) < 0)
         {
-            zend_exception_error(EG(exception), E_ERROR);
+            swoole_php_error(E_WARNING, "create onOpen coroutine error.");
+            serv->close(serv, fd, 0);
+            return;
         }
-        if (retval)
+    }
+    else
+    {
+        zval _retval, *retval = &_retval;
+        if (sw_call_user_function_fast_ex(NULL, fci_cache, retval, 2, args) == FAILURE)
         {
-            zval_ptr_dtor(retval);
+            swoole_php_error(E_WARNING, "onOpen handler error.");
         }
+        zval_ptr_dtor(retval);
     }
 }
 
@@ -295,7 +290,7 @@ void php_swoole_sha1(const char *str, int _len, unsigned char *digest)
     PHP_SHA1Final(digest, &context);
 }
 
-static int websocket_handshake(swListenPort *port, http_context *ctx)
+static int websocket_handshake(swServer *serv, swListenPort *port, http_context *ctx)
 {
     zval *header = ctx->request.zheader;
     HashTable *ht = Z_ARRVAL_P(header);
@@ -339,17 +334,17 @@ static int websocket_handshake(swListenPort *port, http_context *ctx)
 
     swTrace("websocket header len:%ld\n%s \n", swoole_http_buffer->length, swoole_http_buffer->str);
 
-    swConnection *conn = swWorker_get_connection(SwooleG.serv, ctx->fd);
+    swConnection *conn = swWorker_get_connection(serv, ctx->fd);
     if (!conn)
     {
         swoole_error_log(SW_LOG_NOTICE, SW_ERROR_SESSION_CLOSED, "session[%d] is closed.", ctx->fd);
         return SW_ERR;
     }
     conn->websocket_status = WEBSOCKET_STATUS_ACTIVE;
-    return swServer_tcp_send(SwooleG.serv, ctx->fd, swoole_http_buffer->str, swoole_http_buffer->length);
+    return serv->send(serv, ctx->fd, swoole_http_buffer->str, swoole_http_buffer->length);
 }
 
-int swoole_websocket_onMessage(swEventData *req)
+int swoole_websocket_onMessage(swServer *serv, swEventData *req)
 {
     int fd = req->info.fd;
     zend_bool finish = 0;
@@ -377,60 +372,48 @@ int swoole_websocket_onMessage(swEventData *req)
     zval *zframe;
     SW_MAKE_STD_ZVAL(zframe);
     php_swoole_websocket_construct_frame(zframe, opcode, Z_STRVAL_P(zdata), Z_STRLEN_P(zdata), finish);
-    zend_update_property_long(swoole_websocket_frame_class_entry_ptr, zframe, ZEND_STRL("fd"), fd);
-
-    swServer *serv = SwooleG.serv;
-    zval *zserv = (zval *) serv->ptr2;
-    zval *retval = NULL;
+    zend_update_property_long(swoole_websocket_frame_ce_ptr, zframe, ZEND_STRL("fd"), fd);
 
     zend_fcall_info_cache *fci_cache = php_swoole_server_get_fci_cache(serv, req->info.from_fd, SW_SERVER_CB_onMessage);
     zval args[2];
-    args[0] = *zserv;
+    args[0] = *(zval *) serv->ptr2; // zserver
     args[1] = *zframe;
 
     if (SwooleG.enable_coroutine)
     {
-        long ret = sw_coro_create(fci_cache, 2, args, retval);
-        if (ret < 0)
+        if (sw_coro_create(fci_cache, 2, args) < 0)
         {
-            zval_ptr_dtor(zdata);
-            zval_ptr_dtor(zframe);
+            swoole_php_error(E_WARNING, "create onMessage coroutine error.");
             SwooleG.serv->factory.end(&SwooleG.serv->factory, fd);
-            return SW_OK;
         }
     }
     else
     {
-        if (sw_call_user_function_fast_ex(NULL, fci_cache, &retval, 2, args) == FAILURE)
+        zval _retval, *retval = &_retval;
+        if (sw_call_user_function_fast_ex(NULL, fci_cache, retval, 2, args) == FAILURE)
         {
-            swoole_php_error(E_WARNING, "onMessage handler error");
+            swoole_php_error(E_WARNING, "onMessage handler error.");
         }
-    }
-
-    if (EG(exception))
-    {
-        zend_exception_error(EG(exception), E_ERROR);
-    }
-    if (retval)
-    {
         zval_ptr_dtor(retval);
     }
+
     zval_ptr_dtor(zdata);
     zval_ptr_dtor(zframe);
+
     return SW_OK;
 }
 
-int swoole_websocket_onHandshake(swListenPort *port, http_context *ctx)
+int swoole_websocket_onHandshake(swServer *serv, swListenPort *port, http_context *ctx)
 {
     int fd = ctx->fd;
-    int ret = websocket_handshake(port, ctx);
+    int ret = websocket_handshake(serv, port, ctx);
     if (ret == SW_ERR)
     {
-        swServer_tcp_close(SwooleG.serv, fd, 1);
+        swServer_tcp_close(serv, fd, 1);
     }
     else
     {
-        swoole_websocket_onOpen(ctx);
+        swoole_websocket_onOpen(serv, ctx);
     }
 
     //free client data
@@ -444,31 +427,22 @@ int swoole_websocket_onHandshake(swListenPort *port, http_context *ctx)
 
 void swoole_websocket_init(int module_number)
 {
-    SWOOLE_INIT_CLASS_ENTRY(swoole_websocket_server_ce, "swoole_websocket_server", "Swoole\\WebSocket\\Server", swoole_websocket_server_methods);
-    swoole_websocket_server_class_entry_ptr = sw_zend_register_internal_class_ex(&swoole_websocket_server_ce, swoole_http_server_class_entry_ptr, "swoole_http_server");
-    SWOOLE_CLASS_ALIAS(swoole_websocket_server, "Swoole\\WebSocket\\Server");
+    SWOOLE_INIT_CLASS_ENTRY_EX(swoole_websocket_server, "Swoole\\WebSocket\\Server", "swoole_websocket_server", NULL, swoole_websocket_server_methods, swoole_http_server);
+    SWOOLE_SET_CLASS_SERIALIZABLE(swoole_websocket_server, zend_class_serialize_deny, zend_class_unserialize_deny);
+    SWOOLE_SET_CLASS_CLONEABLE(swoole_websocket_server, zend_class_clone_deny);
+    SWOOLE_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_websocket_server, zend_class_unset_property_deny);
 
-    SWOOLE_INIT_CLASS_ENTRY(swoole_websocket_frame_ce, "swoole_websocket_frame", "Swoole\\WebSocket\\Frame", swoole_websocket_frame_methods);
-    swoole_websocket_frame_class_entry_ptr = zend_register_internal_class(&swoole_websocket_frame_ce);
-    SWOOLE_CLASS_ALIAS(swoole_websocket_frame, "Swoole\\WebSocket\\Frame");
-    zend_declare_property_long(swoole_websocket_frame_class_entry_ptr, ZEND_STRL("fd"), 0, ZEND_ACC_PUBLIC);
-    zend_declare_property_string(swoole_websocket_frame_class_entry_ptr, ZEND_STRL("data"), "", ZEND_ACC_PUBLIC);
-    zend_declare_property_long(swoole_websocket_frame_class_entry_ptr, ZEND_STRL("opcode"), WEBSOCKET_OPCODE_TEXT, ZEND_ACC_PUBLIC);
-    zend_declare_property_bool(swoole_websocket_frame_class_entry_ptr, ZEND_STRL("finish"), 1, ZEND_ACC_PUBLIC);
+    SWOOLE_INIT_CLASS_ENTRY(swoole_websocket_frame, "Swoole\\WebSocket\\Frame", "swoole_websocket_frame", NULL, swoole_websocket_frame_methods);
+    SWOOLE_INIT_CLASS_ENTRY_EX(swoole_websocket_closeframe, "Swoole\\WebSocket\\CloseFrame", "swoole_websocket_closeframe", NULL, NULL, swoole_websocket_frame);
 
-    SWOOLE_INIT_CLASS_ENTRY(swoole_websocket_close_frame_ce, "swoole_websocke_closet_frame", "Swoole\\WebSocket\\CloseFrame", NULL);
-    swoole_websocket_close_frame_class_entry_ptr = zend_register_internal_class_ex(&swoole_websocket_close_frame_ce, swoole_websocket_frame_class_entry_ptr);
-    SWOOLE_CLASS_ALIAS(swoole_websocket_close_frame, "Swoole\\WebSocket\\CloseFrame");
-    zend_declare_property_long(swoole_websocket_close_frame_class_entry_ptr, ZEND_STRL("opcode"), WEBSOCKET_OPCODE_CLOSE, ZEND_ACC_PUBLIC);
-    zend_declare_property_long(swoole_websocket_close_frame_class_entry_ptr, ZEND_STRL("code"), WEBSOCKET_CLOSE_NORMAL, ZEND_ACC_PUBLIC);
-    zend_declare_property_string(swoole_websocket_close_frame_class_entry_ptr, ZEND_STRL("reason"), "", ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_websocket_frame_ce_ptr, ZEND_STRL("fd"), 0, ZEND_ACC_PUBLIC);
+    zend_declare_property_string(swoole_websocket_frame_ce_ptr, ZEND_STRL("data"), "", ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_websocket_frame_ce_ptr, ZEND_STRL("opcode"), WEBSOCKET_OPCODE_TEXT, ZEND_ACC_PUBLIC);
+    zend_declare_property_bool(swoole_websocket_frame_ce_ptr, ZEND_STRL("finish"), 1, ZEND_ACC_PUBLIC);
 
-    if (SWOOLE_G(use_shortname))
-    {
-        sw_zend_register_class_alias("Co\\WebSocket\\Server", swoole_websocket_server_class_entry_ptr);
-        sw_zend_register_class_alias("Co\\WebSocket\\Frame", swoole_websocket_frame_class_entry_ptr);
-        sw_zend_register_class_alias("Co\\WebSocket\\CloseFrame", swoole_websocket_close_frame_class_entry_ptr);
-    }
+    zend_declare_property_long(swoole_websocket_closeframe_ce_ptr, ZEND_STRL("opcode"), WEBSOCKET_OPCODE_CLOSE, ZEND_ACC_PUBLIC);
+    zend_declare_property_long(swoole_websocket_closeframe_ce_ptr, ZEND_STRL("code"), WEBSOCKET_CLOSE_NORMAL, ZEND_ACC_PUBLIC);
+    zend_declare_property_string(swoole_websocket_closeframe_ce_ptr, ZEND_STRL("reason"), "", ZEND_ACC_PUBLIC);
 
     // status
     SWOOLE_RAW_DEFINE(WEBSOCKET_STATUS_CONNECTION);
@@ -498,7 +472,7 @@ void swoole_websocket_init(int module_number)
     SWOOLE_RAW_DEFINE(WEBSOCKET_CLOSE_TLS);
 }
 
-static sw_inline int swoole_websocket_server_push(int fd, swString *buffer)
+static sw_inline int swoole_websocket_server_push(swServer *serv, int fd, swString *buffer)
 {
     if (unlikely(fd <= 0))
     {
@@ -506,7 +480,7 @@ static sw_inline int swoole_websocket_server_push(int fd, swString *buffer)
         return SW_ERR;
     }
 
-    swConnection *conn = swWorker_get_connection(SwooleG.serv, fd);
+    swConnection *conn = swWorker_get_connection(serv, fd);
     if (!conn || conn->websocket_status < WEBSOCKET_STATUS_HANDSHAKE)
     {
         SwooleG.error = SW_ERROR_WEBSOCKET_UNCONNECTED;
@@ -515,7 +489,6 @@ static sw_inline int swoole_websocket_server_push(int fd, swString *buffer)
     }
 
     int ret = swServer_tcp_send(SwooleG.serv, fd, buffer->str, buffer->length);
-    swServer *serv = SwooleG.serv;
     if (ret < 0 && SwooleG.error == SW_ERROR_OUTPUT_BUFFER_OVERFLOW && serv->send_yield)
     {
         zval _return_value;
@@ -529,20 +502,20 @@ static sw_inline int swoole_websocket_server_push(int fd, swString *buffer)
     return ret;
 }
 
-static sw_inline int swoole_websocket_server_close(int fd, swString *buffer, uint8_t real_close)
+static sw_inline int swoole_websocket_server_close(swServer *serv, int fd, swString *buffer, uint8_t real_close)
 {
-    int ret = swoole_websocket_server_push(fd, buffer);
+    int ret = swoole_websocket_server_push(serv, fd, buffer);
     if (ret < 0 || !real_close)
     {
         return ret;
     }
-    swConnection *conn = swWorker_get_connection(SwooleG.serv, fd);
+    swConnection *conn = swWorker_get_connection(serv, fd);
     if (conn)
     {
         // Change status immediately to avoid double close
         conn->websocket_status = WEBSOCKET_STATUS_CLOSING;
         // Server close connection immediately
-        return SwooleG.serv->close(SwooleG.serv, fd, SW_FALSE);
+        return serv->close(serv, fd, SW_FALSE);
     }
     else
     {
@@ -566,7 +539,8 @@ static PHP_METHOD(swoole_websocket_server, disconnect)
     {
         RETURN_FALSE;
     }
-    SW_CHECK_RETURN(swoole_websocket_server_close(fd, swoole_http_buffer, 1));
+    swServer *serv = (swServer *) swoole_get_object(getThis());
+    SW_CHECK_RETURN(swoole_websocket_server_close(serv, fd, swoole_http_buffer, 1));
 }
 
 static PHP_METHOD(swoole_websocket_server, push)
@@ -586,13 +560,15 @@ static PHP_METHOD(swoole_websocket_server, push)
     {
         RETURN_FALSE;
     }
+
+    swServer *serv = (swServer *) swoole_get_object(getThis());
     switch (opcode)
     {
     case WEBSOCKET_OPCODE_CLOSE:
-        SW_CHECK_RETURN(swoole_websocket_server_close(fd, swoole_http_buffer, fin));
+        SW_CHECK_RETURN(swoole_websocket_server_close(serv, fd, swoole_http_buffer, fin));
         break;
     default:
-        SW_CHECK_RETURN(swoole_websocket_server_push(fd, swoole_http_buffer));
+        SW_CHECK_RETURN(swoole_websocket_server_push(serv, fd, swoole_http_buffer));
     }
 }
 
