@@ -9,6 +9,14 @@ define('HAS_SSL', defined("SWOOLE_SSL"));
 define('HAS_ASYNC_REDIS', class_exists("swoole_redis", false));
 define('HAS_HTTP2', class_exists("swoole_http2_request", false));
 
+/** ============== Files ============== */
+define('SOURCE_ROOT_PATH', __DIR__ . '/../../');
+define('TRAVIS_DIR_PATH', __DIR__ . '/../../travis/');
+define('TEST_IMAGE', __DIR__ . '/../../examples/test.jpg');
+define('TEST_LOG_FILE', '/tmp/swoole.log');
+define('TEST_PID_FILE', '/tmp/swoole.pid');
+define('SSL_FILE_DIR', __DIR__ . '/api/swoole_http_server/localhost-ssl');
+
 /** ============ Servers ============ */
 define('SERVER_MODE_RANDOM', array_random([SWOOLE_BASE, SWOOLE_PROCESS]));
 
@@ -33,7 +41,9 @@ define('MYSQL_SERVER_PWD', 'root');
 define('MYSQL_SERVER_DB', 'test');
 
 /** ============== Redis ============== */
-define('REDIS_SERVER_PATH', "");
+define('REDIS_SERVER_PATH',
+    IS_IN_TRAVIS ? TRAVIS_DIR_PATH . '/data/redis/redis.sock' : (IS_MAC_OS ? '/tmp/redis.sock' : '/var/run/redis/redis-server.sock')
+);
 define('REDIS_SERVER_HOST', IS_IN_TRAVIS ? 'redis' : '127.0.0.1');
 define('REDIS_SERVER_PORT', 6379);
 
@@ -46,12 +56,6 @@ define('HTTP_PROXY_PORT', IS_MAC_OS ? 1087 : 8888);
 define('SOCKS5_PROXY_HOST', '127.0.0.1');
 define('SOCKS5_PROXY_PORT', IS_MAC_OS ? 1086 : 1080);
 
-/** ============== Files ============== */
-define('SOURCE_ROOT_PATH', __DIR__ . '/../../');
-define('TEST_IMAGE', __DIR__ . '/../../examples/test.jpg');
-define('TEST_LOG_FILE', '/tmp/swoole.log');
-define('SSL_FILE_DIR', __DIR__ . '/api/swoole_http_server/localhost-ssl');
-
 /** ============== Times ============== */
 define('PRESSURE_LEVEL', USE_VALGRIND ? 1 : IS_IN_TRAVIS ? 2 : 3);
 define('MAX_CONCURRENCY', [16, 32, 64, 256][PRESSURE_LEVEL]);
