@@ -19,7 +19,7 @@ phpize > /dev/null && \
 > /dev/null && \
 make clean > /dev/null && \
 make > /dev/null | tee /tmp/compile.log && \
-(test "`cat /tmp/compile.log`" = "" || exit 255) && \
+(test "`cat /tmp/compile.log`"x = ""x || exit 255) && \
 make install && \
 docker-php-ext-enable swoole && \
 echo "swoole.fast_serialize=On" >> /usr/local/etc/php/conf.d/docker-php-ext-swoole-serialize.ini
